@@ -11,7 +11,7 @@ sub title
     print "$line\n";
   }
 
-$translator_file = "$ENV{srcdir}/translators.txt";
+$translator_file = "translators.txt";
 
 # List of the people who wrote the package
 title "Authors of this package.";
@@ -22,14 +22,6 @@ EOC
 
 print "\n";
 
-# List of the people who maintain the package
-title "Interim Maintainer";
-print <<'EOC';
- * Masayuki Hatta <mhatta@gnu.org>
-EOC
-
-print "\n";
-
 # List of the people who helped the translation
 title "Translators";
 open (STDIN, $translator_file)
@@ -64,17 +56,14 @@ while (<>)
     $file =~ s/\.ssh//;
     $names =~ s/.*"(.*)".*/$1/g;
 
-    foreach $name (split (',', $names))
-      {
-	# We don't want to see demailles
-	next
-	  if ($name =~ /Akim Demaille/);
-	# Avoid extraneous white spaces
-	$name =~ s/[ \t]+/ /g;
-	$name =~ s/^ //;
-	$name =~ s/ $//;
-	$authors{$name} .= ($authors{$name} ? ', ' : '') . $file;
-      }
+    # We don't want to see demailles
+    $names =~ s/, Akim Demaille <[^>]*>//g;
+    $names =~ s/Akim Demaille <[^>]*>, //g;
+    $names =~ s/Akim Demaille <[^>]*>//g;
+
+    next
+      unless ($names);
+    $authors{$names} .= ($authors{$names} ? ', ' : '') . $file;
   }
 
 foreach $name (sort (keys (%authors)))
@@ -102,13 +91,8 @@ print "\n";
 # People who package the package
 title "Packagers / Ports";
 print <<'EOC';
+ * Dave Whitinger (RPM) <dave@whitinger.net>
  * Alexander Mai (OS/2 port) <mai@migdal.ikp.physik.tu-darmstadt.de>
- * Ansgar Duelmer (DOS) <Ansgar.Duelmer@anorg.chemie.uni-giessen.de>
- * Chuck Robey (FreeBSD) <chuckr@mat.net>
- * Dirk Eddelbuettel (formerly Debian) <edd@debian.org>
- * Masayuki Hatta (Debian) <mhatta@debian.org>
- * Michael Williams (Mac OS X) <mike@dreamboatrecords.co.uk>
- * Paulo Matos (RPM) <pjsm@students.fct.unl.pt>
 EOC
 
 print "\n";
