@@ -74,8 +74,8 @@ a2ps_common_finalize (struct a2ps_common_s * common, const char * home)
   char buf [512];
 
   /* Add the user's home .a2ps dir to the lib path unless explicitly
-   * discarded with NO_HOMECONF */
-  if (! getenv ("NO_HOMECONF"))
+   * discarded with NO_HOME_CONF */
+  if (! getenv ("NO_HOME_CONF"))
     {
       sprintf (buf, "%s%c.a2ps", home, DIR_SEP);
       common->path = pw_prepend_string_to_path (common->path, 
