@@ -10,7 +10,7 @@
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -94,10 +94,10 @@
   ")url", "}",		\
   "@itemize",		"@itemize @minus",	\
   NULL
-
 
+
 /************************************************************************/
-/*	The authors list handling					*/
+/*      The authors list handling                                       */
 /************************************************************************/
 /*
  * In the following, we do not want to see Akim Demaille's name
@@ -111,11 +111,11 @@
  */
 static inline void
 authors_print (const uchar * authors, FILE * stream,
-	       const char * before,
-	       const char * author_fmt, const char * between,
-	       const char * after)
+	       const char *before,
+	       const char *author_fmt, const char *between,
+	       const char *after)
 {
-  char * cp, * author, * email;
+  char *cp, *author, *email;
   boolean first = true;
 
   if (!authors)
@@ -124,25 +124,28 @@ authors_print (const uchar * authors, FILE * stream,
   /* Work on a copy */
   astrcpy (cp, authors);
   cp = strtok (cp, ",");
-  
-  while (cp) {
-    author = cp;
-    email = author + strcspn (author, "<");
-    *(email - 1) = '\0';
-    email++;
-    *(email + strcspn (email, ">")) = '\0';
-    *(email - 1) = '\0';
-    if (!strequ (author, "Akim Demaille")) {
-      if (first) {
-	fputs (before, stream);
-	first = false;
-      } else {
-	fputs (between, stream);
-      }
-      fprintf (stream, author_fmt, author, email);
+
+  while (cp)
+    {
+      author = cp;
+      email = author + strcspn (author, "<");
+      *(email - 1) = '\0';
+      email++;
+      *(email + strcspn (email, ">")) = '\0';
+      *(email - 1) = '\0';
+      if (!strequ (author, "Akim Demaille"))
+	{
+	  if (first)
+	    {
+	      fputs (before, stream);
+	      first = false;
+	    }
+	  else
+	    fputs (between, stream);
+	  fprintf (stream, author_fmt, author, email);
+	}
+      cp = strtok (NULL, ",");
     }
-    cp = strtok (NULL, ",");
-  }
   if (!first)
     fputs (after, stream);
 }
@@ -152,7 +155,7 @@ authors_print (const uchar * authors, FILE * stream,
  */
 void
 authors_print_plain (const uchar * authors, FILE * stream,
-		     const char * before)
+		     const char *before)
 {
   authors_print (authors, stream,
 		 before, "%s <%s>", ", ", ".\n");
@@ -163,7 +166,7 @@ authors_print_plain (const uchar * authors, FILE * stream,
  */
 void
 authors_print_html (const uchar * authors, FILE * stream,
-		    const char * before)
+		    const char *before)
 {
   authors_print (authors, stream,
 		 before,
@@ -176,57 +179,62 @@ authors_print_html (const uchar * authors, FILE * stream,
  */
 void
 authors_print_texinfo (const uchar * authors, FILE * stream,
-		       const char * before)
+		       const char *before)
 {
-  uchar * cp;
+  uchar *cp;
   /* We must quote the @ of the emails */
   cp = (uchar *) xvstrrpl ((const char *) authors,
 			   AUTHORS_TO_TEXINFO);
+
+  /* Don't print the email, that makes too wide output. */
   authors_print (cp, stream,
-		 before, "@email{%2$s, %1$s}", ", ", ".\n");
+		 before, "%s", ", ", ".\n");
   FREE (cp);
 }
 /************************************************************************/
-/*	The documentation handling					*/
+/*      The documentation handling                                      */
 /************************************************************************/
 /* 1. Plain ASCII */
 void
-documentation_print_plain (const uchar * documentation, 
-			   const char * format, FILE * stream)
+documentation_print_plain (const uchar * documentation,
+			   const char *format, FILE * stream)
 {
-  char * cp;
+  char *cp;
 
-  if (documentation) {
-    cp = xvstrrpl ((const char *) documentation, DOC_TO_PLAIN);
-    fprintf (stream, format, cp);
-    FREE (cp);
-  }
+  if (!documentation)
+    return;
+
+  cp = xvstrrpl ((const char *) documentation, DOC_TO_PLAIN);
+  fprintf (stream, format, cp);
+  FREE (cp);
 }
 
 /* 2. Towards HTML */
 void
-documentation_print_html (const uchar * documentation, 
-			  const char * format, FILE * stream)
+documentation_print_html (const uchar * documentation,
+			  const char *format, FILE * stream)
 {
-  char * cp;
+  char *cp;
 
-  if (documentation) {
-    cp = xvstrrpl ((const char *) documentation, DOC_TO_HTML);
-    fprintf (stream, format, cp);
-    FREE (cp);
-  }
+  if (!documentation)
+    return;
+
+  cp = xvstrrpl ((const char *) documentation, DOC_TO_HTML);
+  fprintf (stream, format, cp);
+  FREE (cp);
 }
 
 /* 3. Towards Texinfo */
 void
-documentation_print_texinfo (const uchar * documentation, 
-			     const char * format, FILE * stream)
+documentation_print_texinfo (const uchar * documentation,
+			     const char *format, FILE * stream)
 {
-  char * cp;
+  char *cp;
 
-  if (documentation) {
-    cp = xvstrrpl ((const char *) documentation, DOC_TO_TEXINFO);
-    fprintf (stream, format, cp);
-    FREE (cp);
-  }
+  if (!documentation)
+    return;
+
+  cp = xvstrrpl ((const char *) documentation, DOC_TO_TEXINFO);
+  fprintf (stream, format, cp);
+  FREE (cp);
 }
