@@ -2,14 +2,14 @@
  * encodings.c
  *
  * definition of the char encodings used
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  * $Id: encoding.c,v 1.82 1998/03/04 17:27:00 demaille Exp $
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -28,7 +28,7 @@
 #include "encoding.h"
 #include "routines.h"
 #include "jobs.h"
-#include "msg.h"
+#include "message.h"
 #include "pathwalk.h"
 #include "pair_ht.h"
 #include "fonts.h"
@@ -36,6 +36,8 @@
 #include "dsc.h"
 #include "getshline.h"
 #include "document.h"
+#include "quotearg.h"
+#include "strverscmp.h"
 
 /* Suffix of the Encoding Description Files */
 #define EDF_SUFFIX ".edf"
@@ -112,7 +114,7 @@
   FontPlain setfont		%% Here is the difference
   Show
   >>
-  
+
   because it is the most open solution.  Especially if the prologue
   makes use of some other faces (in the headers for instance).
 */
@@ -126,10 +128,10 @@
   and finds tags that define the charset to use.  It could read 8859-1,
   or ISO-8859-1 or iso-8859-1 though the user would like latin1.
 
-  The scheme, of course, should be much more general than just skipping 
+  The scheme, of course, should be much more general than just skipping
   ISO|iso-8859, since it is even _wrong_ for, say, latin5 (8859-9)
 
-  From an implementation point of view, it is nothing but a double 
+  From an implementation point of view, it is nothing but a double
   strings hash table (cf. pair_ht.[ch]).
  */
 struct pair_htable *
@@ -174,7 +176,7 @@ load_main_encodings_map (struct a2ps_job * job)
   /* System's */
   file = xpw_find_file (job->common.path, "encoding.map", NULL);
   load_encodings_map (job, file);
-  if (message_test (msg_enc))
+  if (msg_test (msg_enc))
     {
       fprintf (stderr, "Read encoding.map:\n");
       pair_table_list_long (job->encodings_map, stderr);
@@ -205,9 +207,9 @@ wx_self_print (unsigned int * wx, FILE * stream)
 
   if (wx)
     for (j = 0 ; j < 256 ; j += 8)
-      fprintf (stream, 
+      fprintf (stream,
 	       "%3d: %04u %04u %04u %04u %04u %04u %04u %04u\n", j,
-	       wx [j + 0], wx [j + 1], wx [j + 2], wx [j + 3], 
+	       wx [j + 0], wx [j + 1], wx [j + 2], wx [j + 3],
 	       wx [j + 4], wx [j + 5], wx [j + 6], wx [j + 7]);
   else
     fputs ("<No WX defined>\n", stream);
@@ -218,7 +220,7 @@ wx_self_print (unsigned int * wx, FILE * stream)
 /*		     fonts entries handling				*/
 /************************************************************************/
 /*
-  Font entries are hashed, but they will be used only at 
+  Font entries are hashed, but they will be used only at
   initialization: the first thing to do will be to associate
   the font's to the face which uses this font.
 
@@ -292,8 +294,8 @@ font_entry_free (struct font_entry * item)
 static void
 font_entry_self_print (struct font_entry * entry, FILE * stream)
 {
-  fprintf (stream, "***** %s ***** (%s, %s) ", 
-	   entry->key, 
+  fprintf (stream, "***** %s ***** (%s, %s) ",
+	   entry->key,
 	   entry->used ? "used" : "not used",
 	   entry->reencode ? "to share between encodings" : "reencode");
   wx_self_print (entry->wx, stream);
@@ -302,32 +304,32 @@ font_entry_self_print (struct font_entry * entry, FILE * stream)
 /************************************************************************/
 /*		     fonts entries hash table handling			*/
 /************************************************************************/
-static struct hash_table *
+static struct hash_table_s *
 font_table_new (void)
 {
-  struct hash_table * res;
-  
+  struct hash_table_s * res;
+
   res = ALLOC (hash_table, 1);
   hash_init (res, 32,
-	     (hash_func_t) font_entry_hash_1, 
-	     (hash_func_t) font_entry_hash_2, 
+	     (hash_func_t) font_entry_hash_1,
+	     (hash_func_t) font_entry_hash_2,
 	     (hash_cmp_func_t) font_entry_hash_cmp);
   return res;
 }
 
 static void
-font_table_free (struct hash_table * table)
+font_table_free (struct hash_table_s * table)
 {
   hash_free (table, (hash_map_func_t) font_entry_free);
   FREE (table);
 }
 
 static void
-font_table_self_print (struct hash_table * table, FILE * stream)
+font_table_self_print (struct hash_table_s * table, FILE * stream)
 {
   int i;
   struct font_entry ** entries;
-  entries = (struct font_entry **) 
+  entries = (struct font_entry **)
     hash_dump (table, NULL,
 	       (hash_cmp_func_t) font_entry_hash_qcmp);
 
@@ -341,14 +343,14 @@ font_table_self_print (struct hash_table * table, FILE * stream)
  * We make a copy of the name, but not of the array
  */
 static inline void
-font_entry_add (struct hash_table * table,
+font_entry_add (struct hash_table_s * table,
 		const char * name, unsigned int * wx)
 {
   hash_insert (table, font_entry_new (name, wx));
 }
 
 static inline struct font_entry *
-font_entry_get (struct hash_table * table, const char * name)
+font_entry_get (struct hash_table_s * table, const char * name)
 {
   struct font_entry token, * item;
   token.key = (char *) name;
@@ -357,26 +359,26 @@ font_entry_get (struct hash_table * table, const char * name)
 }
 
 static unsigned int *
-font_entry_get_wx (struct hash_table * table,
+font_entry_get_wx (struct hash_table_s * table,
 		   const char * name)
 {
   return font_entry_get (table, name)->wx;
 }
 
 static int
-font_entry_exists (struct hash_table * table, const char * name)
+font_entry_exists (struct hash_table_s * table, const char * name)
 {
   return font_entry_get (table, name) != NULL;
 }
 
 static int
-font_entry_is_used (struct hash_table * table, const char * name)
+font_entry_is_used (struct hash_table_s * table, const char * name)
 {
   return font_entry_get (table, name)->used;
 }
 
 static void
-font_entry_set_used (struct hash_table * table, const char * name)
+font_entry_set_used (struct hash_table_s * table, const char * name)
 {
   font_entry_get (table, name)->used = TRUE;
 }
@@ -403,7 +405,7 @@ struct encoding {
 				/* E.g. Courier has been used, but
 				 * Actually Courier_Ogonki is the real
 				 * font used (cf infra)			*/
-  struct hash_table * fonts;	/* Contains cells that are
+  struct hash_table_s * fonts;	/* Contains cells that are
 				 * 1. name of font, 2. int wx[256] 	*/
   unsigned int * faces_wx[NB_FACES];
 };
@@ -417,7 +419,7 @@ encoding_new (const char * key)
   int i;
 
   NEW (struct encoding, res);
-  
+
   res->key = xstrdup (key);
   res->name = NULL;
   res->default_font = NULL;
@@ -446,7 +448,7 @@ encoding_free (struct encoding * encoding)
   XFREE (encoding->name);
   XFREE (encoding->default_font);
   XFREE (encoding->documentation);
-  
+
   for (i = 0 ; i < 256 ; i++)
     FREE (encoding->vector [i]);
 
@@ -511,7 +513,7 @@ encoding_resolve_font_substitute (struct a2ps_job * job,
 	error (1, 0, "Cannot find font %s, nor any substitute",
 	       font_name);
     }
-  
+
   message (msg_enc,
 	   (stderr, "In encoding %s, font %s is resolved as %s\n",
 	    encoding->key, font_name, res));
@@ -525,7 +527,8 @@ encoding_resolve_font_substitute (struct a2ps_job * job,
 #define GET_LINE_TOKEN(from) (strtok ((from), "\n"))
 #define CHECK_TOKEN()                                                   \
   if (token2 == NULL)                                                   \
-    error_at_line (1, 0, fname, firstline, _("missing argument for `%s'"), token);
+    error_at_line (1, 0, fname, firstline,				\
+		   _("missing argument for `%s'"), quotearg (token));
 
 static void
 load_encoding_description_file (a2ps_job * job,
@@ -553,7 +556,7 @@ load_encoding_description_file (a2ps_job * job,
   fname = xpw_find_file (job->common.path, encoding->key, ".edf");
   stream = xrfopen (fname);
 
-  while (getshline_numbered (&firstline, &lastline, 
+  while (getshline_numbered (&firstline, &lastline,
 			     &buf, &bufsiz, stream) != -1)
     {
       token = GET_TOKEN (buf);
@@ -567,15 +570,16 @@ load_encoding_description_file (a2ps_job * job,
 	  int c = 0;
 	  char buf2 [256];
 
-	  while (c < 256) {
-	    if (!fgets (buf2, sizeof (buf2), stream))
-	      error_at_line (1, 0, fname, firstline, 
-			     _("missing argument for `%s'"), "Vector:");;
-	    lastline++;
-	    token = GET_TOKEN (buf2);
-	    encoding->vector [c++] = xstrdup (token);
-	    while ((token2 = GET_TOKEN (NULL)))
-	      encoding->vector [c++] = xstrdup (token2);
+	  while (c < 256)
+	    {
+	      if (!fgets (buf2, sizeof (buf2), stream))
+		error_at_line (1, 0, fname, firstline,
+			       _("missing argument for `%s'"), "`Vector:'");
+	      lastline++;
+	      token = GET_TOKEN (buf2);
+	      encoding->vector [c++] = xstrdup (token);
+	      while ((token2 = GET_TOKEN (NULL)))
+		encoding->vector [c++] = xstrdup (token2);
 	  }
 	}
       else if (strequ (token, "Name:"))
@@ -609,8 +613,8 @@ load_encoding_description_file (a2ps_job * job,
 	      obstack_grow (&documentation_stack, buf2, read_length);
 	    }
 	  if (!strprefix ("EndDocumentation", buf2))
-	    error (1, 0, fname, firstline, _("missing argument for `%s'"),
-		   "Documentation");
+	    error (1, 0, fname, firstline,
+		   _("missing argument for `%s'"), "`Documentation'");
 	  /* Finish the obstack, and store in the encoding entry */
 	  obstack_1grow (&documentation_stack, '\0');
 	  documentation =
@@ -621,7 +625,7 @@ load_encoding_description_file (a2ps_job * job,
       else if (strequ (token, "Substitute:"))
 	{
 	  char * orig, * subs;
-	  
+
 	  token2 = GET_TOKEN (NULL);
 	  CHECK_TOKEN ();
 	  orig = token2;
@@ -632,7 +636,7 @@ load_encoding_description_file (a2ps_job * job,
 	}
       else
         error_at_line (1, 0, fname, firstline,
-                       _("illegal option: `%s'"), token);
+                       _("invalid option `%s'"), quotearg (token));
 
     }
   fclose (stream);
@@ -663,7 +667,7 @@ encoding_print_signature (struct encoding * item, FILE * stream)
  * Output the correct setup of ENCODING
  */
 static void
-encoding_output_ps_vector (struct a2ps_job * job, 
+encoding_output_ps_vector (struct a2ps_job * job,
 			   struct encoding * encoding)
 {
   int c;
@@ -672,11 +676,12 @@ encoding_output_ps_vector (struct a2ps_job * job,
 	  encoding->name);
   output (job->ps_encodings, "/%sEncoding [",
 	  encoding->name);
-  for (c = 0 ; c < 256 ; c++) {
-    if (!(c % 8))
-      output_char (job->ps_encodings, '\n');
-    output (job->ps_encodings, "/%s ", encoding->vector [c]);
-  }
+  for (c = 0 ; c < 256 ; c++)
+    {
+      if (!(c % 8))
+	output_char (job->ps_encodings, '\n');
+      output (job->ps_encodings, "/%s ", encoding->vector [c]);
+    }
   output (job->ps_encodings, "\n] def\n");
   output (job->ps_encodings, "%%%%EndResource\n");
 }
@@ -693,32 +698,35 @@ encoding_self_print (struct encoding * item, FILE * stream)
 
   if (! IS_EMPTY (item->default_font))
     fprintf (stream, "Default font is `%s'\n", item->default_font);
-  
+
   fprintf (stream, "Substitution of fonts:\n");
   pair_table_list_long (item->substitutes, stream);
   fprintf (stream, "Name of fonts used (before substitution):\n");
   da_self_print (item->font_names_used, stream);
 
   fprintf (stream, "Encoding array:");
-  for (c = 0 ; c < 256 ; c++) {
-    if (!(c % 8))
-      putc ('\n', stream);
-    fprintf (stream, "%-10s ", item->vector [c]);
-  }
+  for (c = 0 ; c < 256 ; c++)
+    {
+      if (!(c % 8))
+	putc ('\n', stream);
+      fprintf (stream, "%-10s ", item->vector [c]);
+    }
   putc ('\n', stream);
-  
+
   fprintf (stream, "Fonts:\n");
   font_table_self_print (item->fonts, stream);
-  if (message_test (msg_enc)) {
-    enum face_e face;
-    int i;
-    fprintf (stream, "Faces:\n");
-    for (i = 0 ; base_faces [i] != -1 ; i++) {
-      face = base_faces [i];
-      fprintf (stream, "Face %s (%d)\n", face_to_string (face), face);
-      wx_self_print (item->faces_wx [face], stream);
+  if (msg_test (msg_enc))
+    {
+      enum face_e face;
+      int i;
+      fprintf (stream, "Faces:\n");
+      for (i = 0 ; base_faces [i] != -1 ; i++)
+	{
+	  face = base_faces [i];
+	  fprintf (stream, "Face %s (%d)\n", face_to_string (face), face);
+	  wx_self_print (item->faces_wx [face], stream);
+	}
     }
-  }
 }
 
 /*
@@ -768,12 +776,13 @@ dump_encoding_setup (FILE * stream,
 
   /* We do not want to reencode the fonts that should not be
    * reencoded */
-  for (i = 0 ; i < encoding->font_names_used->len ; i++) {
-    real_font_name = encoding_resolve_font_substitute (job, encoding, 
-						       font_names [i]);
-    if (!font_is_to_reencode (job, real_font_name))
-      da_remove_at (encoding->font_names_used, i, (da_map_func_t) free);
-  }
+  for (i = 0 ; i < encoding->font_names_used->len ; i++)
+    {
+      real_font_name = encoding_resolve_font_substitute (job, encoding,
+							 font_names [i]);
+      if (!font_is_to_reencode (job, real_font_name))
+	da_remove_at (encoding->font_names_used, i, (da_map_func_t) free);
+    }
 
   /* The number of fonts that, finally, have to be encoded
    * in the current ENCODING	*/
@@ -781,12 +790,12 @@ dump_encoding_setup (FILE * stream,
 
   /* Create the dictionary and fill it */
   fprintf (stream, "%% Dictionary for %s support\n",
-	  encoding->name); 
+	  encoding->name);
   fprintf (stream, "/%sdict %d dict begin\n", encoding->key, nb);
   for (i = 0 ; i < nb ; i++)
     fprintf (stream, "  /f%s %sEncoding /%s reencode_font\n",
-	     font_names [i], 
-	     encoding->name, 
+	     font_names [i],
+	     encoding->name,
 	     encoding_resolve_font_substitute (job, encoding, font_names [i]));
   fputs ("currentdict end def\n", stream);
 }
@@ -816,7 +825,7 @@ encoding_hash_cmp (struct encoding * x, struct encoding * y)
 }
 
 static int
-encoding_hash_qcmp (struct encoding ** x, 
+encoding_hash_qcmp (struct encoding ** x,
 			  struct encoding ** y)
 {
   return_STRING_COMPARE ((*x)->key, (*y)->key);
@@ -825,15 +834,15 @@ encoding_hash_qcmp (struct encoding ** x,
 /*
  * 2. Hash table routines
  */
-struct hash_table *
+struct hash_table_s *
 encodings_table_new (void)
 {
-  struct hash_table * res;
-  
+  struct hash_table_s * res;
+
   res = ALLOC (hash_table, 1);
   hash_init (res, 32,
-	     (hash_func_t) encoding_hash_1, 
-	     (hash_func_t) encoding_hash_2, 
+	     (hash_func_t) encoding_hash_1,
+	     (hash_func_t) encoding_hash_2,
 	     (hash_cmp_func_t) encoding_hash_cmp);
   return res;
 }
@@ -842,7 +851,7 @@ encodings_table_new (void)
  * Free the table and content
  */
 void
-encodings_table_free (struct hash_table * table)
+encodings_table_free (struct hash_table_s * table)
 {
   hash_free (table, (hash_map_func_t) encoding_free);
   FREE (table);
@@ -858,19 +867,20 @@ encoding_setup (struct a2ps_job * job,
   char * cp;
 
   load_encoding_description_file (job, encoding);
-  switch (job->output_format) {
-  case ps:
-  case eps:     /* Producing PostScript */
-    /* FIXME: Remove this
-     * Include its encoding definition */
-    /* This, too, should be handled by the pseudo PDD files */
-    cp = ALLOCA (char, strlen (encoding->name) + strlen ("Encoding") + 1);
-    sprintf (cp, "%sEncoding", encoding->name);
+  switch (job->output_format)
+    {
+    case ps:
+    case eps:     /* Producing PostScript */
+      /* FIXME: Remove this
+       * Include its encoding definition */
+      /* This, too, should be handled by the pseudo PDD files */
+      cp = ALLOCA (char, strlen (encoding->name) + strlen ("Encoding") + 1);
+      sprintf (cp, "%sEncoding", encoding->name);
 
-    add_supplied_resource (job, "encoding", cp);
-    encoding_output_ps_vector (job, encoding);
-    break;
-  }
+      add_supplied_resource (job, "encoding", cp);
+      encoding_output_ps_vector (job, encoding);
+      break;
+    }
 }
 
 /*
@@ -889,7 +899,7 @@ encoding_add (struct a2ps_job * job, const char * key)
    * get the WX per face */
   if (face_eo_font_is_set (job))
     encoding_build_faces_wx (job, encoding);
-  if (message_test (msg_enc))
+  if (msg_test (msg_enc))
     encoding_self_print (encoding, stderr);
   hash_insert (job->encodings, encoding);
 }
@@ -916,20 +926,21 @@ encoding_build_faces_wx (a2ps_job * job, struct encoding * encoding)
   enum face_e face;
   const char * true_font_name;
 
-  for (i = 0 ; base_faces [i] != -1 ; i++) {
-    face = base_faces [i];
-    encoding_add_font_name_used (encoding,
-				       job->face_eo_font [face]);
-    /* E.g. in Latin 2, don't use Courier but Courier-Ogonki */
-    true_font_name = 
-      encoding_resolve_font_substitute (job, encoding, 
-					job->face_eo_font [face]);
-    /* Get the wx related to the FACE */
-    font_info_get_wx_for_vector (job,
-				 true_font_name,
-				 encoding->vector,
-				 encoding->faces_wx [face]);
-  }
+  for (i = 0 ; base_faces [i] != -1 ; i++)
+    {
+      face = base_faces [i];
+      encoding_add_font_name_used (encoding,
+				   job->face_eo_font [face]);
+      /* E.g. in Latin 2, don't use Courier but Courier-Ogonki */
+      true_font_name =
+	encoding_resolve_font_substitute (job, encoding,
+					  job->face_eo_font [face]);
+      /* Get the wx related to the FACE */
+      font_info_get_wx_for_vector (job,
+				   true_font_name,
+				   encoding->vector,
+				   encoding->faces_wx [face]);
+    }
 }
 
 /*
@@ -943,10 +954,11 @@ encoding_get (struct a2ps_job * job, const char * key)
   token.key = (char *) key;
   res = (struct encoding *) hash_find_item (job->encodings, &token);
 
-  if (!res) {	/* The encoding has never been read yet */
-    encoding_add (job, key);
-    res = (struct encoding *) hash_find_item (job->encodings, &token);
-  }
+  if (!res)
+    {	/* The encoding has never been read yet */
+      encoding_add (job, key);
+      res = (struct encoding *) hash_find_item (job->encodings, &token);
+    }
   return res;
 }
 
@@ -978,13 +990,13 @@ dump_encodings_setup (FILE * stream,
   struct encoding ** encodings;
 
   /* Get the list of the encodings */
-  encodings = (struct encoding **) 
+  encodings = (struct encoding **)
     hash_dump (job->encodings, NULL,
 	       (hash_cmp_func_t) encoding_hash_qcmp);
 
   for (i = 0 ; encodings [i] ; i++)
     dump_encoding_setup (stream, job, encodings [i]);
-  
+
   FREE (encodings);
 }
 
@@ -1004,59 +1016,60 @@ char_WX (a2ps_job * job, uchar c)
   if (wx)
     return wx;
 
-  switch (job->unprintable_format) {
-  case hexa:
+  switch (job->unprintable_format)
     {
-      char buf [3];
-      sprintf (buf, "%02x", c);
-      return _WX_ ('\\') + _WX_('x') + _WX_ (buf[0]) + _WX_ (buf[1]);
-    }
-
-  case octal:
-    {
-      char buf [4];
-      sprintf (buf, "%03o", c);
-      return (_WX_ ('\\')
-	      + _WX_ (buf [0]) + _WX_ (buf [1]) + _WX_ (buf [2]));
-    }
-
-  case space:
-    return _WX_(' ');
-  
-  case question_mark:
-    return _WX_('?');
-  
-  case caret:
-    if (0177 < c)
+    case hexa:
       {
-	wx += _WX_('M') + _WX_('-');
-	c &= 0177;
+	char buf [3];
+	sprintf (buf, "%02x", c);
+	return _WX_ ('\\') + _WX_('x') + _WX_ (buf[0]) + _WX_ (buf[1]);
       }
-    if (c == 0177)
-      wx += _WX_('^') + _WX_('?');
-    else if (c < ' ')
-      wx += _WX_ ('^') + _WX_ (c + '@');
-    else
-      wx += _WX_ (c);
-    return wx;
 
-  case Emacs:
-    if (0177 < c)
+    case octal:
       {
-	wx += _WX_('M') + _WX_('-');
-	c &= 0177;
+	char buf [4];
+	sprintf (buf, "%03o", c);
+	return (_WX_ ('\\')
+		+ _WX_ (buf [0]) + _WX_ (buf [1]) + _WX_ (buf [2]));
       }
-    if (c == 0177)
-      wx += _WX_('C') + _WX_('-') + _WX_('?');
-    else if (c < ' ')
-      wx += _WX_('C') + _WX_('-') + _WX_ (c + '@');
-    else
-      wx += _WX_ (c);
-    return wx;
 
-  default:
-    abort ();
-  }
+    case space:
+      return _WX_(' ');
+
+    case question_mark:
+      return _WX_('?');
+
+    case caret:
+      if (0177 < c)
+	{
+	  wx += _WX_('M') + _WX_('-');
+	  c &= 0177;
+	}
+      if (c == 0177)
+	wx += _WX_('^') + _WX_('?');
+      else if (c < ' ')
+	wx += _WX_ ('^') + _WX_ (c + '@');
+      else
+	wx += _WX_ (c);
+      return wx;
+
+    case Emacs:
+      if (0177 < c)
+	{
+	  wx += _WX_('M') + _WX_('-');
+	  c &= 0177;
+	}
+      if (c == 0177)
+	wx += _WX_('C') + _WX_('-') + _WX_('?');
+      else if (c < ' ')
+	wx += _WX_('C') + _WX_('-') + _WX_ (c + '@');
+      else
+	wx += _WX_ (c);
+      return wx;
+
+    default:
+      abort ();
+    }
 
   return 0;	/* For -Wall */
 }
@@ -1064,7 +1077,7 @@ char_WX (a2ps_job * job, uchar c)
 /*
  *	Returns the WX of a string (including M- and ^)
  */
-unsigned int 
+unsigned int
 string_WX (a2ps_job * job, uchar * string)
 {
   unsigned int result=0;
@@ -1084,14 +1097,13 @@ set_encoding (struct a2ps_job * job, struct encoding * enc)
   job->encoding = enc;
 }
 
-/*
- * Collated comparison between names of encodings
- */
+/* Compare names with strverscmp, so that latin 15 is after latin 2.  */
+
 static int
 da_encoding_name_cmp (char * key1, char * key2, struct a2ps_job * job)
 {
-  return strcoll ((char *) encoding_get (job, key1)->name,
-		  (char *) encoding_get (job, key2)->name);
+  return strverscmp ((char *) encoding_get (job, key1)->name,
+		     (char *) encoding_get (job, key2)->name);
 }
 
 /*
@@ -1100,20 +1112,9 @@ da_encoding_name_cmp (char * key1, char * key2, struct a2ps_job * job)
 void
 list_encodings_short (a2ps_job * job, FILE * stream)
 {
-  struct darray * entries;
-  size_t i;
-
-  entries = pw_glob_on_suffix (job->common.path, EDF_SUFFIX);
-  
-  fputs (_("Known encodings"), stream);
-
-  for (i = 0 ; i < entries->len ; i++){
-    if (!(i % 7))
-      fputs ("\n  ", stream);
-    fprintf (stream, "%-10s", (char *) entries->content[i]);
-  }
+  fputs (_("Known Encodings"), stream);
   putc ('\n', stream);
-  da_free (entries, (da_map_func_t) free);
+  pw_lister_on_suffix (stream, job->common.path, EDF_SUFFIX);
 }
 
 /*
@@ -1125,22 +1126,23 @@ list_encodings_long (a2ps_job * job, FILE * stream)
   struct darray * entries;
   struct encoding * encoding;
   size_t i;
-  
+
   entries = pw_glob_on_suffix (job->common.path, EDF_SUFFIX);
 
   /* We want them to be in order of the names, not keys */
-  da_qsort_with_arg (entries, 
+  da_qsort_with_arg (entries,
 		     (da_cmp_arg_func_t) da_encoding_name_cmp, job);
-  
-  fputs (_("Known encodings"), stream);
+
+  fputs (_("Known Encodings"), stream);
   putc ('\n', stream);
 
-  for (i = 0 ; i < entries->len ; i++) {
-    /* Don't forget to cut the suffix */
-    encoding = encoding_get (job, entries->content[i]);
-    encoding_print_signature (encoding, stream);
-    putc ('\n', stream);
-  }
+  for (i = 0 ; i < entries->len ; i++)
+    {
+      /* Don't forget to cut the suffix */
+      encoding = encoding_get (job, entries->content[i]);
+      encoding_print_signature (encoding, stream);
+      putc ('\n', stream);
+    }
   da_free (entries, (da_map_func_t) free);
 }
 /************************************************************************/
@@ -1154,9 +1156,9 @@ static void
 encoding_texinfo_print_signature (struct encoding * encoding,
 					FILE * stream)
 {
-  fprintf (stream, "@deftp {Encoding} {%s} (@file{%s.edf})\n", 
+  fprintf (stream, "@deftp {Encoding} {%s} (@file{%s.edf})\n",
 	   encoding->name, encoding->key);
-  
+
   documentation_print_texinfo (encoding->documentation, "%s", stream);
   fputs ("@end deftp\n\n", stream);
 }
@@ -1170,19 +1172,20 @@ list_texinfo_encodings_long (a2ps_job * job, FILE * stream)
   struct darray * entries;
   struct encoding * encoding;
   unsigned int i;
-  
+
   entries = pw_glob_on_suffix (job->common.path, EDF_SUFFIX);
 
   /* We want them to be in order of the names, not keys */
   da_qsort_with_arg (entries,
 		     (da_cmp_arg_func_t) da_encoding_name_cmp, job);
-  
+
   fputs ("The known encodings are:\n", stream);
-  
-  for (i = 0 ; i < entries->len ; i++) {
-    /* Don't forget to cut the suffix */
-    encoding = encoding_get (job, entries->content[i]);
-    encoding_texinfo_print_signature (encoding, stream);
-  }
+
+  for (i = 0 ; i < entries->len ; i++)
+    {
+      /* Don't forget to cut the suffix */
+      encoding = encoding_get (job, entries->content[i]);
+      encoding_texinfo_print_signature (encoding, stream);
+    }
   da_free (entries, (da_map_func_t) free);
 }
