@@ -18,7 +18,7 @@
 
 /*
  * This version has been modified by Akim Demaille <demaille@inf.enst.fr>
- * so that 
+ * so that
  * - hash_free_item takes as argument the free function to be applied
  *   to the items.
  * - hash_free takes a function pointer supposed to be the free
@@ -30,22 +30,24 @@
 #ifndef _HASHTAB_H_
 #define _HASHTAB_H_
 
-#ifndef __P
+#ifndef PARAMS
 # if defined (__GNUC__) || (defined (__STDC__) && __STDC__)
-#  define __P(args) args
+#  define PARAMS(args) args
 # else
-#  define __P(args) ()
+#  define PARAMS(args) ()
 # endif  /* GCC.  */
-#endif  /* Not __P.  */
+#endif  /* Not PARAMS.  */
 
 #include <stdio.h>
+#include <ctype.h>
 
-typedef unsigned long (*hash_func_t) __P((void const *key));
-typedef int (*hash_cmp_func_t) __P((void const *x, void const *y));
-typedef void (*hash_map_func_t) __P((void const *item));
-typedef void (*hash_maparg_func_t) __P((void const *item, void const * arg));
+typedef unsigned long (*hash_func_t) PARAMS((void const *key));
+typedef int (*hash_cmp_func_t) PARAMS((void const *x, void const *y));
+typedef int (*hash_select_func_t) PARAMS ((void const *item));
+typedef void (*hash_map_func_t) PARAMS((void const *item));
+typedef void (*hash_maparg_func_t) PARAMS((void const *item, void const * arg));
 
-struct hash_table
+struct hash_table_s
 {
   void **ht_vec;
   unsigned long ht_size;	/* total number of slots (power of 2) */
@@ -59,26 +61,28 @@ struct hash_table
   hash_cmp_func_t ht_compare;	/* comparison function */
 };
 
-typedef int (*qsort_cmp_t) __P((void const *, void const *));
+typedef int (*qsort_cmp_t) PARAMS((void const *, void const *));
 
-void hash_init __P((struct hash_table *ht, unsigned long size,
+void hash_init PARAMS((struct hash_table_s *ht, unsigned long size,
 		    hash_func_t hash_1, hash_func_t hash_2, hash_cmp_func_t hash_cmp));
-void hash_load __P((struct hash_table *ht, void *item_table,
+void hash_load PARAMS((struct hash_table_s *ht, void *item_table,
 		    unsigned long cardinality, unsigned long size));
-void **hash_find_slot __P((struct hash_table *ht, void const *key));
-void *hash_find_item __P((struct hash_table *ht, void const *key));
-void *hash_insert __P((struct hash_table *ht, void *item));
-void *hash_insert_at __P((struct hash_table *ht, void *item, void const *slot));
-void *hash_delete __P((struct hash_table *ht, void const *item));
-void *hash_delete_at __P((struct hash_table *ht, void const *slot));
-void hash_delete_items __P((struct hash_table *ht));
-void hash_free_items __P((struct hash_table *ht, hash_map_func_t free_fn));
-void hash_free __P((struct hash_table *ht, hash_map_func_t free_fn));
-void hash_map __P((struct hash_table *ht, hash_map_func_t map));
-void hash_maparg __P((struct hash_table *ht, hash_maparg_func_t map, 
+void **hash_find_slot PARAMS((struct hash_table_s *ht, void const *key));
+void *hash_find_item PARAMS((struct hash_table_s *ht, void const *key));
+void *hash_insert PARAMS((struct hash_table_s *ht, void *item));
+void *hash_insert_at PARAMS((struct hash_table_s *ht, void *item, void const *slot));
+void *hash_delete PARAMS((struct hash_table_s *ht, void const *item));
+void *hash_delete_at PARAMS((struct hash_table_s *ht, void const *slot));
+void hash_delete_items PARAMS((struct hash_table_s *ht));
+void hash_free_items PARAMS((struct hash_table_s *ht, hash_map_func_t free_fn));
+void hash_free PARAMS((struct hash_table_s *ht, hash_map_func_t free_fn));
+void hash_map PARAMS((struct hash_table_s *ht, hash_map_func_t map));
+void hash_maparg PARAMS((struct hash_table_s *ht, hash_maparg_func_t map,
 		      void const *arg, qsort_cmp_t compare));
-void hash_print_stats __P((struct hash_table *ht, FILE *out_FILE));
-void **hash_dump __P((struct hash_table *ht, void **vector_0, qsort_cmp_t compare));
+void hash_print_stats PARAMS((struct hash_table_s *ht, FILE *out_FILE));
+void **hash_dump PARAMS((struct hash_table_s *ht, void **vector_0, qsort_cmp_t compare));
+void **hash_dump_select PARAMS((struct hash_table_s *ht, void **vector_0,
+				hash_select_func_t select_fn, qsort_cmp_t compare));
 
 extern void *hash_deleted_item;
 #define HASH_VACANT(item) ((item) == 0 || (void *) (item) == hash_deleted_item)
@@ -86,7 +90,7 @@ extern void *hash_deleted_item;
 
 /* hash and comparison macros for string keys. */
 
-#define STRING_HASH_1(_key_, _result_) { \
+#define STRING_HASH_1(_key_, _result_) do { \
   unsigned char const *kk = (unsigned char const *) (_key_) - 1; \
   while (*++kk) \
     (_result_) += (*kk << (kk[1] & 0xf)); \
@@ -125,6 +129,48 @@ extern void *hash_deleted_item;
   return result; \
 } while (0)
 
+/* hash and comparison macros for string keys, case insensitive. */
+
+#define STRING_CASE_HASH_1(_key_, _result_) do { \
+  unsigned char const *kk = (unsigned char const *) (_key_) - 1; \
+  while (*++kk) \
+    (_result_) += (tolower (*kk) << (tolower (kk[1]) & 0xf)); \
+} while (0)
+#define return_STRING_CASE_HASH_1(_key_) do { \
+  unsigned long result = 0; \
+  STRING_CASE_HASH_1 ((_key_), result); \
+  return result; \
+} while (0)
+
+#define STRING_CASE_HASH_2(_key_, _result_) do { \
+  unsigned char const *kk = (unsigned char const *) (_key_) - 1; \
+  while (*++kk) \
+    (_result_) += (tolower(*kk) << (tolower(kk[1]) & 0x7)); \
+} while (0)
+#define return_STRING_CASE_HASH_2(_key_) do { \
+  unsigned long result = 0; \
+  STRING_HASH_2 ((_key_), result); \
+  return result; \
+} while (0)
+
+#define STRING_CASE_COMPARE(_x_, _y_, _result_) do { \
+  unsigned char const *xx = (unsigned char const *) (_x_) - 1; \
+  unsigned char const *yy = (unsigned char const *) (_y_) - 1; \
+  do { \
+    if (*++xx == '\0') { \
+      yy++; \
+      break; \
+    } \
+    yy++; \
+  } while (tolower (*xx) == tolower (*yy)); \
+  (_result_) = tolower(*xx) - tolower(*yy); \
+} while (0)
+#define return_STRING_CASE_COMPARE(_x_, _y_) do { \
+  int result; \
+  STRING_CASE_COMPARE (_x_, _y_, result); \
+  return result; \
+} while (0)
+
 /* hash and comparison macros for integer keys. */
 
 #define INTEGER_HASH_1(_key_, _result_) do { \
