@@ -48,7 +48,7 @@
 #endif
 
 #ifndef PARAMS
-#  if defined (__STDC__) || defined (_AIX) \
+#  if PROTOTYPES || defined (__STDC__) || defined (_AIX) \
       || (defined (__mips) && defined (_SYSTYPE_SVR4)) \
       || defined(WIN32) || defined(__cplusplus)
 #    define PARAMS(protos) protos
