@@ -9,7 +9,7 @@
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -35,7 +35,7 @@
 #include "jobs.h"
 #include "fjobs.h"
 #include "stpncpy.h"
-#include "msg.h"
+#include "message.h"
 #include "metaseq.h"
 #include "xobstack.h"
 #include "pair_ht.h"
@@ -67,7 +67,7 @@ macro_meta_sequence_table_free (struct pair_htable * table)
 /*
  * Check if is a valid name, and add.
  * (Note, strdup is done so that no memory is shared with key and value)
- * 
+ *
  */
 boolean
 macro_meta_sequence_add (struct a2ps_job * job,
@@ -77,7 +77,7 @@ macro_meta_sequence_add (struct a2ps_job * job,
     return false;
 
   /* We want to remove any white space before the command */
-  pair_add (job->macro_meta_sequences, 
+  pair_add (job->macro_meta_sequences,
 	    key, value + strspn (value, "\t "));
   return true;
 }
@@ -101,6 +101,7 @@ macro_meta_sequences_list_short (struct a2ps_job * job, FILE * stream)
      are things such as #(psnup) which is substituted to a bigger strings,
      e.g. -#v #?q|-q|| #?j|-d|| #?r||-c| -w#w -h#h */
   fprintf (stream, _("Known Variables"));
+  putc ('\n', stream);
   pair_table_list_short (job->macro_meta_sequences, stream);
 }
 
@@ -151,7 +152,7 @@ macro_meta_sequences_list_long (struct a2ps_job * job,
 
 /*
  * We can't use strtok, that would skip the empty fields
- */	    
+ */
 #define SPLIT(to,sep,esc,cat)						\
    do {									\
      to = next ;							\
@@ -184,14 +185,15 @@ macro_meta_sequences_list_long (struct a2ps_job * job,
 
 
 /*
- * Once again, I must admit a brand lot of this comes from
- * GNU enscript
+ * Using the data in JOB, and in the current FILE data descriptor,
+ * expand the possilbity escaped STR in the current USER_STRING_STACK.
+ * Use CONTEXT_NAME as a mean to report more understandable error/logs.
  */
 static void
 grow_user_string_obstack (struct obstack * user_string_stack,
 			  struct a2ps_job * job,
 			  struct file_job * file,
-			  const uchar * context_name, 
+			  const uchar * context_name,
 			  const uchar * str)
 {
   uchar * cp, * cp2;
@@ -205,30 +207,30 @@ grow_user_string_obstack (struct obstack * user_string_stack,
   for (i = 0; str[i] != '\0'; i++)
     {
       int type;
-      
+
       type = str[i];
       if (type == '%' || type == '$' || type == '#' || type == '\\') {
 	i++;
 	width = 0;
 	justification = 1;
 	padding = ' ';
-	
+
 	/* Get optional width and justification. */
 	if (str[i] == '-') {
 	  i++;
 	  justification = -1;
-	  if (!isdigit ((int) str[i]))
+	  if (!ISDIGIT ((int) str[i]))
 	    padding = str[i++];
 	}
 	if (str[i] == '+') {
 	  i++;
 	  justification = 1;
-	  if (!isdigit ((int) str[i]))
+	  if (!ISDIGIT ((int) str[i]))
 	    padding = str[i++];
 	}
-	while (isdigit ((int) str[i]))
+	while (ISDIGIT ((int) str[i]))
 	  width = width * 10 + str[i++] - '0';
-	
+
 	/* Handle escapes. */
 	switch (type) {
 	  /*
@@ -244,22 +246,6 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	   */
 	case '\\':
 	  switch (str[i]) {
-	  case '\\':	/* `\\' character `\' */
-	    APPEND_CH ('\\');
-	    break;
-
-	  case '%':	/* `\%' character % */
-	    APPEND_CH ('%');
-	    break;
-
-	  case '$':	/* `\$' character $ */
-	    APPEND_CH ('$');
-	    break;
-
-	  case '#':	/* `\#' character # */
-	    APPEND_CH ('#');
-	    break;
-
 	  case 'f':	/* `\f' character \f */
 	    APPEND_CH ('\f');
 	    break;
@@ -269,8 +255,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  default:
-	    error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
-		   context_name, "\\", str[i], str[i]);
+	    APPEND_CH (str [i]);
 	    break;
 	  }
 	  break;
@@ -292,77 +277,33 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	  case '%':	/* `%%' character `%' */
 	    APPEND_CH ('%');
 	    break;
-	    
+
 	  case '#':	/* `%#': total number of files */
 	    APPEND_CH (JOB_NB_FILES);
 	    break;
 
 	  case 'a':	/* `%a' NLS'ed `printed by USERNAME */
-	    if (str[i + 1] == '{')
-	      {
-		/* `%a{username@machine}' */
-		for (j = 0, i += 2;
-		     j < sizeof (buf) && str[i] && str[i] != '}';
-		     i++, j++)
-		  buf[j] = str[i];
-		if (str[i] != '}')
-		  error (1, 0, _("%s: too long argument for %s escape"),
-			 context_name, "%a{}");
-		      
-		buf[j] = '\0';
-		cp = ustrrchr (buf, '@');
-		if (cp)
-		  *cp = NUL;
-	      } else {
-		/* `%a' just prints NLS'ed `printed by USERNAME' */
-		ustrcpy (buf, job->user->pw_gecos);
-		cp = ustrchr (buf, ',');
-		if (cp)
-		  *cp = '\0';
-	      }
-	    sprintf ((char *)buf2, _("Printed by %s"), buf);
+	    sprintf ((char *) buf2,
+		     _("Printed by %s"),
+		     macro_meta_sequence_get (job, VAR_USER_NAME));
 	    APPEND_STR (buf2);
 	    break;
 
 	  case 'A':  /* `%A' NLS'ed `printed by USERNAME from MACHINE */
-	    if (str[i + 1] == '{')
-	      {		      		/* `%a{username@machine}' */
-		for (j = 0, i += 2;
-		     j < sizeof (buf) && str[i] && str[i] != '}';
-		     i++, j++)
-		  buf[j] = str[i];
-		if (str[i] != '}')
-		  error (1, 0, _("%s: too long argument for %s escape"),
-			 "%A{}", context_name);
-		buf[j] = '\0';
-
-		cp = ustrrchr (buf, '@');
-		if (cp) {
-		  *cp = NUL;
-		  cp ++;
-		}
-	      } else {				/* `%a' */
-		ustrcpy (buf, job->user->pw_gecos);
-		cp = ustrchr (buf, ',');
-		if (cp)
-		  *cp = '\0';
-		cp = (uchar *) xgethostname ();
-		cp2 = ustrchr (cp, '.');
-		if (cp2)
-		  *cp2 = '\0';
-	      }
-	    if (cp)
-	      sprintf ((char *)buf3, 
-		       _("Printed by %s from %s"), buf, cp);
+	    cp = macro_meta_sequence_get (job, VAR_USER_NAME);
+	    cp2 = macro_meta_sequence_get (job, VAR_USER_HOST);
+	    if (cp2)
+	      sprintf ((char *) buf3,
+		       _("Printed by %s from %s"), cp, cp2);
 	    else
-	      sprintf ((char *)buf3, _("Printed by %s"), buf);
+	      sprintf ((char *) buf3, _("Printed by %s"), cp);
 	    APPEND_STR (buf3);
 	    break;
 
 	  case 'c':	/* `%c' trailing component of pwd. */
 	    cp = (uchar *) xgetcwd ();
 	    if (!cp)
-	      error (1, errno, 
+	      error (1, errno,
 		     _("cannot get current working directory"));
 	    cp2 = ustrrchr (cp, DIR_SEP);
 	    if (cp2)
@@ -381,7 +322,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	  case 'd':	/* `%d' current working directory */
 	    cp = (uchar *) xgetcwd ();
 	    if (!cp)
-	      error (1, errno, 
+	      error (1, errno,
 		     _("cannot get current working directory"));
 	    APPEND_STR (cp);
 	    XFREE (cp);
@@ -398,40 +339,40 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		if (str[i] != '}')
 		  error (1, 0, _("%s: too long argument for %s escape"),
 			 context_name, "%D{}");
-		      
+
 		buf2[j] = '\0';
-		strftime ((char *) buf, sizeof (buf), 
+		strftime ((char *) buf, sizeof (buf),
 			  (char *) buf2, &job->run_tm);
 	      }
 	    else
 	      {
 		/* `%D' run date in `yy-mm-dd' format */
-		sprintf ((char *)buf, "%02d-%02d-%02d", 
+		sprintf ((char *)buf, "%02d-%02d-%02d",
 			 job->run_tm.tm_year % 100,
-			 job->run_tm.tm_mon + 1, 
+			 job->run_tm.tm_mon + 1,
 			 job->run_tm.tm_mday);
 	      }
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'e':	/* `%e' run date in localized short format */
-	    strftime ((char *) buf, sizeof (buf), 
+	    strftime ((char *) buf, sizeof (buf),
 		      /* Translators: please make a short date format
 		       * according to the std form in your language, using
 		       * the standard strftime(3) */
 		      (_("%b %d, %y")), &job->run_tm);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'E':	/* `%E' run date in localized long format */
 	    /* Translators: please make a long date format
 	     * according to the std form in your language, using
 	     * the standard strftime (3) */
-	    strftime ((char *) buf, sizeof (buf), 
+	    strftime ((char *) buf, sizeof (buf),
 		      (_("%A %B %d, %Y")), &job->run_tm);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'F':	/* `%F' run date in `dd.mm.yyyy' format */
 	    sprintf ((char *)buf, "%d.%d.%d",
 		     job->run_tm.tm_mday,
@@ -441,30 +382,25 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  case 'm':	/* `%m' the hostname up to the first `.' */
-	    cp = (uchar *) xgethostname ();
-	    cp2 = ustrchr (cp, '.');
-	    if (cp2)
-	      *cp2 = '\0';
-	    APPEND_STR (cp);
-	    XFREE (cp);
+	    cp = macro_meta_sequence_get (job, VAR_USER_HOST);
+	    cp2 = ALLOCA (uchar, strlen (cp) + 1);
+	    strcpy (cp2, cp);
+	    cp = ustrchr (cp2, '.');
+	    if (cp)
+	      *cp = '\0';
+	    APPEND_STR (cp2);
 	    break;
 
 	  case 'M':	/* `%M' the full hostname */
-	    cp = (uchar *) xgethostname ();
-	    APPEND_STR (cp);
-	    XFREE (cp);
+	    APPEND_STR (macro_meta_sequence_get (job, VAR_USER_HOST));
 	    break;
 
-	  case 'n':	/* `%n' username */
-	    APPEND_STR (job->user->pw_name);
+	  case 'n':	/* `%n' user's login */
+	    APPEND_STR (macro_meta_sequence_get (job, VAR_USER_LOGIN));
 	    break;
 
-	  case 'N':	/* `%N' pw_gecos up to the first `,' char */
-	    ustrcpy (buf, job->user->pw_gecos);
-	    cp = ustrchr (buf, ',');
-	    if (cp)
-	      *cp = '\0';
-	    APPEND_STR (buf);
+	  case 'N':	/* `%N' user's name */
+	    APPEND_STR (macro_meta_sequence_get (job, VAR_USER_NAME));
 	    break;
 
 	  case 'p':	/* `%p' related to the pages of the job */
@@ -477,25 +413,25 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    case '#':	/* `%p#' total number of pages */
 	      APPEND_CH (JOB_NB_PAGES);
 	      break;
-	
+
 	    default:
 	      error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		     context_name, "%p", str [i], str [i]);
 	      break;
 	    }
 	    break;
-		  
+
 	  case 'q':	/* `%q' localized `Page %d' */
 	    sprintf ((char *)buf, _("Page %d"), job->pages);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'Q':	/* `%Q' localized `Page %d/%c' */
-	    sprintf ((char *)buf, _("Page %d/%c"), 
+	    sprintf ((char *)buf, _("Page %d/%c"),
 		     job->pages, JOB_NB_PAGES);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 's':	/* `%s' related to the sheets of the job */
 	    switch (str [++i]) {
 	    case '.':	/* `%s.' current sheet number */
@@ -506,14 +442,14 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    case '#':	/* `%s#' total number of sheets */
 	      APPEND_CH (JOB_NB_SHEETS);
 	      break;
-	
+
 	    default:
 	      error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		     context_name, "%s", str [i], str [i]);
 	      break;
 	    }
 	    break;
-		  
+
 	  case 't':	/* `%t' runtime in 12-hour am/pm format */
 	    sprintf ((char *)buf, "%d:%02d%s",
 		     job->run_tm.tm_hour > 12
@@ -524,14 +460,14 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  case 'T':	/* `%T' runtime in 24-hour format */
-	    sprintf ((char *)buf, "%d:%02d", 
+	    sprintf ((char *)buf, "%d:%02d",
 		     job->run_tm.tm_hour, job->run_tm.tm_min);
 	    APPEND_STR (buf);
 	    break;
 
 	  case '*':	/* `%*' runtime in 24-hour format with secs */
 	    sprintf ((char *)buf, "%d:%02d:%02d",
-		     job->run_tm.tm_hour, 
+		     job->run_tm.tm_hour,
 		     job->run_tm.tm_min,
 		     job->run_tm.tm_sec);
 	    APPEND_STR (buf);
@@ -543,9 +479,9 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  case 'W':	/* `%W' run date in `mm/dd/yy' format */
-	    sprintf ((char *)buf, "%02d/%02d/%02d", 
+	    sprintf ((char *)buf, "%02d/%02d/%02d",
 		     job->run_tm.tm_mon + 1,
-		     job->run_tm.tm_mday, 
+		     job->run_tm.tm_mday,
 		     job->run_tm.tm_year % 100);
 	    APPEND_STR (buf);
 	    break;
@@ -554,11 +490,11 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		   context_name, "%", str[i], str[i]);
 	    break;
-		
+
 	  }
 	  break;
 
-	  /*  
+	  /*
 	   *		 #####
 	   *		#  #  #
 	   *		#  #
@@ -575,10 +511,10 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	  case '$':	/* `$$' character `$' */
 	    APPEND_CH ('$');
 	    break;
-	    
+
 	  case '*':	/* `$*' modif time in 24-hour format with secs */
-	    sprintf ((char *)buf, "%d:%02d:%02d", 
-		     file->mod_tm.tm_hour, 
+	    sprintf ((char *)buf, "%d:%02d:%02d",
+		     file->mod_tm.tm_hour,
 		     file->mod_tm.tm_min,
 		     file->mod_tm.tm_sec);
 	    APPEND_STR (buf);
@@ -626,7 +562,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      error (1, 0, _("%s: too long argument for %s escape"),
 		     context_name, "${}");
 	    buf[j] = '\0';
-	    
+
 	    /* Get the value of the env var */
 	    cp = (uchar *) getenv ((char *)buf);
 	    if (IS_EMPTY (cp2))
@@ -645,14 +581,14 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		  else
 		    APPEND_STR (cp2 + 1);
 		  break;
-		  
+
 		case '+':	/* if defined, then word */
 		  if (!IS_EMPTY (cp))
 		    APPEND_STR (cp2 + 1);
 		  break;
-		  
+
 		default:
-		  error (1, 0, 
+		  error (1, 0,
 			 _("%s: invalid separator `%s%c' for `%s' escape"),
 			 context_name, ":", *cp2, "${}");
 		}
@@ -660,12 +596,12 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  case '[':	/* `$[]' command line options */
-	    if (!isdigit ((int) str[i]))
+	    if (!ISDIGIT ((int) str[i]))
 	      error (1, 0,  _("%s: invalid argument for %s%c escape"),
 		     context_name, "$[", ']');
 	    {
 	      size_t value = 0;
-	      while (isdigit ((int) str[i]))
+	      while (ISDIGIT ((int) str[i]))
 		value = value * 10 + str[i++] - '0';
 	      if (str[i] == '\0')
 		error (1, 0,  _("%s: missing `%c' for %s%c escape"),
@@ -685,9 +621,9 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    break;
 
 	  case 'C':	/* `$C' modtime in `hh:mm:ss' format */
-	    sprintf ((char *)buf, "%d:%02d:%02d", 
+	    sprintf ((char *)buf, "%d:%02d:%02d",
 		     file->mod_tm.tm_hour,
-		     file->mod_tm.tm_min, 
+		     file->mod_tm.tm_min,
 		     file->mod_tm.tm_sec);
 	    APPEND_STR (buf);
 	    break;
@@ -734,13 +670,13 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    /* Translators: please make a short date format
 	     * according to the std form in your language, using
 	     * GNU strftime(3) */
-	    strftime ((char *) buf, sizeof (buf), 
+	    strftime ((char *) buf, sizeof (buf),
 		      (_("%b %d, %y")), &(file->mod_tm));
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'E':	/* `$E' mod date in localized long format */
-	    strftime ((char *) buf, sizeof (buf), 
+	    strftime ((char *) buf, sizeof (buf),
 		      /* Translators: please make a long date format
 		       * according to the std form in your language, using
 		       * GNU strftime(3) */
@@ -771,7 +707,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      sprintf ((char *)buf, "%d", file->lines - 1);
 	      APPEND_STR (buf);
 	      break;
-	      
+
 	    case '#':		/* `$l#' number of lines in this file */
 	      if (file != CURRENT_FILE (job)) {
 		/* This file is finised, we do know its real number of lines */
@@ -782,7 +718,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		APPEND_CH (FILE_NB_LINES);
 	      }
 	      break;
-	
+
 	    default:
 	      error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		     context_name, "$l", str [i], str [i]);
@@ -825,7 +761,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      sprintf ((char *)buf, "%d", file->top_page);
 	      APPEND_STR (buf);
 	      break;
-	      
+
 	    case '-':	/* `$p-' interval of the pages of the current file
 			 * appearing in the current sheet */
 	      if (file->top_page == file->pages)
@@ -834,12 +770,12 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		sprintf ((char *)buf, "%d-%d", file->top_page, file->pages);
 	      APPEND_STR (buf);
 	      break;
-	      
+
 	    case '<':	/* `$p<' first page number for this file */
 	      sprintf ((char *)buf, "%d", file->first_page);
 	      APPEND_STR (buf);
 	      break;
-	      
+
 	    case '.':	/* `$p.' current page number */
 	      sprintf ((char *)buf, "%d", file->pages);
 	      APPEND_STR (buf);
@@ -855,7 +791,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		APPEND_CH (FILE_LAST_PAGE);
 	      }
 	      break;
-	      
+
 	    case '#':	/* `$p#' total number of pages */
 	      if (file != CURRENT_FILE (job)) {
 		/* This file is finised, we do know its real number of pages */
@@ -866,39 +802,39 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		APPEND_CH (FILE_NB_PAGES);
 	      }
 	      break;
-	      
+
 	    default:
 	      error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		     context_name, "$p", str [i], str [i]);
 	      break;
 	    }
 	    break;
-	    
+
 	  case 'q':	/* `$q' localized `Page $p' */
 	    sprintf ((char *)buf, _("Page %d"), file->pages);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 'Q':	/* `$Q' localized `Page $p/$P' */
 	    if (file != CURRENT_FILE (job))
 	      /* This file is finised, we do know its real number of pages */
-	      sprintf ((char *) buf, _("Page %d/%d"), 
+	      sprintf ((char *) buf, _("Page %d/%d"),
 		       file->pages, file->pages);
 	    else
 	      /* It is not know: delay it to the end of the job */
-	      sprintf ((char *) buf, _("Page %d/%c"), 
+	      sprintf ((char *) buf, _("Page %d/%c"),
 		       file->pages,
 		       FILE_NB_PAGES);
 	    APPEND_STR (buf);
 	    break;
-		  
+
 	  case 's':	/* `$s' related to the sheets of the file */
 	    switch (str [++i]) {
 	    case '<':	/* `$s<' first sheet for this file */
 	      sprintf ((char *)buf, "%d", file->first_sheet);
 	      APPEND_STR (buf);
 	      break;
-	      
+
 	    case '.':	/* `$s.' current sheet number */
 	      sprintf ((char *)buf, "%d", file->sheets);
 	      APPEND_STR (buf);
@@ -914,7 +850,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		APPEND_CH (FILE_LAST_SHEET);
 	      }
 	      break;
-	      
+
 	    case '#':	/* `$s#' total number of sheets */
 	      if (file != CURRENT_FILE (job)) {
 		/* This file is finised, we know its number of sheets */
@@ -925,7 +861,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		APPEND_CH (FILE_NB_SHEETS);
 	      }
 	      break;
-	      
+
 	    default:
 	      error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		     context_name, "$s", str [i], str [i]);
@@ -939,44 +875,44 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      i++;
 	      APPEND_STR (job->tag1);
 	      break;
-		    
+
 	    case '2':	/* `$t2' second marker grabbed from file */
 	      i++;
 	      APPEND_STR (job->tag2);
 	      break;
-		    
+
 	    case '3':	/* `$t3' third marker grabbed from file */
 	      i++;
 	      APPEND_STR (job->tag3);
 	      break;
-		    
+
 	    case '4':	/* `$t4' fourth marker grabbed from file */
 	      i++;
 	      APPEND_STR (job->tag4);
 	      break;
-		    
+
 	    default:	/* `$t' runtime in 12-hour am/pm format */
 	      sprintf ((char *)buf, "%d:%02d%s",
 		       (file->mod_tm.tm_hour > 12
-			?file->mod_tm.tm_hour-12 
+			?file->mod_tm.tm_hour-12
 			:file->mod_tm.tm_hour),
 		       file->mod_tm.tm_min,
 		       file->mod_tm.tm_hour > 12 ? "pm" : "am");
 	      APPEND_STR (buf);
 	    }
 	    break;
-		  
+
 	  case 'T':	/* `$T' runtime in 24-hour format */
-	    sprintf ((char *)buf, "%d:%02d", 
+	    sprintf ((char *)buf, "%d:%02d",
 		     file->mod_tm.tm_hour,
 		     file->mod_tm.tm_min);
 	    APPEND_STR (buf);
 	    break;
 
 	  case 'W':	/* `$W' run date in `mm/dd/yy' format */
-	    sprintf ((char *)buf, "%02d/%02d/%02d", 
+	    sprintf ((char *)buf, "%02d/%02d/%02d",
 		     file->mod_tm.tm_mon + 1,
-		     file->mod_tm.tm_mday, 
+		     file->mod_tm.tm_mday,
 		     file->mod_tm.tm_year % 100);
 	    APPEND_STR (buf);
 	    break;
@@ -1002,14 +938,14 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	  case '#':	/* `##' character `#' */
 	    APPEND_CH ('#');
 	    break;
-	    
+
 	  case '(':	/* #(macro meta sequence)  */
 	    /* FIXME: Some day should disapear in favor of #{} */
 	    for (j = 0, i++;
 		 str[i] && str[i] != ')' && j < sizeof (buf) - 1;
 		 i++)
 	      buf[j++] = str[i];
-	    
+
 	    if (str[i] == '\0')
 	      error (1, 0, _("%s: missing `%c' for %s%c escape"),
 		     context_name, ')', "#(", ')');
@@ -1017,11 +953,11 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      error (1, 0, _("%s: too long argument for %s escape"),
 		     context_name, "#()");
 	    buf[j] = '\0';
-	    
+
 	    cp = (uchar *) macro_meta_sequence_get (job,
 						    (char *) buf);
 	    if (cp)
-	      grow_user_string_obstack (user_string_stack, 
+	      grow_user_string_obstack (user_string_stack,
 					job, file,
 					context_name, cp);
 	    break;
@@ -1050,14 +986,14 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      error (1, 0, _("%s: too long argument for %s escape"),
 		     context_name, "#{}");
 	    buf[j] = '\0';
-	    
+
 	    /* Get the value of the macro */
 	    cp = (uchar *) macro_meta_sequence_get (job, (char *) buf);
 	    if (IS_EMPTY (cp2))
 	      {
 		/* No word specified */
 		if (cp)
-		  grow_user_string_obstack (user_string_stack, 
+		  grow_user_string_obstack (user_string_stack,
 					    job, file,
 					    context_name, cp);
 	      }
@@ -1067,20 +1003,20 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		switch (*cp2) {
 		case '-':	/* if macro defined value else word */
 		  if (!IS_EMPTY (cp))
-		    grow_user_string_obstack (user_string_stack, 
+		    grow_user_string_obstack (user_string_stack,
 					      job, file,
 					      context_name, cp);
 		  else
 		    APPEND_STR (cp2 + 1);
 		  break;
-		  
+
 		case '+':	/* if macro defined, word */
 		  if (!IS_EMPTY (cp))
 		    APPEND_STR (cp2 + 1);
 		  break;
-		  
+
 		default:
-		  error (1, 0, 
+		  error (1, 0,
 			 _("%s: invalid separator `%s%c' for `%s' escape"),
 			 context_name, ":", *cp2, "#{}");
 		}
@@ -1100,7 +1036,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      uchar * next;
 
 	      cond = str[++i];
-	      sep = str[++i];		    
+	      sep = str[++i];
 	      next = xustrdup(str + ++i);
 
 	      SPLIT (if_true, sep, "#?", cond);
@@ -1122,7 +1058,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		break;
 
 	      case 'd': /* `#?d' Double sided printing		*/
-		test = job->duplex == true;
+		test = job->duplex == duplex || job->duplex == tumble;
 		break;
 
   	      case 'j': /* `#?j' Bordering is asked (-j)		*/
@@ -1143,7 +1079,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		break;
 
 	      case 'q':	/* `#?q' in quiet mode */
-		test = message_verbosity == 0;
+		test = msg_verbosity == 0;
 		break;
 
 	      case 'r': /* `#?r' madir = row		*/
@@ -1151,7 +1087,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		break;
 
 	      case 'V':	/* `#?V' verbose mode */
-		test = message_test (msg_tool);
+		test = msg_test (msg_tool);
 		break;
 
 	      case 'v': /* `#?v' on a verso side */
@@ -1172,11 +1108,11 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	       * direction
 	       */
 	      if (test)
-		grow_user_string_obstack (user_string_stack, 
+		grow_user_string_obstack (user_string_stack,
 					  job, file,
 					  context_name, if_true);
 	      else
-		grow_user_string_obstack (user_string_stack, 
+		grow_user_string_obstack (user_string_stack,
 					  job, file,
 					  context_name, if_false);
 	      FREE (if_true);
@@ -1212,7 +1148,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		  }
 		}
 		break;
-		
+
 	      case 'f':		/* `#!f': enumeration of the input files */
 		{
 		  size_t fnum, fmax;
@@ -1230,13 +1166,13 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		  }
 		}
 		break;
-		
+
 	      case 'F':		/* `#!F': enumeration of the input files
 				 * in alpha order */
 		{
 		  size_t fnum, fmax;
 		  struct darray * ordered;
-		  
+
 		  /* Make a ordered clone of the jobs array */
 		  ordered = da_clone (job->jobs);
 		  ordered->cmp = (da_cmp_func_t) file_name_cmp;
@@ -1256,27 +1192,27 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		  da_erase (ordered);
 		}
 		break;
-		
+
 	      case 's':		/* `#!s': enumeration of the input files
 				 * appearing in the current sheets */
 		{
 		  size_t fnum, fmax;
 		  struct darray * selected;
-		  
+
 		  /* Make a ordered clone of the jobs array */
 		  selected = da_clone (job->jobs);
 
 		  /* Make the selection:
-		   * Only the files before this sheet are known, 
+		   * Only the files before this sheet are known,
 		   * so just test on the last page number */
-		  fnum = 0 ; 
+		  fnum = 0 ;
 		  while (fnum < selected->len) {
 		    if (fjob (selected, fnum)->last_sheet < job->sheets)
 		      da_remove_at (selected, fnum, NULL);
 		    else
 		      fnum++;
 		  }
-		  
+
 		  fmax = limit_by_width (selected->len);
 		  for (fnum = 0 ; fnum < fmax ; fnum++) {
 		    grow_user_string_obstack (user_string_stack,
@@ -1292,7 +1228,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		  da_erase (selected);
 		}
 		break;
-		
+
 	      default:
 		error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		       context_name, "#!", category, category);
@@ -1308,6 +1244,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      if (k < 0 || 9 < k)
 		error (1, 0, _("%s: unknown `%s' escape `%c' (%d)"),
 		       context_name, "#f", str [i], str [i]);
+	      tempname_ensure (job->tmp_filenames [k], job->common.tmpdir);
 	      APPEND_STR (job->tmp_filenames [k]);
 	    }
 	    break;
@@ -1317,20 +1254,18 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    APPEND_STR (buf);
 	    break;
 
-	  case 'o':	/* `#o' name of destination,
-			 * before evaluation */
-	    APPEND_STR (a2ps_printers_output_name_get (job->printers));
+	  case 'o':	/* `#o' name of destination, before evaluation */
+	    APPEND_STR (a2ps_printers_flag_output_name_get (job->printers));
 	    break;
 
-	  case 'O':	/* `#O' name of destination,
-			 * after evaluation */
-	    if (a2ps_printers_output_is_printer_get (job->printers))
-	      grow_user_string_obstack 
+	  case 'O':	/* `#O' name of destination, after evaluation */
+	    if (a2ps_printers_flag_output_is_printer_get (job->printers))
+	      grow_user_string_obstack
 		(user_string_stack, job, file,
 		 (const uchar *) _("output command"),
-		 (const uchar *) a2ps_printers_output_name_get(job->printers));
+		 (const uchar *) a2ps_printers_flag_output_name_get(job->printers));
 	    else
-	      APPEND_STR (a2ps_printers_output_name_get (job->printers));
+	      APPEND_STR (a2ps_printers_flag_output_name_get (job->printers));
 	    break;
 
 	  case 'p':	/* `#p' page range of what remains to be printed.
@@ -1371,7 +1306,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 uchar *
 expand_user_string (struct a2ps_job * job,
 		    struct file_job * file,
-		    const uchar * context_name, 
+		    const uchar * context_name,
 		    const uchar * str)
 {
   static int first_time = 1;
@@ -1384,18 +1319,18 @@ expand_user_string (struct a2ps_job * job,
       first_time = 0;
       obstack_init (&user_string_stack);
     }
-  
+
   message (msg_meta,
 	   (stderr, "Expanding of %s user string (`%s')\n",
 	    context_name, str));
 
   grow_user_string_obstack (&user_string_stack,
 			    job, file, context_name, str);
-  
+
   obstack_1grow (&user_string_stack, '\0');
   res = (uchar *) obstack_finish (&user_string_stack);
   obstack_free (&user_string_stack, res);
-  
+
   message (msg_meta,
 	   (stderr, "Expansion of %s (`%s') is `%s'\n",
 	    context_name, str, res));
