@@ -1,15 +1,15 @@
 /*
- * confg.c
+ * options.c
  *
- * Read and understanding everything about the options 
- * & (dynamic) configuration of a2ps.
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Read and understanding everything about the options
+ *
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -36,22 +36,28 @@
 /*			I n c l u d e   f i l e s			*/
 /*                                                                      */
 /************************************************************************/
+#include <assert.h>
+
 #include "a2ps.h"
 #include "routines.h"
 #include "options.h"
 #include "argmatch.h"
 #include "getopt.h"
 #include "caret.h"
-#include "msg.h"
+#include "message.h"
 #include "jobs.h"
 #include "useropt.h"
 #include "prange.h"
 #include "metaseq.h"
 #include "madir.h"
 #include "getnum.h"
+#include "argv.h"
+#include "quotearg.h"
+#include "filalign.h"
 
 #define MAN_LINES               66	/* no lines for a man */
-extern char * program_name;
+extern char *program_name;
+extern const char *program_invocation_name;
 
 /*
  * Hooks used
@@ -60,7 +66,7 @@ option_hook handle_option_hook = NULL;
 
 
 
-static struct option long_options[] =
+static struct option const long_options[] =
 {
   /* Tasks */
   {"help",		no_argument,		0, 'h'},
@@ -76,9 +82,9 @@ static struct option long_options[] =
   {"help-pretty-print",	no_argument,		0, 162},
 
   /* Global */
-  {"macro-meta-sequence",required_argument,	0, 137},
-  {"variable",		required_argument,	0, 137},
-  {"define",		required_argument,	0, 137},
+  {"macro-meta-sequence",required_argument,	0, 'D'},
+  {"variable",		required_argument,	0, 'D'},
+  {"define",		required_argument,	0, 'D'},
   {"user-option",	required_argument,	0, '='},
 
   /* Pretty Print */
@@ -89,19 +95,19 @@ static struct option long_options[] =
   {"prologue",		required_argument,	0, 134},
   {"include",		required_argument,	0, 134},
   {"sides", 		required_argument,	0, 's'},
-  {"statusdict",	required_argument,	0, 'S'},
-  {"setpagedevice",	required_argument,      0, 'D'},
+  {"statusdict",	required_argument,	0, 164},
+  {"setpagedevice",	required_argument,      0, 'S'},
 
   /* Not sorted yet */
   {"columns",		required_argument,	0, 132},
   {"rows",		required_argument,	0, 133},
-  {"compact",		required_argument,      0, 150}, /* -A */
+  {"compact",		required_argument,      0, 'A'}, /* -A */
+  {"file-align",	required_argument,      0, 'A'}, /* -A */
   {"header",		optional_argument,	0, 'b'},
   {"no-header", 	no_argument, 		0, 'B'},
   {"truncate-lines",	required_argument,	0, 151}, /* -c */
   {"line-numbers",	required_argument,      0, 152}, /* -C */
 
-
   {"font-size",		required_argument, 	0, 'f'},
 
   /* Obsolete, replaced by the following */
@@ -112,7 +118,7 @@ static struct option long_options[] =
   {"end-of-line", 	required_argument,	0, 169},
   {"borders", 		required_argument, 	0, 156}, /* -j */
   {"page-prefeed",	no_argument,		0, 'k'},
-  {"no-page-prefeed",	no_argument,		0, 'K'},  
+  {"no-page-prefeed",	no_argument,		0, 'K'},
   {"lines-per-page",	required_argument,	0, 'L'},
   {"chars-per-line",	required_argument,	0, 'l'},
   {"catman",		no_argument,		0, 'm'},
@@ -159,12 +165,12 @@ static struct option long_options[] =
   {"toc",		optional_argument,	0, 167},
   {"pages", 		optional_argument,	0, 'a'},
 
-  /* Free: 164, 165, 168, 170, 171, 172 Next to use is: 175 */
+  /* Free: 137, 150, 165, 168, 170, 171, 172 Next to use is: 175 */
   {NULL, 0, 0, 0}
 };
 
 #define OPT_STRING \
-"123456789=:Aa:b::BcCdD:E::f:gGhijkKl:L:mM:n:o:P:qrRs:S:t::T:u::v::VX:Z"
+"123456789=:A:a:b::BcCdD:E::f:gGhijkKl:L:mM:n:o:P:qrRs:S:t::T:u::v::VX:Z"
 
 /************************************************************************/
 /*				arguments				*/
@@ -172,39 +178,6 @@ static struct option long_options[] =
 /************************************************************************/
 /* Helping routines for the options' arguments				*/
 /************************************************************************/
-/*
- * What about the backup machinery
- * (inspired by "getversion.c" by David MacKenzie <djm@gnu.ai.mit.edu>
- */
-static const char *const backup_args[] =
-{
-  "none", "off", 
-  "never", "simple", 
-  "nil", "existing", 
-  "t", "numbered", 
-  0
-};
-
-static const enum backup_type backup_types[] =
-{
-  none, none, 
-  simple, simple, 
-  numbered_existing, numbered_existing, 
-  numbered, numbered
-};
-
-/* Return the type of backup indicated by VERSION.
-   Unique abbreviations are accepted. */
-enum backup_type
-a2ps_get_backup_type (const char *version)
-{
-  if (version == 0 || *version == 0)
-    return numbered_existing;
-  else
-    return XARGCASEMATCH ("--version-control", version, 
-                          backup_args, backup_types);
-}
-
 /*
  * What about the booleans
  */
@@ -225,15 +198,14 @@ static const boolean boolean_types[] =
  * Return the boolean value
  */
 boolean
-a2ps_get_boolean (const char * option, const char * arg)
+a2ps_get_boolean (const char *option, const char *arg)
 {
+  ARGMATCH_ASSERT (boolean_args, boolean_types);
   return XARGCASEMATCH (option, arg, boolean_args, boolean_types);
 }
 
 
-/*
- * Return the non printable format
- */
+/* --non-printable-format */
 static const char *const non_printable_args[] =
 {
   "octal",
@@ -255,44 +227,127 @@ static const enum unprintable_format non_printable_types[] =
   caret
 };
 
-static enum unprintable_format
-get_non_printable_format (const char * option, const char * arg)
+/* --sides: Duplex, Simplex, Tumble.
+   I don't want `DuplexTumble' and `DuplexNoTumble' because it makes
+   too many characters of ambiguity.  This way, the user can say `-sd'
+   to mean `-sduplex'. */
+static const char *const duplex_args[] =
 {
-  return XARGCASEMATCH (option, arg, non_printable_args, non_printable_types);
-}
+  "simplex", "1", "none",
+  "duplex", "2",
+  "tumble",
+  0
+};
+
+static const enum duplex_e duplex_types[] =
+{
+  simplex, simplex, simplex,
+  duplex, duplex,
+  tumble
+};
+
 
 /************************************************************************/
 /*	Handle the options						*/
 /************************************************************************/
-int
-a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
+/* There is a difficult task which is expand the user options.
+   Because one cannot make recursive calls to getopt, because it is
+   better not to try to patch getopt, we do it in two steps.
+
+   First call getopt to expand the options in pairs (option, argument).
+
+   Second process these pairs of (option, argument).
+
+   Since during the second step we are out of getopt, it is then safe
+   to call getopt again, so there are no special problem in the
+   handling of user options, even if user options use user options.
+
+   This simple algorithm was suggested by Tristan Gingold. */
+
+/* A link list of (option, argument) */
+struct opt_optarg
 {
-  int c;
+  int option;
+  char *optarg;
+  struct opt_optarg *next;
+};
+
+int
+a2ps_handle_options (a2ps_job * job, int argc, char *argv[])
+{
+  struct opt_optarg *opt_optarg_head = NULL;
+  struct opt_optarg *opt_optarg = NULL;
+  int res;
 
   /* Reset optind so that getopt is reinitialized. */
   optind = 0;
-  
+
   while (1)
     {
       /* getopt_long stores the index of the option here. */
       int option_index = 0;
+      int option;
+      struct opt_optarg *new;
 
-      c = getopt_long (argc, argv, OPT_STRING, long_options, &option_index);
+      option = getopt_long (argc, argv,
+			    OPT_STRING, long_options, &option_index);
 
-      if (c == EOF)
+      if (option == '?')
+	{
+	  /* Error message is done by getopt */
+	  fprintf (stderr,
+		   _("Try `%s --help' for more information.\n"),
+		   program_invocation_name);
+	  exit (EXIT_BADARG);
+	}
+
+      /* Store the pair. */
+      new = ALLOCA (struct opt_optarg, 1);
+      new->option = option;
+      new->optarg = optarg;
+      new->next = NULL;
+      if (!opt_optarg)
+	{
+	  opt_optarg_head = new;
+	  opt_optarg = opt_optarg_head;
+	}
+      else
+	{
+	  opt_optarg->next = new;
+	  opt_optarg = opt_optarg->next;
+	}
+
+      /* Exit at first non option. */
+      if (option == EOF)
         break;
+    }
 
-      /* 
+  /* When processing the options, getopt can be called (e.g., user
+     options), so the global var optind (which specifies the rank of
+     the first non option in argv) *will* be changed.  We want to
+     return its original value, not the one which can be defined here
+     after. */
+  res = optind;
+
+  /* Now process the options. */
+  for (opt_optarg = opt_optarg_head ;
+       opt_optarg->option != EOF ;
+       opt_optarg = opt_optarg->next)
+    {
+      optarg = opt_optarg->optarg;
+
+      /*
        * First the option is passed to the application, in case
        * the application wants to redefine some options
        */
-      if (handle_option_hook && handle_option_hook (c, optarg))
+      if (handle_option_hook
+	  && handle_option_hook (opt_optarg->option, optarg))
 	/* The option has been handled, continue processing */
 	continue;
 
       /* The option has not been recognized by the application,
        * let the lib handle it */
-      switch (c) {
+      switch (opt_optarg->option) {
 #define define_nup(_col_,_row_,_orient_)	\
 	job->columns = _col_;		\
 	job->rows = _row_;		\
@@ -338,23 +393,15 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case '=':				/* A user option */
-	{
-	  /* Caution with recursive calls to getopt.
-	   * Actually, this is not enough: there is an error if the
-	   * the user option if after the files.  I don't understand
-	   * why (getopt internals) nor see what to do. */
-	  int saved_optind = optind;
-	  a2ps_handle_string_options (job, user_option_get (job, optarg));
-	  optind = saved_optind;
-	}
+	a2ps_handle_string_options (job, user_option_get (job, optarg));
 	break;
 
-      case 'a':			/* --pages= pages to print */
+      case 'a':				/* --pages= pages to print */
 	a2ps_page_range_set_string (job, optarg);
 	break;
 
-      case 'A':				/* allow two files per sheet */
-	job->compact_mode = TRUE;
+      case 'A':				/* --file-align=MODE. */
+	job->file_align = file_align_argmatch ("--file-align", optarg);
 	break;
 
       case 'b':				/* sheet header value */
@@ -387,38 +434,50 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 151:				/* cut lines too large */
-	job->folding = (a2ps_get_boolean ("--truncate-lines", optarg)
-			 ? FALSE : TRUE);
+	job->folding = a2ps_get_boolean ("--truncate-lines", optarg);
 	break;
 
       case 'C':				/* line numbering */
 	job->numbering = 5; /* Default is 5 by 5 */
 	break;
 
-      case 152:				/* line numbering */
-	job->numbering = 
-	  get_integer_in_range ("--line-numbers", optarg, 0, 0, range_min);
+      case 152:				/* --line-number[=INTERVAL] */
+	/* Alex would like a default of 1. */
+	job->numbering = (optarg
+			  ? get_integer_in_range ("--line-numbers", optarg,
+						  0, 0, range_min)
+			  : 1);
 	break;
 
-      case 'd':				/* fork a process to print */ 
-	a2ps_printers_output_set (job->printers, NULL, true);
+      case 'd':				/* fork a process to print */
+	a2ps_printers_flag_output_set (job->printers, NULL, true);
 	break;
 
-      case 'D': 			/* --setpagedevice		*/
+      case 'S': 			/* -S, --setpagedevice */
 	{
 	  char *value;
-	  value = strchr (optarg, ':');
-	  if (IS_EMPTY (value)) {
-	    delpagedevice (job, optarg);
-	  } else {
-	    *value = NUL;
-	    value ++;
-	    setpagedevice (job, optarg, value);
-	  }
+	  value = optarg + strcspn (optarg, " \t:=");
+	  /* If `-SFeature:' then remove Feature,
+	     If `-SFeature', pass the empty value to libppd,
+	     it may decide the right thing to do,
+	     otherwise (-SFeature:value) pass to libppd */
+	  if (*value)
+	    {
+	      *value = NUL;
+	      value ++;
+	      if (*value)
+		setpagedevice (job, optarg, value);
+	      else
+		delpagedevice (job, optarg);
+	    }
+	  else
+	    {
+	      setpagedevice (job, optarg, NULL);
+	    }
 	}
       break;
 
-      case 'f': 
+      case 'f':
 	{
 	  char * cp;
 	  /* This is for compatibility with the previous scheme */
@@ -428,10 +487,10 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	  else
 	    cp = optarg;
 	  /* A font size is given */
-	  job->fontsize = get_length ("--font-size", cp, 
+	  job->fontsize = get_length ("--font-size", cp,
 				      0.0, 0.0, "pt", range_min_strict);
 	  job->columns_requested = 0;
-	  job->lines_requested = 0;	  
+	  job->lines_requested = 0;
 	}
       break;
 
@@ -461,7 +520,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 
       case 'l':
 	/* set columns per line, useful for most cases */
-	job->columns_requested = 
+	job->columns_requested =
 	  get_integer_in_range ("--chars-per-line", optarg, 1, 0, range_min);
 	job->lines_requested = 0;
 	break;
@@ -469,7 +528,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
       case 'L':
 	/* set lines per page.  Useful with preformatted files. Scaling is
 	 * automatically done when necessary.  */
-	job->lines_requested = 
+	job->lines_requested =
 	  get_integer_in_range ("--lines-per-page", optarg, 1, 0, range_min);
 	/* Unset value given to columns-per-page, so that this one
 	 * is not hidden */
@@ -482,24 +541,26 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 'M':                 		/* select a medium */
-	job->medium = a2ps_get_medium (job, optarg);
+	xstrcpy (job->medium_request, optarg);
 	break;
 
       case 'n':				/* n copies */
-	job->copies = 
+	job->copies =
 	  get_integer_in_range ("--copies", optarg, 1, 0, range_min);
 	break;
 
       case 'o':			/* output goes into a file */
-	a2ps_printers_output_set (job->printers, optarg, false);
+	a2ps_printers_flag_output_set (job->printers, optarg, false);
 	break;
 
-      case 'P':					/* fork a process to print */ 
-	a2ps_printers_output_set (job->printers, optarg, true);
+      case 'P':					/* fork a process to print */
+	a2ps_printers_flag_output_set (job->printers, optarg, true);
 	break;
 
       case 'q':			       /* don't say anything but errors */
-	message_verbosity = msg_null;
+	/* Respect the envvar A2PS_VERBOSITY. */
+	if (!getenv ("A2PS_VERBOSITY"))
+	  msg_verbosity = msg_null;
 	break;
 
       case 'r':
@@ -511,34 +572,45 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 's':
-	switch (get_integer_in_range ("--sides", optarg,
-				      1, 2, range_min_max)) {
-	case 1:
-	  job->duplex = 0;
-	  delpagedevice (job, "Duplex");
-	  break;
-	  
-	case 2:
-	  job->duplex = 1;
-	  setpagedevice (job, "Duplex", "true");
-	  break;
-	}
+	/* I'd like to have this test be run at compile time, but how? */
+	ARGMATCH_ASSERT (duplex_args, duplex_types);
+	job->duplex = XARGCASEMATCH ("--sides", optarg,
+				     duplex_args, duplex_types);
+	switch (job->duplex)
+	  {
+	  case simplex:
+	    delpagedevice (job, "Duplex");
+	    break;
+
+	  case duplex:
+	    setpagedevice (job, "Duplex", "true");
+	    setpagedevice (job, "Tumble", "false");
+	    break;
+
+	  case tumble:
+	    setpagedevice (job, "Duplex", "true");
+	    setpagedevice (job, "Tumble", "true");
+	    break;
+	  }
 	break;
 
-      case 'S':				/* statusdict definitions */
+      case 164:				/* statusdict definitions */
 	{
 	  char *value;
 	  value = strchr (optarg, ':');
-	  if (IS_EMPTY (value)) {
-	    delstatusdict (job, optarg);
-	  } else {
-	    *value = NUL;
-	    value ++;
-	    if (*value == ':')
-	       setstatusdict (job, optarg, value + 1, TRUE);
-	     else
-	       setstatusdict (job, optarg, value, FALSE);
-	  }
+	  if (IS_EMPTY (value))
+	    {
+	      delstatusdict (job, optarg);
+	    }
+	  else
+	    {
+	      *value = NUL;
+	      value ++;
+	      if (*value == ':')
+		setstatusdict (job, optarg, value + 1, TRUE);
+	      else
+		setstatusdict (job, optarg, value, FALSE);
+	    }
 	}
 	break;
 
@@ -547,7 +619,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 'T':
-	job->tabsize = 
+	job->tabsize =
 	  get_integer_in_range ("--tabsize", optarg, 1, 0, range_min);
 	break;
 
@@ -556,11 +628,18 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 'v':					    /* verbosity */
-	if (!IS_EMPTY(optarg)) {
-	  message_verbosity = 
-	    get_integer_in_range ("--verbose", optarg, 0, 0, range_min);
-	} else
-	  message_verbosity = msg_report1;
+	/* If the user specified the verbosity level by envvar,
+	   don't override it. */
+	if (!getenv ("A2PS_VERBOSITY"))
+	    {
+	      if (!IS_EMPTY(optarg))
+		{
+		  msg_verbosity =
+		    msg_verbosity_argmatch ("--verbose", optarg);
+		}
+	      else
+		msg_verbosity = msg_report1;
+	    }
 	break;
 
       case 'X': 			/* change the encoding scheme */
@@ -578,7 +657,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 132:				/* Number of columns */
-	job->columns = 
+	job->columns =
 	  get_integer_in_range ("--columns", optarg, 1, 0, range_min);
 	break;
 
@@ -592,27 +671,38 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 135:				/* --non-printable-format */
-	job->unprintable_format = 
-	  get_non_printable_format ("--non-printable", optarg);
+	ARGMATCH_ASSERT (non_printable_args, non_printable_types);
+	job->unprintable_format =
+	  XARGCASEMATCH ("--non-printable", optarg,
+			 non_printable_args, non_printable_types);
 	break;
 
       case 136:				/* --print-anyway=bool */
 	job->print_binaries = a2ps_get_boolean ("--print-anyway", optarg);
 	break;
 
-      case 137:			/* --macro-meta-sequence=key:value */
+      case 'D':				/* --define=key:value */
 	{
 	  char *value;
-	  value = strchr (optarg, ':');
-	  if (value) {
-	    *value = NUL;
-	    value ++;
-	    if (!macro_meta_sequence_add (job, optarg, value))
-	      error (1, 0, 
-		     _("invalid macro identifier `%s'"), optarg);
-	  } else {
-	    macro_meta_sequence_delete (job, optarg);
-	  } 
+	  value = optarg + strcspn (optarg, " \t:=");
+	  /* A value is given if there is a separator.  The value can
+	     then be empty (`--var=foo=' i.e. foo = "") which is not
+	     the same thing as --var=foo (i.e., undefine the var foo) */
+	  if (*value)
+	    *value++ = NUL;
+	  else
+	    value = NULL;
+	  if (value)
+	    {
+	      if (!macro_meta_sequence_add (job, optarg, value))
+		error (1, 0,
+			       _("invalid variable identifier `%s'"),
+			       quotearg (optarg));
+	    }
+	  else
+	    {
+	      macro_meta_sequence_delete (job, optarg);
+	    }
 	}
 	break;
 
@@ -652,16 +742,12 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	  job->margin = 12;
 	break;
 
-      case 150:				/* allow two files per sheet */
-	job->compact_mode = a2ps_get_boolean ("--compact", optarg);
-	break;
-
       case 157: 			/* --major= */
 	job->madir = madir_argmatch ("--major", optarg);
 	break;
 
       case 158:				/* --version-control */
-	job->backup_type = a2ps_get_backup_type (optarg);
+	job->backup_type = get_version ("--version-control", optarg);
 	break;
 
       case 159:				/* --suffix	*/
@@ -677,94 +763,53 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case '?':				/* Unknown option */
-	/* Error message is done by getopt */
-	fprintf (stderr,
-		 _("Try `%s --help' for more information.\n"), program_name);
-	exit (EXIT_BADARG);
+	/* Should never happen, since getopt was called in the previous
+	   loop. */
+	abort ();
 	break;
       }
     }
-  return optind;
-}
 
+  return res;
+}
+
+/* Process STRING as if it were given on the command line, i.e.  make
+   it an argument vector, while respecting shell-escape and
+   shell-grouping mechanics (" and '), and them run
+   a2ps_handle_options onto the arguement vector.
+
+   Return optind, i.e., the index of the the first non option argument.
+   But I doubt arguments are allowed in user options.
+*/
 int
 a2ps_handle_string_options (a2ps_job * job, const char * string)
 {
   int argc;
   char **argv;
-  char *str;
-  int i, res;
+  int res;
 
   if (string == NULL)
     return 0;
 
   message (msg_opt, (stderr, "handle_string_options(%s)", string));
 
-  /* Alloca-copy string so we can modify it in place. */
-  astrcpy (str, string);
+  /* Build the arguments vector, and add program name */
+  argc = 1;
+  argv = buildargv_argc (string, &argc);
+  argv[0] = program_name;
 
-  /*
-   * We can count this, each option takes at least 1 character and one 
-   * space.  We also need one for program's name and one for the 
-   * trailing NULL. 
-   */
-  argc = (strlen (str) + 1) / 2 + 2;
-  argv = (char **) CALLOC (char *, argc);	/* FIXME: alloca? */
-  
-  /* Set program name. */
-  argc = 0;
-  argv[argc++] = program_name;
-
-  /* Split string and set arguments to argv array. */
-  i = 0;
-  while (str[i])
+  if (msg_test (msg_opt))
     {
-      /* Skip leading whitespace. */
-      for (; str[i] && isspace ((int) str[i]); i++)
-	;
-      if (!str[i])
-	break;
-
-      /* Check for quoted arguments. */
-      if (str[i] == '"' || str[i] == '\'')
-	{
-	  int endch = str[i++];
-
-	  argv[argc++] = str + i;
-
-	  /* Skip until we found the end of the quotation. */
-	  for (; str[i] && str[i] != endch; i++)
-	    ;
-	  if (!str[i])
-	    error (1, 0, _("syntax error in option string `%s':\n\
-missing end of quotation: %c"), string, endch);
-
-	  str[i++] = '\0';
-	}
-      else
-	{
-	  argv[argc++] = str + i;
-
-	  /* Skip until whitespace if found. */
-	  for (; str[i] && !isspace ((int) str[i]); i++)
-	    ;
-	  if (str[i])
-	    str[i++] = '\0';
-	}
+      int i;
+      for (i = 0; i < argc; i++)
+        fprintf (stderr, "   %3d = `%s'\n", i, argv[i]);
     }
-  
-  /* argv[argc] must be NULL. */
-  argv[argc] = NULL;
-
-  message (msg_opt, (stderr, " (argc=%d):\n", argc));
-  for (i = 0; i < argc; i++)
-    message (msg_opt, (stderr, "   %3d = `%s'\n", i, argv[i]));
 
   /* Process options. */
-  res = a2ps_handle_option (job, argc, argv);
+  res = a2ps_handle_options (job, argc, argv);
 
   /* Cleanup. */
-  XFREE (argv);
+  freeargv_from (argv, 1);
 
   return res;
 }
