@@ -1,15 +1,15 @@
 /*
  * options.h
  *
- * options handling & (dynamic) configuration of a2ps.
+ * Read and understanding everything about the options
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -26,14 +26,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _OPTIONS_H_
-#define _OPTIONS_H_
-
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
-#include "backupfile.h"
+#ifndef OPTIONS_H_
+# define OPTIONS_H_
 
 struct a2ps_job;
 
@@ -41,20 +35,20 @@ struct a2ps_job;
  * Type of the functions for options hooks
  */
 typedef int (* option_hook) PARAMS ((int opt, char * arg));
+
 extern option_hook handle_option_hook;
 
-/*
- * Handle options' arguments
- */
-enum backup_type a2ps_get_backup_type PARAMS ((const char * word));
-boolean a2ps_get_boolean PARAMS ((const char * option, const char * arg));
+/* Handle arguments */
+boolean a2ps_get_boolean
+  PARAMS ((const char * option, const char * arg));
 
-/*
- * Handle the options.
- */
-int a2ps_handle_option PARAMS ((struct a2ps_job * job,
-				int argc, char *argv[]));
-int a2ps_handle_string_options PARAMS ((struct a2ps_job * job,
-					const char *string));
+/* Handle options.  Returns the indice of the first element of ARGV
+   which is not an option. */
 
-#endif
+int a2ps_handle_options
+   PARAMS ((struct a2ps_job * job, int argc, char *argv[]));
+
+int a2ps_handle_string_options
+   PARAMS ((struct a2ps_job * job, const char *string));
+
+#endif /* !OPTIONS_H_ */
