@@ -3,13 +3,13 @@
  *
  * routines for ram-diverted output
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1995, 96, 97, 98, 99 Akim Demaille, Miguel Santana
  * $Id: output.c,v 1.77 1998/03/04 17:27:02 demaille Exp $
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -34,25 +34,28 @@
 #include "darray.h"
 #include "dstring.h"
 #include "printers.h"
-#include "msg.h"
+#include "message.h"
 #include "xstdarg.h"
 #include "dsc.h"
 #include "stream.h"
 #include "metaseq.h"
+#include "quotearg.h"
 
 #define MIN_CONTENT	1024 * 50 	/* 50 Kb buffer			*/
 #define CHUNK_MARGIN	1024		/* Be sure there is always that left */
 #define MIN_DERIVATIONS	10
 
-enum derivation_type {
-  nothing, 
-  delayed_int, 
-  delayed_string, 
-  delayed_routine, 
+enum derivation_type
+{
+  nothing,
+  delayed_int,
+  delayed_string,
+  delayed_routine,
   delayed_chunk
 };
 
-struct derivation {
+struct derivation
+{
   enum derivation_type type;
   void * arg;
   delayed_routine_t delayed_routine;
@@ -62,7 +65,8 @@ struct derivation {
   struct output * delayed_chunk;
 };
 
-struct output {
+struct output
+{
   const char * name;
   struct dstring * chunk;
   struct darray * derivations;
@@ -82,31 +86,32 @@ static void
 derivation_self_print (struct derivation * derivation, FILE * stream)
 {
   fprintf (stream, "At %x: ", (int) derivation);
-  switch (derivation->type) {
-  case nothing:
-    fprintf (stream, "nothing ");
-    break;
+  switch (derivation->type)
+    {
+    case nothing:
+      fprintf (stream, "nothing ");
+      break;
 
-  case delayed_int:
-    fprintf (stream, "delayed_int (%d)", *derivation->delayed_int);
-    break;
+    case delayed_int:
+      fprintf (stream, "delayed_int (%d)", *derivation->delayed_int);
+      break;
 
-  case delayed_string:
-    fprintf (stream, "delayed_string ");
-    if (*derivation->delayed_string)
-      fprintf (stderr, "(%s)", *derivation->delayed_string);
-    else 
-      fprintf (stderr, "##BROKEN##");
-    break;
+    case delayed_string:
+      fprintf (stream, "delayed_string ");
+      if (*derivation->delayed_string)
+	fprintf (stderr, "(%s)", *derivation->delayed_string);
+      else
+	fprintf (stderr, "##BROKEN##");
+      break;
 
-  case delayed_routine:
-    fprintf (stream, "delayed_routine ");
-    break;
+    case delayed_routine:
+      fprintf (stream, "delayed_routine ");
+      break;
 
-  case delayed_chunk:
-    fprintf (stream, "delayed_chunk ");
-    break;
-  }
+    case delayed_chunk:
+      fprintf (stream, "delayed_chunk ");
+      break;
+    }
 }
 
 /************************************************************************/
@@ -119,7 +124,7 @@ struct output *
 output_new (const char * name)
 {
   struct output * res = NULL;
-  
+
   res = ALLOC (struct output, 1);
   res->name = name;
   res->chunk = ds_new (MIN_CONTENT, ds_geometrical, 2);
@@ -128,7 +133,7 @@ output_new (const char * name)
 			    (da_print_func_t) derivation_self_print,
 			     NULL);
   res->to_void = FALSE;
-  
+
   return res;
 }
 
@@ -202,7 +207,7 @@ output (out, format, va_alist)
   ds_unsafe_cat_vsprintf (out->chunk,
 			 format, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif
-} 
+}
 
 /*
  * Add C to the end of output
@@ -220,12 +225,12 @@ output_char (struct output * out, uchar c)
  * the routines is called with job as arg, at dump time
  */
 void
-output_delayed_routine (struct output * out, 
+output_delayed_routine (struct output * out,
 			delayed_routine_t fn,
 			void * fn_arg)
 {
   struct derivation *tmp;
- 
+
   if (out->to_void)
     return;
 
@@ -241,7 +246,7 @@ void
 output_delayed_chunk (struct output * out, struct output * out2)
 {
   struct derivation *tmp;
- 
+
   if (out->to_void)
     return;
 
@@ -291,16 +296,16 @@ output_delayed_string (struct output * out, uchar ** ptr)
 /* FIXME: Must be robust to missing arguments */
 #define GET_TOKEN(from) (strtok ((from), " \t\n"))
 #define GET_LINE_TOKEN(from) (strtok ((from), "\n"))
- 
+
 #define CHECK_TOKEN() 							\
   if (token2 == NULL) 							\
     error_at_line (1, 0, filename, line, 				\
-		   _("missing argument for `%s'"), token);
+		   _("missing argument for `%s'"), quotearg (token));
 /*
  * Dump a library file content
  */
 void
-output_file (struct output * out, a2ps_job * job, 
+output_file (struct output * out, a2ps_job * job,
 	     const char *name, const char *suffix)
 {
   char buf[512];
@@ -318,7 +323,7 @@ output_file (struct output * out, a2ps_job * job,
 
   filename = xpw_find_file (job->common.path, name, suffix);
   stream = xrfopen (filename);
-  
+
   /* Find the end of the header. */
 #define HDR_TAG "% -- code follows this line --"
   while ((fgets (buf, sizeof (buf), stream))) {
@@ -326,7 +331,7 @@ output_file (struct output * out, a2ps_job * job,
     if (strprefix (HDR_TAG, buf))
       break;
   }
-  
+
   /* Dump rest of file. */
   while ((fgets (buf, sizeof (buf), stream))) {
     line++;
@@ -349,23 +354,23 @@ output_file (struct output * out, a2ps_job * job,
 #if 0
 	char * fontname;
 	font_t font;
-	
+
 	fontname =  buf + strlen (FONTUSED_TAG);
 	/* What is the face declared? */
 	fontname = strtok (fontname, " \t\n");
 	font = string_to_font (fontname);
-	
+
 	if (font == unknown_font)
 	  error_at_line (1, 0, filename, line,
-			 _("unrecognized font: `%s'"), fontname);
+			 "unknown font `%s'", quotearg (fontname));
 	if (!(job->status->font_used [font]))
 	  dont_output = TRUE;
 #endif
 	continue;
       }
-    
+
 #define INCL_RES_TAG 	"%%IncludeResource:"
-    if (strprefix (INCL_RES_TAG, buf)) 
+    if (strprefix (INCL_RES_TAG, buf))
       {
 	char * value, * res, * buf2;
 	buf2 = xstrdup (buf);
@@ -379,14 +384,15 @@ output_file (struct output * out, a2ps_job * job,
 	    token2 = GET_TOKEN (NULL);
 	    CHECK_TOKEN ();
 	    value = token2;
-	    if (!exist_resource (job, res, value)) {
-	      add_needed_resource (job, res, value);
-	      message (msg_file, 
-		       (stderr, 
-			"Including file '%s' upon request given in '%s':%d\n", 
-			value, filename, line));
-	      output_file (dest, job, value, NULL);
-	    }
+	    if (!exist_resource (job, res, value))
+	      {
+		add_needed_resource (job, res, value);
+		message (msg_file,
+			 (stderr,
+			  "Including file '%s' upon request given in '%s':%d\n",
+			  value, filename, line));
+		output_file (dest, job, value, NULL);
+	      }
 	  }
 	else
 	  {
@@ -399,7 +405,7 @@ output_file (struct output * out, a2ps_job * job,
 	continue;
       }
 #define COLOR_TAG 	"%%DocumentProcessColors:"
-    else if (strprefix (COLOR_TAG, buf)) 
+    else if (strprefix (COLOR_TAG, buf))
       {
 	token = GET_TOKEN (buf);
 	token2= GET_TOKEN (NULL);
@@ -433,7 +439,7 @@ output_file (struct output * out, a2ps_job * job,
 	dest = out;
 	continue;
       }
-    
+
 #define SETUP_TAG 	"%%BeginSetup"
     else if (strprefix (SETUP_TAG, buf))
       {
@@ -444,8 +450,7 @@ output_file (struct output * out, a2ps_job * job,
 #define FACE_TAG	 "%Face:"
     else if (strprefix (FACE_TAG, buf))
       {
-	/* FIXME: We must make a symbol table between
-	 * fonts and faces */
+	/* FIXME: We must make a symbol table between fonts and faces. */
 	char * fontname;
 	enum face_e face;
 
@@ -456,17 +461,16 @@ output_file (struct output * out, a2ps_job * job,
 	CHECK_TOKEN ();
 	face = string_to_face (token2);
 	if (face == No_face)
+	  /* TRANS: a face is a virtual `font', for instance Keyword,
+	     or Comment_strong, or String are faces. */
 	  error_at_line (1, 0, filename, line,
-			 /* TRANS: a face is a virtual `font', for
-			    instance Keyword, or Comment_strong, or
-			    String are faces */
-			 _("unrecognized face: `%s'"), token2);
+			 _("invalid face `%s'"), quotearg (token2));
 
 	/* What is the corresponding physical font? */
 	token2 = GET_TOKEN (NULL);
 	CHECK_TOKEN ();
 	fontname = token2;
-	
+
 	/* Bind font to face */
 	face_set_font (job, face, fontname);
 
@@ -477,6 +481,7 @@ output_file (struct output * out, a2ps_job * job,
 		fontname, token2);
 	continue;
       }
+
 #define FONT_TAG	 "%Font:"
     else if (strprefix (FONT_TAG, buf))
       {
@@ -484,7 +489,7 @@ output_file (struct output * out, a2ps_job * job,
 	const char * true_font_name;
 
 	token = GET_TOKEN (buf);
-	
+
 	/* What is the asked font name (before subsitution)?
 	 * Register the corresponding font */
 	token2 = GET_TOKEN (NULL);
@@ -501,13 +506,13 @@ output_file (struct output * out, a2ps_job * job,
 	 * We don't want to read its AFM file: no use of the
 	 * WX.  Just put it in the DSC storage, that will decide
 	 * whether including the font's file, or not */
-	true_font_name = 
+	true_font_name =
 	  encoding_resolve_font_substitute (job,
 					    job->requested_encoding,
 					    basefontname);
 	add_required_font (job, true_font_name);
 
-	
+
 	output (dest, "  f%s %s scalefont setfont\n", basefontname, token2);
 	continue;
       }
@@ -517,14 +522,14 @@ output_file (struct output * out, a2ps_job * job,
 	char * expansion;
 
 	token = GET_LINE_TOKEN (buf + strlen (EXPAND_TAG));
-	expansion = ((char *) 
-		     expand_user_string (job, FIRST_FILE (job), 
+	expansion = ((char *)
+		     expand_user_string (job, FIRST_FILE (job),
 					 (const uchar *) "Expand: requirement",
 					 (const uchar *) token));
 	output (dest, expansion);
 	continue;
       }
-    
+
     output (dest, "%s", buf);
   }
 
@@ -533,7 +538,7 @@ output_file (struct output * out, a2ps_job * job,
     error_at_line (1, 0, filename, line,
 		   _("`%s' with no matching `%s'"),
 		   SETUP_TAG, END_SETUP_TAG);
-  
+
   fclose (stream);
   FREE (filename);
 }
@@ -544,30 +549,31 @@ output_file (struct output * out, a2ps_job * job,
 inline static void
 underivation (FILE * stream, const struct derivation * derivation)
 {
-  switch (derivation->type) {
-  case delayed_routine:
-    derivation->delayed_routine (stream,
-				 derivation->delayed_routine_arg);
-    break;
-    
-  case delayed_int:
-    fprintf (stream, "%d", *derivation->delayed_int);
-    break;
-    
-  case delayed_string:
-    fprintf (stream, "%s", *derivation->delayed_string);
-    break;
-    
-  case delayed_chunk:
-    output_dump (derivation->delayed_chunk, stream);
-    break;
-    
-  case nothing:
-    break;
-    
-  default:
-    error (1, 0, "undivert");
-  }
+  switch (derivation->type)
+    {
+    case delayed_routine:
+      derivation->delayed_routine (stream,
+				   derivation->delayed_routine_arg);
+      break;
+
+    case delayed_int:
+      fprintf (stream, "%d", *derivation->delayed_int);
+      break;
+
+    case delayed_string:
+      fprintf (stream, "%s", *derivation->delayed_string);
+      break;
+
+    case delayed_chunk:
+      output_dump (derivation->delayed_chunk, stream);
+      break;
+
+    case nothing:
+      break;
+
+    default:
+      abort ();
+    }
 }
 
 /*
@@ -590,18 +596,19 @@ output_dump (struct output * out, FILE * stream)
   char * piece = out->chunk->content;
   struct derivation ** derivations =
     (struct derivation **) out->derivations->content;
-  
-  if (message_test (msg_tool))
+
+  if (msg_test (msg_tool))
     output_report (out, stderr);
 
   fputs ((char *) piece, stream);
   piece += ustrlen (piece);
-  for (i = 0 ; i < out->derivations->len ; i++) {
-    underivation (stream, derivations [i]);
-    piece ++;
-    fputs ((char *) piece, stream);
-    piece += ustrlen (piece);
-  }
+  for (i = 0 ; i < out->derivations->len ; i++)
+    {
+      underivation (stream, derivations [i]);
+      piece ++;
+      fputs ((char *) piece, stream);
+      piece += ustrlen (piece);
+    }
 }
 
 /*
@@ -613,7 +620,7 @@ undivert (a2ps_job * job)
 {
   /* Open the destination */
   a2ps_open_output_stream (job);
-  
+
   /* Dump the PostScript and close */
   output_dump (job->divertion, job->output_stream->fp);
 
