@@ -19,14 +19,14 @@
 /*
  * Grammar for parsing the style sheets
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  *
  */
 
 /*
  * This file is part of a2ps
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -44,36 +44,33 @@
  */
 
 /*
- * $Id: parseppd.c,v 1.2 1998/03/02 09:50:57 demaille Exp $
+ * $Id: parseppd.y,v 1.11 1998/03/02 08:57:07 demaille Exp $
  */
 
 #include "a2ps.h"
 #include "jobs.h"
 #include "ppd.h"
-#include "msg.h"
+#include "message.h"
 #include "routines.h"
-#include "yy2ppd.h"
+
+/* We need to use the same `const' as bison, to avoid the following
+   prototypes to diverge from the function declarations */
+#undef const
+#ifndef __cplusplus
+# ifndef __STDC__
+#   define const
+# endif
+#endif
+
+#include "lexppd.h"
 
 #define YYDEBUG 1
 #define YYERROR_VERBOSE 1
 #define YYPRINT(file, type, value)   yyprint (file, type, value)
 
-/* We need to use the same `const' as bison, to avoid the following
- * prototypes to diverge from the function declarartions */
-#undef const
-#ifndef __cplusplus
-#ifndef __STDC__
-#define const
-#endif
-#endif
-
-/* Comes from the lexer */
-extern int ppdlineno;
-
 /* Comes from the caller */
 extern FILE * ppdin;
 extern struct a2ps_job * job;
-const char * ppdfilename;
 
 /* Local prototypes */
 void yyerror PARAMS ((const char *msg));
@@ -83,11 +80,11 @@ static void yyprint ();
 void ppdlex_initialize PARAMS ((void));
 
 int yylex PARAMS ((void));
-struct ppd * a2ps_ppd_parse PARAMS ((const char * filename));
+
 static struct ppd * ppd_parse_result;
 
 
-#line 73 "parseppd.y"
+#line 70 "parseppd.y"
 typedef union
 {
   char * string;
@@ -162,8 +159,8 @@ static const short yyrhs[] = {    13,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    94,   102,   107,   112,   118,   124,   128,   132,   136,   140,
-   149,   152
+    91,    99,   104,   109,   115,   121,   125,   129,   133,   137,
+   146,   149
 };
 #endif
 
@@ -720,77 +717,77 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 96 "parseppd.y"
-{ 
+#line 93 "parseppd.y"
+{
       ppd_parse_result = yyvsp[0].ppd;
       /* Nothing Right Now */;
     ;
     break;}
 case 2:
-#line 104 "parseppd.y"
+#line 101 "parseppd.y"
 {
       yyval.ppd = ppd_new ();
     ;
     break;}
 case 3:
-#line 108 "parseppd.y"
-{ 
+#line 105 "parseppd.y"
+{
       ppd_font_add (yyvsp[-2].ppd, yyvsp[-1].string);
       yyval.ppd = yyvsp[-2].ppd;
     ;
     break;}
 case 4:
-#line 113 "parseppd.y"
-{ 
+#line 110 "parseppd.y"
+{
       if (!yyvsp[-4].ppd->modelname)
 	yyvsp[-4].ppd->modelname = yyvsp[-1].string;
       yyval.ppd = yyvsp[-4].ppd;
     ;
     break;}
 case 5:
-#line 119 "parseppd.y"
-{ 
+#line 116 "parseppd.y"
+{
       if (!yyvsp[-4].ppd->nickname)
 	yyvsp[-4].ppd->nickname = yyvsp[-1].string;
       yyval.ppd = yyvsp[-4].ppd;
     ;
     break;}
 case 6:
-#line 125 "parseppd.y"
+#line 122 "parseppd.y"
 {
       yyval.ppd = yyvsp[-1].ppd;
     ;
     break;}
 case 7:
-#line 129 "parseppd.y"
+#line 126 "parseppd.y"
 {
       yyval.ppd = yyvsp[-1].ppd;
     ;
     break;}
 case 8:
-#line 133 "parseppd.y"
+#line 130 "parseppd.y"
 {
       yyval.ppd = yyvsp[-1].ppd;
     ;
     break;}
 case 9:
-#line 137 "parseppd.y"
+#line 134 "parseppd.y"
 {
       yyval.ppd = yyvsp[-1].ppd;
     ;
     break;}
 case 10:
-#line 141 "parseppd.y"
+#line 138 "parseppd.y"
 {
       yyval.ppd = yyvsp[-1].ppd;
     ;
     break;}
 case 11:
-#line 151 "parseppd.y"
+#line 148 "parseppd.y"
 { yyval.string= yyvsp[-5].string; ;
     break;}
 case 12:
-#line 153 "parseppd.y"
+#line 150 "parseppd.y"
 { yyval.string= yyvsp[0].string; ;
     break;}
 }
@@ -991,7 +988,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 155 "parseppd.y"
+#line 152 "parseppd.y"
 
 
 void
@@ -1022,18 +1019,20 @@ yyprint (FILE *file, int type, YYSTYPE value)
 }
 
 struct ppd *
-a2ps_ppd_parse (const char * filename)
+a2ps_ppd_parse (const char * filename, char * const * path)
 {
-  ppdfilename = filename;
+  /* The filename won't be changed. */
+  ppdfilename = (char *) filename;
   ppdlineno = 1;
   ppdin = xrfopen (ppdfilename);
+  ppdpath = path;
 
   message (msg_file | msg_ppd | msg_parse,
 	   (stderr, "Parsing file `%s'\n", ppdfilename));
 
   ppdlex_initialize ();
 
-  if (message_test (msg_parse))
+  if (msg_test (msg_parse))
     yydebug = TRUE;
   else
     yydebug = FALSE;
