@@ -1,9 +1,9 @@
 /*
  * pathwalk.c
  *
- * functions for ooking for files, reading files etc. 
+ * functions for ooking for files, reading files etc.
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1995, 96, 97, 98, 99 Akim Demaille, Miguel Santana
  * $Id: pathwalk.c,v 1.34 1998/03/03 15:51:21 demaille Exp $
  */
 
@@ -27,9 +27,12 @@
 #include "a2ps.h"
 #include "pathwalk.h"
 #include "darray.h"
-#include "msg.h"
+#include "message.h"
 #include "routines.h"
 #include "filtdir.h"
+#include "lister.h"
+#include "strverscmp.h"
+#include "quotearg.h"
 
 /*
  * Build in a alloca'ed zone the concat of DIR and FILE
@@ -44,23 +47,6 @@
         (*_tmp1_++) = DIR_SEP;				\
         stpcpy (_tmp1_, _tmp2_);			\
    } while (0)
-
-/*
- * Path walk functions
- */
-typedef struct file_lookup_ctx_st {
-  char name[256];
-  char suffix[256];
-  char fullname[512];
-} FileLookupCtx;
-
-typedef struct file_suffix_ctx_st {
-  const char * dirfmt;
-  const char * filefmt;
-  const char * enddir;
-  char suffix[256];
-} FileSuffixCtx;
-
 
 /************************************************************************/
 /*	Handling the path: an array, NULL terminated of char *		*/
@@ -73,19 +59,19 @@ pw_internal_string_to_path (const char * path, char sep, int * length)
   int entries = 0;
   const char * cp, * cp2;
   int len;
-  
+
   res = CALLOC (char *, allocated);
   for (cp = path; cp; cp = strchr (cp, sep))
     {
       if (cp != path)
         cp++;
-      
+
       cp2 = strchr (cp, sep);
       if (cp2)
         len = cp2 - cp;
       else
         len = strlen (cp);
-      
+
       if (len == 0) {
 	/* Skip empty entries */
         cp++;
@@ -95,11 +81,11 @@ pw_internal_string_to_path (const char * path, char sep, int * length)
 	if (cp [len] == DIR_SEP)
 	  len --;
       }
-      
+
       res [ entries ] = MALLOC (len + 1);
       strncpy (res [entries],  cp, len);
       res [entries] [len] = '\0';
-      
+
       entries++;
       if (entries >= allocated) {
 	allocated *= 2;
@@ -223,15 +209,15 @@ pw_find_file_index (char * const * path,
   int i;
   struct stat stat_st;
   char * filename, * fullpath;
-  
+
   if (suffix)
     astrcat2 (filename, name, suffix);
   else
     filename = (char *) name;
 
-  if (message_test (msg_pw)) 
+  if (msg_test (msg_pw))
     {
-      message (msg_pw, 
+      message (msg_pw,
 	       (stderr, "Looking for `%s' in the path:\n", filename));
       pw_fprintf_path (stderr, "\t-> %s\n", path);
     }
@@ -247,7 +233,7 @@ pw_find_file_index (char * const * path,
 	    return i + 1;
 	  }
       }
-  
+
   message (msg_pw, (stderr, "failure\n"));
   return 0;
 }
@@ -268,7 +254,7 @@ pw_file_exists_p (char * const * path,
  */
 static inline char *
 _pw_find_file (char * const * path,
-	      const char * name, const char * suffix)
+	       const char * name, const char * suffix)
 {
   char * res;
   int i;
@@ -280,11 +266,11 @@ _pw_find_file (char * const * path,
       /* Return a malloc'ed full file name */
       if (suffix)
 	{
-	  res = MALLOC (strlen (path[i-1]) + 2 
+	  res = MALLOC (strlen (path[i-1]) + 2
 			+ strlen (name) + strlen (suffix));
 	  sprintf (res, "%s%c%s%s", path [i-1], DIR_SEP, name, suffix);
 	} else {
-	  res = MALLOC (strlen (path[i-1]) + 2 
+	  res = MALLOC (strlen (path[i-1]) + 2
 			+ strlen (name));
 	  sprintf (res, "%s%c%s", path [i-1], DIR_SEP, name);
 	}
@@ -304,27 +290,61 @@ pw_find_file (char * const * path,
   return _pw_find_file (path, name, suffix);
 }
 
-/*
- * Return the malloc'd full path of existing file
- * named concat(NAME, SUFFIX), exits on failure
- */
+/* Return the malloc'd full path of existing file named concat(NAME,
+   SUFFIX), exits on failure. */
+
 char *
 xpw_find_file (char * const * path,
-	      const char * name, const char * suffix)
+	       const char * name, const char * suffix)
 {
   char * res = _pw_find_file (path, name, suffix);
 
   if (!res)
-    error (1, errno, _("cannot find file `%s%s'"), name, UNNULL (suffix));
-
+    {
+      char *file;
+      file = ALLOCA (char, strlen (name) + (suffix ? strlen (suffix) : 0) + 1);
+      sprintf (file, "%s%s",  name, UNNULL (suffix));
+      error (1, errno, _("cannot find file `%s'"), quotearg (file));
+    }
   return res;
 }
+
+/* Idem, but look first around the given INCLUDING_FILE. */
+
+char *
+xpw_find_included_file (char * const *path,
+			const char *including_file,
+			const char *name, const char *suffix)
+{
+  char *dir;	/* Of the including file. */
+  char *res;
+  struct stat statbuf;
+
+  if (*name == DIR_SEP)
+    /* Path is absolute */
+    dir = NULL;
+  else
+    /* Relative.  Give its root. */
+    dir = dir_name (including_file);
+
+  res = ALLOCA (char, (strlen (dir)
+		       + strlen (name)
+		       + (suffix ? strlen (suffix) : 0)
+		       + 2));
+  sprintf (res, "%s%c%s%s", dir, DIR_SEP, name, suffix ? suffix : "");
+  XFREE (dir);
+  if (stat (res, &statbuf) == 0)
+    return xstrdup (res);
+
+  /* Find in the library. */
+  return xpw_find_file (path, name, suffix);
+}
 
 /*
  * Dump a library file content
  */
 int
-pw_paste_file (char * const * path, 
+pw_paste_file (char * const * path,
 	       const char * name, const char * suffix)
 {
   char buf[512];
@@ -338,7 +358,7 @@ pw_paste_file (char * const * path,
 
   if (!fullpath)
     return 0;
-  
+
   fp = fopen (fullpath, "r");
 
   if (fp == NULL)
@@ -363,11 +383,11 @@ pw_paste_file (char * const * path,
 	file = strtok (file, " \n\t");
 	message (msg_pw,
 		 (stderr,
-		  "Including file '%s' upon request given in '%s':%d\n", 
+		  "Including file '%s' upon request given in '%s':%d\n",
 		  file, fullpath, line));
 	if (!pw_paste_file (path, file, NULL))
 	  error_at_line (1, errno, fullpath, line,
-			 _("cannot find file `%s'"), file);
+			 _("cannot find file `%s'"), quotearg (file));
 	continue;
       }
       fputs (buf, stdout);
@@ -378,24 +398,11 @@ pw_paste_file (char * const * path,
   return 1;
 }
 
-/* Do a collated comparison of A and B.  */
-static int
-collated_compare (const char * a, const char * b)
-{
-  if (a == b)
-    return 0;
-  if (a == NULL)
-    return 1;
-  if (b == NULL)
-    return -1;
-  return strcoll (a, b);
-}
-
 /*
  * Call glob on PATTERN in each dir of PATH.
  * Return a malloc'd char ** (char * malloc'ed too)
  */
-struct darray * 
+struct darray *
 pw_glob (char * const * path, const char * pattern)
 {
   struct darray * res;
@@ -403,14 +410,14 @@ pw_glob (char * const * path, const char * pattern)
   res = da_new ("Dir entries", 20,
 		da_geometrical, 2,
 		(da_print_func_t) da_str_print,
-		(da_cmp_func_t) collated_compare);
-  
+		(da_cmp_func_t) strverscmp);
+
   for (/* Nothing */ ; *path ; path++)
     filterdir_pattern (*path, res, pattern);
 
   da_qsort (res);
   da_unique (res, (da_map_func_t) free);
-  
+
   return res;
 }
 
@@ -427,7 +434,7 @@ da_str_cut_suffix (char * string)
  * Return malloc'd array of malloc'ed char *
  * of the prefix part of the file having SUFFIX as suffix in PATH
  */
-struct darray * 
+struct darray *
 pw_glob_on_suffix (char * const * path, const char * suffix)
 {
   struct darray * res;
@@ -442,3 +449,19 @@ pw_glob_on_suffix (char * const * path, const char * suffix)
 
   return res;
 }
+
+/* Use lister to report on STREAM the list of files in PATH that end
+   by SUFFIX. */
+
+void
+pw_lister_on_suffix (FILE * stream, char * const * path, const char * suffix)
+{
+  struct darray * entries;
+
+  entries = pw_glob_on_suffix (path, suffix);
+  lister_fprint_vertical (NULL, stream,
+			  (void *) entries->content, entries->len,
+			  (lister_width_t) strlen,
+			  (lister_print_t) fputs);
+  da_free (entries, (da_map_func_t) free);
+}
