@@ -1,15 +1,15 @@
 /*
  * pathwalk.h
  *
- * functions for ooking for files, reading files etc. 
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * functions for ooking for files, reading files etc.
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  * $Id: pathwalk.h,v 1.12 1998/03/03 15:51:21 demaille Exp $
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -29,23 +29,23 @@
 #ifndef _PATHWALK_H_
 #define _PATHWALK_H_
 
-/* 
+/*
  * Build a path (as array null terminated) out of path as string
  * and the separator.
  */
 char ** pw_string_to_path PARAMS ((const char * string, char sep));
-char ** pw_append_string_to_path PARAMS ((char ** path1, 
+char ** pw_append_string_to_path PARAMS ((char ** path1,
 					  const char * dir2, char sep));
-char ** pw_prepend_string_to_path PARAMS ((char ** path1, 
+char ** pw_prepend_string_to_path PARAMS ((char ** path1,
 					   const char * dir2, char sep));
 void pw_free_path PARAMS ((char ** path));
-void pw_fprintf_path PARAMS ((FILE * stream, 
+void pw_fprintf_path PARAMS ((FILE * stream,
 			   const char * format, char * const * path));
 
 /*
  * Dump NAME.SUFFIX in the PATH on STDOUT
  */
-int pw_paste_file PARAMS ((char * const * path, 
+int pw_paste_file PARAMS ((char * const * path,
 			   const char *name, const char *suffix));
 
 /*
@@ -57,12 +57,18 @@ int pw_file_exists_p PARAMS ((char * const * path,
 /*
  * Return malloc'ed path to NAME.SUFFIX if in PATH, NULL otherwise
  */
-char * pw_find_file PARAMS ((char * const * path,
-			     const char *name, const char *suffix));
+char * pw_find_file
+  PARAMS ((char * const * path, const char *name, const char *suffix));
 /* Idem, but exits upon failure */
-char * xpw_find_file PARAMS ((char * const * path,
-			      const char *name, const char *suffix));
+char * xpw_find_file
+  PARAMS ((char * const * path, const char *name, const char *suffix));
 
+/* Idem, but look first if the file is not in the same dir as
+   INCLUDING_FILE. */
+
+char * xpw_find_included_file
+  PARAMS ((char * const *path, const char *including_file,
+	   const char *name, const char *suffix));
 /*
  * Call glob on PATTERN in each dir of PATH.
  * Return a malloc'd char ** (char * malloc'ed too)
@@ -72,4 +78,10 @@ struct darray * pw_glob PARAMS ((char * const * path,
 struct darray * pw_glob_on_suffix PARAMS ((char * const * path,
 					   const char * suffix));
 
+/* Use lister to report on STREAM the list of files in PATH that end
+   by SUFFIX. */
+
+void pw_lister_on_suffix PARAMS ((FILE * stream,
+				  char * const * path, const char * suffix));
+
 #endif /* not defined _PATHWALK_H_ */
