@@ -3,13 +3,13 @@
  *
  * PostScript Printer Description files
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -39,15 +39,14 @@
 #include "darray.h"
 #include "str_ht.h"
 #include "title.h"
-#define PPD_SUFFIX ".ppd"
+#include "lister.h"
 
-/* From parsepdd.y */
-struct ppd * a2ps_ppd_parse PARAMS ((const char * filename));
+#define PPD_SUFFIX ".ppd"
 
 /************************************************************************/
 /*	Handling the fonts						*/
 /************************************************************************/
-void 
+void
 ppd_font_add (struct ppd * ppd, const char * fontname)
 {
   string_htable_add (ppd->fonts, fontname);
@@ -94,8 +93,7 @@ ppd_self_print (struct ppd * ppd, FILE * stream)
 /* It must include the white spaces put before */
 #define TAB_MAX	76
   char * cp;
-  char ** fonts, ** font;
-  int tab = TAB_MAX, len;
+  char ** fonts;
 
   if (ppd->nickname)
     cp = ppd->nickname;
@@ -103,29 +101,23 @@ ppd_self_print (struct ppd * ppd, FILE * stream)
     cp = ppd->modelname;
   else
     cp = ppd->key;
-  title (stream, '-', "%s (%s)", cp, ppd->key);
+  title (stream, '-', true, "%s (%s)", cp, ppd->key);
 
   /* Report the known fonts */
   fonts = string_htable_dump_sorted (ppd->fonts);
-  fputs (_("List of the known fonts:"), stream);
+  fputs (_("Known Fonts"), stream);
   if (!*fonts)
-    fputs (_("\n   None.\n"), stream);
+    /* TRANS: This `none' is an answer to `List of known fonts: None' */
+    fputs (_("\n  None.\n"), stream);
   else
     {
-      for (font = fonts ; *font ; font++)
-	{
-	  len = strlen (*font) + (*(font + 1) ? 2 : 1);
-	  tab += len;
-	  if (tab > TAB_MAX) {
-	    fputs ("\n   ", stream);
-	    tab = len;
-	  }
-	  fputs (*font, stream);
-	  fputs (*(font + 1) ? ", " : ".\n", stream);
-	}
+      putc ('\n', stream);
+      lister_fprint_separated (NULL, stream,
+			       (void **) fonts, (size_t) -1,
+			       (lister_width_t) strlen,
+			       (lister_print_t) fputs);
     }
   FREE (fonts);
-  putc ('\n', stream);
 }
 
 /*
@@ -137,9 +129,9 @@ _a2ps_ppd_get (char * const * path, const char * key)
 {
   struct ppd * res;
   char * ppd_filepath;
-  
+
   ppd_filepath = xpw_find_file (path, key, ".ppd");
-  res = a2ps_ppd_parse (ppd_filepath);
+  res = a2ps_ppd_parse (ppd_filepath, path);
   res->key = xstrdup (key);
   FREE (ppd_filepath);
   return res;
@@ -154,20 +146,9 @@ _a2ps_ppd_get (char * const * path, const char * key)
 void
 _a2ps_ppd_list_short (char * const * path, FILE * stream)
 {
-  struct darray * entries;
-  size_t i;
-
-  entries = pw_glob_on_suffix (path, PPD_SUFFIX);
-
   fputs (_("Known PostScript Printer Descriptions"), stream);
-
-  for (i = 0 ; i < entries->len ; i++){
-    if (!(i % 7))
-      fputs ("\n  ", stream);
-    fprintf (stream, "%-10s", (char *) entries->content[i]);
-  }
   putc ('\n', stream);
-  da_free (entries, (da_map_func_t) free);
+  pw_lister_on_suffix (stream, path, PPD_SUFFIX);
 }
 
 /*
@@ -181,12 +162,14 @@ _a2ps_ppd_list_long (char * const * path, FILE * stream)
 
   entries = pw_glob_on_suffix (path, PPD_SUFFIX);
 
-  fputs (_("Known PostScript Printer Descriptions"), stream);
+  title (stream, '=', true, _("Known PostScript Printer Descriptions"));
   putc ('\n', stream);
 
   for (i = 0 ; i < entries->len ; i++)
-    ppd_self_print (_a2ps_ppd_get (path, entries->content[i]), stream);
+    {
+      ppd_self_print (_a2ps_ppd_get (path, entries->content[i]), stream);
+      putc ('\n', stream);
+    }
 
   da_free (entries, (da_map_func_t) free);
 }
-
