@@ -3,12 +3,12 @@
  * routines for the postscript generation
  *
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1995, 96, 97, 98, 99 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -34,7 +34,7 @@
 #include "routines.h"		/* general interest routines		*/
 #include "output.h"		/* Diverted outputs			*/
 #include "media.h"
-#include "msg.h"
+#include "message.h"
 #include "dsc.h"
 #include "jobs.h"
 #include "fonts.h"
@@ -95,20 +95,9 @@ a2ps_job_stream_hook ps_comment_hook = NULL;
 void
 prologues_list_short (a2ps_job * job, FILE * stream)
 {
-  struct darray * entries;
-  size_t i;
-
-  entries = pw_glob_on_suffix (job->common.path, PS_PROLOGUE_SUFFIX);
-
-  fputs (_("Known prologues"), stream);
-
-  for (i = 0 ; i < entries->len ; i++){
-    if (!(i % 7))
-      fputs ("\n  ", stream);
-    fprintf (stream, "%-10s", (char *) entries->content[i]);
-  }
+  fputs (_("Known Prologues"), stream);
   putc ('\n', stream);
-  da_free (entries, (da_map_func_t) free);
+  pw_lister_on_suffix (stream, job->common.path, PS_PROLOGUE_SUFFIX);
 }
 
 /*
@@ -134,7 +123,7 @@ prologue_print_signature (struct a2ps_job * job,
   fp = xrfopen (filename);
 
   /* Dump rest of file. */
-  while ((getshline_numbered (&firstline, &lastline, 
+  while ((getshline_numbered (&firstline, &lastline,
 			      &buf, &bufsiz, fp) != -1)
 	 && !done)
     {
@@ -153,8 +142,8 @@ prologue_print_signature (struct a2ps_job * job,
 	      (*documentation_fn) ((uchar *) buf2, "%s", stream);
 	    }
 	  if (!strprefix (END_DOC_TAG, buf2))
-	    error (1, 0, filename, firstline, _("missing argument for `%s'"),
-		   "Documentation");
+	    error (1, 0, filename, firstline,
+		   _("missing argument for `%s'"), "`Documentation'");
 	  done = TRUE;
 	}
     }
@@ -169,7 +158,7 @@ prologue_print_signature (struct a2ps_job * job,
  * List the prologues together with there description
  */
 static inline void
-prologues_list_long_internal (a2ps_job * job, FILE * stream, 
+prologues_list_long_internal (a2ps_job * job, FILE * stream,
 			      const char * name_format,
 			      documentation_fn_t documentation_fn)
 {
@@ -189,10 +178,10 @@ prologues_list_long_internal (a2ps_job * job, FILE * stream,
 void
 prologues_list_long (a2ps_job * job, FILE * stream)
 {
-  fputs (_("Known prologues"), stream);
+  fputs (_("Known Prologues"), stream);
   putc ('\n', stream);
 
-  prologues_list_long_internal (job, stream, 
+  prologues_list_long_internal (job, stream,
 				"Prologue \"%s\":\n",
 				documentation_print_plain);
 }
@@ -236,22 +225,22 @@ dump_prolog_comments (FILE * stream, struct a2ps_job * job)
    */
   fputs ((char *) job->status->magic_number, stream);
   putc ('\n', stream);
-  
-  cp = expand_user_string (job, FIRST_FILE (job), 
+
+  cp = expand_user_string (job, FIRST_FILE (job),
 			   (uchar *) "Document title", job->title);
   fprintf (stream, "%%%%Title: %s\n", cp);
 
-  cp = expand_user_string (job, FIRST_FILE (job), 
+  cp = expand_user_string (job, FIRST_FILE (job),
 			  (uchar *) "User Name", (const uchar *) "%N");
   fprintf (stream, "%%%%For: %s\n", cp);
   fprintf (stream, "%%%%Creator: %s version %s\n", PACKAGE, VERSION);
   fprintf (stream, "%%%%CreationDate: %s", asctime(&job->run_tm));
 
-  fprintf (stream, "%%%%BoundingBox: %d %d %d %d\n", 
+  fprintf (stream, "%%%%BoundingBox: %d %d %d %d\n",
 	   job->medium->llx, job->medium->lly,
 	   job->medium->urx, job->medium->ury);
   fprintf (stream, "%%%%DocumentData: Clean7Bit\n");
-  fprintf (stream, "%%%%Orientation: %s\n", 
+  fprintf (stream, "%%%%Orientation: %s\n",
 	   (job->orientation == landscape) ? "Landscape" : "Portrait");
   fprintf (stream, "%%%%Pages: %d\n", job->sheets);
   if (job->status->page_are_ordered)
@@ -259,9 +248,9 @@ dump_prolog_comments (FILE * stream, struct a2ps_job * job)
   else
     fprintf (stream, "%%%%PageOrder: Special\n");
   fprintf (stream, "%%%%DocumentMedia: %s %d %d 0 () ()\n",
-	   job->medium->name, 
+	   job->medium->name,
 	   job->medium->w, job->medium->h);
-  
+
   dump_needed_resources (stream, job);
   dump_process_color (stream, job);
   dump_requirements (stream, job);
@@ -283,7 +272,7 @@ dump_prolog_comments (FILE * stream, struct a2ps_job * job)
 /*
  * Print the PostScript prolog.
  */
-static void 
+static void
 output_prolog (a2ps_job * job)
 {
   /* Comments */
@@ -309,7 +298,7 @@ output_prolog (a2ps_job * job)
 /*
  * Print the PostScript document setup part.
  */
-static void 
+static void
 output_document_setup (a2ps_job * job)
 {
   int i, j;
@@ -339,7 +328,7 @@ output_document_setup (a2ps_job * job)
   switch (job->orientation) {
   case portrait:
     /* FIXME: prendre en compte les marges */
-    output (jdiv, "/sh %d def\n", job->medium->h); 
+    output (jdiv, "/sh %d def\n", job->medium->h);
     output (jdiv, "/sw %d def\n", job->medium->w);
     output (jdiv, "/llx %d def\n", job->medium->llx);
     output (jdiv, "/urx %d def\n", job->medium->urx - job->margin);
@@ -352,9 +341,9 @@ output_document_setup (a2ps_job * job)
     output (jdiv, "/sw %d def\n", job->medium->h);
     output (jdiv, "/llx %d def\n", job->medium->lly);
     output (jdiv, "/urx %d def\n", job->medium->ury);
-    output (jdiv, "/ury %d def\n", 
+    output (jdiv, "/ury %d def\n",
 	    job->medium->w - job->medium->llx);
-    output (jdiv, "/lly %d def\n", 
+    output (jdiv, "/lly %d def\n",
 	    job->medium->w - job->medium->urx + job->margin);
     break;
   }
@@ -371,7 +360,7 @@ output_document_setup (a2ps_job * job)
 
   /* Page device definitions */
   output_pagedevice (job);
-  
+
   /* Header size */
   output (jdiv, "/th %f def\n", job->status->title_bar_height);
 
@@ -400,14 +389,14 @@ output_document_setup (a2ps_job * job)
 
 
   output (jdiv, "/hm fnfs 0.25 mul def\n");
-  
+
   /* Page attributes */
   output (jdiv, "/pw\n");
-  output (jdiv, "   cw %f mul\n", 
+  output (jdiv, "   cw %f mul\n",
 	  (float) job->status->columnsperline + 2 * SIDE_MARGIN_RATIO);
   output (jdiv, "def\n");
   output (jdiv, "/ph\n");
-  output (jdiv, "   %f th add\n", 
+  output (jdiv, "   %f th add\n",
 	  (job->status->linesperpage + BOTTOM_MARGIN_RATIO) * job->fontsize);
   output (jdiv, "def\n");
   if (job->columns > 1)
@@ -417,13 +406,13 @@ output_document_setup (a2ps_job * job)
     output (jdiv, "/pmw 0 def\n");
   if (job->rows > 1)
     output (jdiv, "/pmh ury lly sub ph %d mul sub %d sub %d div def\n",
-	    job->rows, 
-	    (PRINT_HEADER + PRINT_FOOTER) * HEADERS_H, 
+	    job->rows,
+	    (PRINT_HEADER + PRINT_FOOTER) * HEADERS_H,
 	    job->rows - 1);
   else
     output (jdiv, "/pmh 0 def\n");
   output (jdiv, "/v 0 def\n");
-  
+
   switch (job->madir) {
   case madir_rows:
     output (jdiv, "/x [\n");
@@ -464,10 +453,10 @@ output_document_setup (a2ps_job * job)
     error (1, 0, "output_document_setup: bad switch");
     break;
   }
-    
+
   output (jdiv, "/scx sw 2 div def\n");
   output (jdiv, "/scy sh 2 div def\n");
-  
+
   output (jdiv, "/snx urx def\n");
   output (jdiv, "/sny lly 2 add def\n");
   output (jdiv, "/dx llx def\n");
@@ -488,7 +477,7 @@ output_document_setup (a2ps_job * job)
 static void
 end_document (a2ps_job * job)
 {
-  clean_up (job);
+  page_flush (job);
   output (jdiv, "\n%%%%Trailer\n");
   if (job->page_prefeed)
     output (jdiv, "false page_prefeed\n");
@@ -503,7 +492,7 @@ end_document (a2ps_job * job)
  * Called by the main loop to initialize postscript printing
  */
 void
-ps_init (a2ps_job * job)
+ps_begin (a2ps_job * job)
 {
   float area_w = 0.0, area_h = 0.0;	/* Dimension of drawable area	*/
   float printing_h, printing_w;  	/* Body size of virtual pages	*/
@@ -545,7 +534,7 @@ ps_init (a2ps_job * job)
     job->status->title_font_size = 15;
     job->status->title_bar_height = PORTRAIT_HEADER;
   }
-  
+
   /* Area inside the frame of a virtual page */
   printing_h = ((area_h
 		 /* room for title */
@@ -553,46 +542,46 @@ ps_init (a2ps_job * job)
 		 /* Space between the virtual pages */
 		 - ((job->rows > 1) ? PAGE_MARGIN : 0))
 		/ job->rows);
-  printing_w = ((area_w 
+  printing_w = ((area_w
 		 /* Space between the virtual pages */
 		 - ((job->columns > 1) ? PAGE_MARGIN : 0))
 		/ job->columns);
-  
-  /* 
+
+  /*
    * Determine the font size according to (decreasing priority):
    * 1. --font=@<size>
-   * 2. --columns-per-page, 
+   * 2. --columns-per-page,
    * 3. --lines-per-page,
    * 4. default values
    */
   /* width = 0.6 font size */
   if (job->columns_requested != 0) {
-    job->fontsize = ((printing_w / 
-		      (job->columns_requested + prefix_size + 
+    job->fontsize = ((printing_w /
+		      (job->columns_requested + prefix_size +
 		       2 * SIDE_MARGIN_RATIO))
 		     / 0.6);
   } else if (job->lines_requested != 0) {
-    job->fontsize = (printing_h / 
+    job->fontsize = (printing_h /
 		     (job->lines_requested + BOTTOM_MARGIN_RATIO));
   } else if (job->fontsize == 0.0)
-    job->fontsize = ((job->orientation == landscape) ? 6.8 
+    job->fontsize = ((job->orientation == landscape) ? 6.8
 		     : (job->columns * job->rows > 1) ? 6.4 : 9.0);
 
   /* fontsize is OK.  Calculate the other variables */
-  job->status->linesperpage = 
+  job->status->linesperpage =
     (int) ((printing_h / job->fontsize) - BOTTOM_MARGIN_RATIO);
-  job->status->columnsperline = 
+  job->status->columnsperline =
     (int) ((printing_w / (job->fontsize * 0.6)) - 2 * SIDE_MARGIN_RATIO);
 
   if (job->columns_requested > 0)
     job->status->columnsperline = job->columns_requested + prefix_size;
   else if (job->lines_requested > 0)
     job->status->linesperpage = job->lines_requested;
-  
+
   if (job->status->linesperpage <= 0 || job->status->columnsperline <= 0)
     error (1, 0, _("font %f too big"), job->fontsize);
 
-  job->status->wxperline = 
+  job->status->wxperline =
     (job->status->columnsperline - prefix_size) * COURIER_WX;
 
   output_document_setup (job);
@@ -600,11 +589,11 @@ ps_init (a2ps_job * job)
   /* Set the encoding.  Usualy it is done in begin_sheet, but
    * begin_sheet may not be called if, for instance, the first page is
    * skip (by --pages).  We must guarantee at least the first encoding
-   * 
+   *
    * FIXME: This does not work.  For instance if encoding=ascii,
    * and the first thing that the doc does is the switch to
    * latin1.  Then, there is an encoding closed that was not opened in the ps.
-   */ 
+   */
 /*  set_encoding (job, job->requested_encoding);*/
   job->saved_encoding = job->requested_encoding;
 }
@@ -613,7 +602,7 @@ ps_init (a2ps_job * job)
  * Finish the poscript printing
  */
 void
-ps_close (a2ps_job * job)
+ps_end (a2ps_job * job)
 {
 /* Make sure that the output is really processed.  It
    * could have been closed by a page range selection, but
