@@ -2,13 +2,13 @@
  * psgen.c
  * routines for the postscript generation
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -37,7 +37,7 @@
 #include "media.h"
 #include "faces.h"
 #include "metaseq.h"
-#include "msg.h"
+#include "message.h"
 #include "fonts.h"
 #include "dsc.h"
 #include "jobs.h"
@@ -77,7 +77,7 @@
  * Print a char in a form accepted by postscript printers.
  * Returns number of columns used (on the output) to print the char.
  */
-static int 
+static int
 ps_escape_char (a2ps_job * job, uchar c, uchar * res)
 {
   int len=0;
@@ -121,17 +121,17 @@ ps_escape_char (a2ps_job * job, uchar c, uchar * res)
   case space:
     USTRCCAT (res, ' ');
     return 1;
-    
+
   case caret:
     if (0177 < c) {
       ustrcat(res, "M-");
-      len += 2; 
+      len += 2;
       c &= 0177;
     }
-    
+
     if (c < ' ') {
       USTRCCAT(res, '^');
-      len += 2; 
+      len += 2;
       c += '@';
       if (c == '(' || c == ')' || c == '\\')
 	USTRCCAT(res, '\\');
@@ -150,13 +150,13 @@ ps_escape_char (a2ps_job * job, uchar c, uchar * res)
   case Emacs:
     if (0177 < c) {
       ustrcat(res, "M-");
-      len += 2; 
+      len += 2;
       c &= 0177;
     }
-    
+
     if (c < ' ') {
       ustrcat (res, "C-");
-      len += 3; 
+      len += 3;
       c += '@';
       if (c == '(' || c == ')' || c == '\\')
 	USTRCCAT(res, '\\');
@@ -179,15 +179,15 @@ ps_escape_char (a2ps_job * job, uchar c, uchar * res)
 /*
  * Print a string in a form accepted by postscript printers.
  */
-static int 
+static int
 ps_escape_string (a2ps_job * job, uchar * string, uchar * res)
 {
   size_t i;
   int delta_column=0;
 
-  for (i = 0 ; i < ustrlen (string) ; i++) 
+  for (i = 0 ; i < ustrlen (string) ; i++)
     delta_column += ps_escape_char (job, string[i], res);
-  
+
   return delta_column;
 }
 
@@ -200,9 +200,9 @@ output_marker (a2ps_job * job, const char * kind, uchar * marker)
   uchar *cp, buf[256];
   int i;
 
-  cp = expand_user_string (job, CURRENT_FILE (job), 
+  cp = expand_user_string (job, CURRENT_FILE (job),
 			   (const uchar *) kind, marker);
-  
+
   for (i = 0 ; cp[i] ; i++) {
     switch (cp[i]) {
     case FILE_LAST_PAGE:
@@ -222,7 +222,7 @@ output_marker (a2ps_job * job, const char * kind, uchar * marker)
       break;
     case JOB_NB_PAGES:
       output_delayed_int (jdiv, &job->pages);
-      break;      
+      break;
     case JOB_NB_SHEETS:
       output_delayed_int (jdiv, &job->sheets);
       break;
@@ -265,7 +265,7 @@ ps_end_encoding (a2ps_job * job)
 /*
  * Switch of encoding while we are _not_ in the body.  E.g., we
  * are dealing with the headers.
- * 
+ *
  * We want to save the previous encoding, because, say we
  * were printing -X latin1, we have switched to latin2 dynamically.
  * Right now, we need to change page,
@@ -339,8 +339,8 @@ ps_pop_encoding (a2ps_job * job)
  * Note that is may called for a second time on a single sheet if two
  * different files are printed on the same sheet.
  */
-static void 
-begin_page (a2ps_job * job)
+static void
+page_begin (a2ps_job * job)
 {
   job->virtual = 1;
 
@@ -355,27 +355,26 @@ begin_page (a2ps_job * job)
    * We do this because one would like to have all the information on
    * what is on the current page before making the page label.
    */
-  if (!output_is_to_void (jdiv)) {
-    output (jdiv, "%%%%Page: (");
-    job->status->page_label = ALLOC (uchar *, 1);
-    output_delayed_string (jdiv, job->status->page_label);
-    output (jdiv, ") %d\n", job->sheets);
-  }
-  
+  if (!output_is_to_void (jdiv))
+    {
+      output (jdiv, "%%%%Page: (");
+      job->status->page_label = ALLOC (uchar *, 1);
+      output_delayed_string (jdiv, job->status->page_label);
+      output (jdiv, ") %d\n", job->sheets);
+    }
+
   /* Reinitialize state variables for each new sheet */
   output (jdiv, "%%%%BeginPageSetup\n");
   output (jdiv, "/pagesave save def\n");
 
   /* Shift front side sheets */
   if (job->margin
-      && (!job->duplex || (job->sheets % 2)))
+      && (job->duplex == simplex || (job->sheets % 2)))
     output (jdiv, "%d 0 translate\n", job->margin);
 
-  if (job->orientation == landscape) {
-    output (jdiv, "sh 0 translate\n");
-    output (jdiv, "90 rotate\n");
-  }
-  
+  if (job->orientation == landscape)
+    output (jdiv, "sh 0 translate 90 rotate\n");
+
   output (jdiv, "%%%%EndPageSetup\n");
 
   if (job->debug)
@@ -398,26 +397,26 @@ begin_page (a2ps_job * job)
   if (!IS_EMPTY(job->water)) {
     output_char (jdiv, '(');
     output_marker (job, "water mark", job->water);
-    output (jdiv, 
+    output (jdiv,
 	    ") %4.2f water\n",
-	    ((float) atan2 ((double) job->medium->w - job->margin, 
-			    (double) job->medium->h) 
+	    ((float) atan2 ((double) job->medium->w - job->margin,
+			    (double) job->medium->h)
 	     / 3.14159265 * 180));
   }
-  
+
   /* Move to the lower left point of the drawable area */
   output (jdiv, "gsave\n");
-  output (jdiv, "llx lly %d add translate\n", 
+  output (jdiv, "llx lly %d add translate\n",
 	  PRINT_FOOTER * HEADERS_H);
   /* Set the encoding */
   ps_internal_switch_encoding (job, job->saved_encoding);
 }
 
-/* The job on the page is over: puts the headers and footers, 
+/* The job on the page is over: puts the headers and footers,
  * then print the physical page.
  */
-static void 
-end_page (a2ps_job * job)
+static void
+page_end (a2ps_job * job)
 {
   /* The page label has been divertered (through a handler).
    * Fill that handler with the correct page label value
@@ -426,15 +425,14 @@ end_page (a2ps_job * job)
     xustrdup (expand_user_string (job, CURRENT_FILE (job),
 				  (const uchar *) "Page label",
 				  job->status->page_label_format));
-  
-  output (jdiv, "%% End of virtual page\n");
+
   output (jdiv, "grestore\n");
 
   /* All the headers should be written using the requested encoding */
   ps_push_encoding (job, job->requested_encoding);
 
   /* Print the right header */
-  if (PRINT_HEADER) { 
+  if (PRINT_HEADER) {
     output_char (jdiv, '(');
     output_marker (job, "right header", job->header);
     output (jdiv, ") rhead\n");
@@ -450,17 +448,17 @@ end_page (a2ps_job * job)
       output (jdiv, ") ");
     } else
       output (jdiv, "() ");
-    
+
     /* Print the right footer */
-    if (!IS_EMPTY(job->right_footer)) { 
+    if (!IS_EMPTY(job->right_footer)) {
       output_char (jdiv, '(');
       output_marker (job, "right footer", job->right_footer);
       output (jdiv, ") ");
     } else
       output (jdiv, "() ");
-    
+
     /* Print the left footer */
-    if (!IS_EMPTY(job->left_footer)) { 
+    if (!IS_EMPTY(job->left_footer)) {
       output_char (jdiv, '(');
       output_marker (job, "left footer", job->left_footer);
       output (jdiv, ") ");
@@ -469,10 +467,10 @@ end_page (a2ps_job * job)
       output (jdiv, "() ");
     output (jdiv, "footer\n");
   }
-  
+
   /* Close the current encoding */
   ps_end_encoding (job);
-  
+
   output (jdiv, "pagesave restore\n");
   output (jdiv, "showpage\n");
 
@@ -486,8 +484,8 @@ end_page (a2ps_job * job)
  * Prints page header and page border and
  * initializes printing of the file lines.
  */
-static void 
-begin_virtual (a2ps_job * job)
+static void
+virtual_begin (a2ps_job * job)
 {
   job->pages++;
 
@@ -500,17 +498,16 @@ begin_virtual (a2ps_job * job)
 
   /* We test to one, since it has just been incremented above */
   if (job->virtual == 1)
-    begin_page (job);
+    page_begin (job);
 
-  if (!job->encoding) {
+  if (!job->encoding)
     ps_set_encoding (job, job->saved_encoding);
-  }  
 
   file_job_synchronize_pages (job);
   file_job_synchronize_sheets (job);
-  
+
   output (jdiv, "/v %d store\n", job->virtual - 1);
-  output (jdiv, "/x0 x v get %f add sx cw mul add store\n", 
+  output (jdiv, "/x0 x v get %f add sx cw mul add store\n",
 	  SIDE_MARGIN_RATIO * job->fontsize * 0.6);
   output (jdiv, "/y0 y v get bfs %s sub store\n",
 	  PRINT_TITLE ? "th add" : "");
@@ -521,8 +518,8 @@ begin_virtual (a2ps_job * job)
  * Adds a sheet number to the page (footnote) and prints the formatted
  * page (physical impression). Activated at the end of each source page.
  */
-static void 
-end_virtual (a2ps_job * job)
+static void
+virtual_end (a2ps_job * job)
 {
   /*
    *	Print the titles with the option-given encoding
@@ -530,82 +527,103 @@ end_virtual (a2ps_job * job)
   /* Draw the header and its content */
   if (PRINT_TITLE) {
     ps_push_encoding (job, job->requested_encoding);
-    if (!IS_EMPTY(job->center_title)) { 
+    if (!IS_EMPTY(job->center_title)) {
       output_char (jdiv, '(');
       output_marker (job, "center title", job->center_title);
       output (jdiv, ") ");
     } else
       output (jdiv, "() ");
-    
-    if (!IS_EMPTY(job->right_title)) { 
+
+    if (!IS_EMPTY(job->right_title)) {
       output_char (jdiv, '(');
       output_marker (job, "right title", job->right_title);
       output (jdiv, ") ");
     } else
       output (jdiv, "() ");
-    
-    if (!IS_EMPTY(job->left_title)) { 
+
+    if (!IS_EMPTY(job->left_title)) {
       output_char (jdiv, '(');
       output_marker (job, "left title", job->left_title);
       output (jdiv, ") ");
     } else
       output (jdiv, "() ");
-    
+
     output (jdiv, "title\n");
     ps_pop_encoding (job);
   }
-  
+
   if (job->border)
     output (jdiv, "border\n");
-  
+
   if (job->virtual == (job->columns * job->rows))
-    end_page (job);
+    page_end (job);
 
   job->status->line = 0;
 }
 
+/* Issue an empty virtual (used for file alignment). */
+
+static void
+virtual_empty_output (a2ps_job *job)
+{
+  /* Let's use a hidden option which lets choose between an empty
+     virtual (with headers etc.), or nothing printed at all. */
+  if (macro_meta_sequence_get (job, VAR_OPT_VIRTUAL_FORCE))
+    {
+      virtual_begin (job);
+      virtual_end (job);
+    }
+  else
+    {
+      /* Just increase the virtual number, that's enough. */
+      job->pages++;
+      job->virtual++;
+      if (job->virtual == (job->columns * job->rows))
+	page_end (job);
+    }
+}
+
 /*
  * Flush the (physical) page, ready for new page
- * Used 
- * -- at end of whole job
- * -- at end of page
+ * Used
+ * -- at the end of the whole job
+ * -- at the end of a page
  * -- when a fresh page is required (eg, when after an a2ps
  *    generated ps page arrives a delegated ps file)
  */
-void 
-clean_up (a2ps_job * job)
+void
+page_flush (a2ps_job * job)
 {
   /* Everything that follows _must_ be dumped */
   output_to_void (job->divertion, FALSE);
 
   /* If the sheet has not been printed, flush it */
   if (job->virtual != 0)
-    end_page (job);
+    page_end (job);
+}
 
-  if (!job->compact_mode 
-      && job->duplex 
-      && (job->sheets & 0x1) != 0)
-    {
-      uchar * page_label;
-      
-      job->sheets++;
-      file_job_synchronize_sheets (job);
-      
-      page_label = expand_user_string (job, CURRENT_FILE (job),
-				       (uchar *) "Page label",
-				       job->status->page_label_format);
-      output (jdiv, "%%%%Page: (%s) %d\n", 
-	      page_label, job->sheets);
-      
-      /* Reinitialize state variables for each new sheet */
-      output (jdiv, "%%%%BeginPageSetup\n");
-      output (jdiv, "/pagesave save def\n");
-      output (jdiv, "%%%%EndPageSetup\n");
-      
-      output (jdiv, "%% Blank 2nd side sheet\n");
-      output (jdiv, "pagesave restore\n");
-      output (jdiv, "showpage\n");
-    }
+/* Output an empty page (used for instance to align files.
+   No page should be waiting (issue flushes before). */
+
+static void
+page_empty_output (a2ps_job *job)
+{
+  job->sheets++;
+  file_job_synchronize_sheets (job);
+  output (jdiv, "%%%%Page: (*) %d\n", job->sheets);
+  output (jdiv, "%% Empty Page\n");
+  output (jdiv, "showpage\n");
+}
+
+static void
+sheet_flush (a2ps_job *job)
+{
+  page_flush (job);
+
+  /* Need an empty back side? */
+  if ((job->duplex == duplex || job->duplex == tumble)
+      && (job->sheets % 2) != 0)
+    page_empty_output (job);
 }
 
 /*
@@ -617,10 +635,9 @@ void
 require_fresh_page (a2ps_job * job)
 {
   /* If this is not a blank sheet, end it */
-  if (job->virtual != 0) {
+  if (job->virtual != 0)
     /* The clean up _must_ be done */
-    clean_up (job);
-  }
+    page_flush (job);
 }
 
 /****************************************************************/
@@ -637,9 +654,9 @@ require_fresh_page (a2ps_job * job)
 static void
 check_binary_file (a2ps_job * job)
 {
-  if (job->status->chars > 120) 
+  if (job->status->chars > 120)
     {
-      if (!job->print_binaries 
+      if (!job->print_binaries
 	  && (job->status->nonprinting_chars*100 / job->status->chars) >= 40)
 	error (1, 0, _("`%s' is a binary file, printing aborted"),
 	       CURRENT_FILE (job)->name);
@@ -649,14 +666,14 @@ check_binary_file (a2ps_job * job)
 static inline void
 end_of_line (struct a2ps_job * job)
 {
-  if (!job->status->face_declared) {			
+  if (!job->status->face_declared) {
     output (jdiv, ") %s n\n", face_eo_ps (job->status->face));
-    job->status->face_declared = TRUE;			
-  } else							
-    output (jdiv, ") N\n");				
-  job->status->line++;					
-  job->status->column = 0;				
-  job->status->wx = 0;					
+    job->status->face_declared = TRUE;
+  } else
+    output (jdiv, ") N\n");
+  job->status->line++;
+  job->status->column = 0;
+  job->status->wx = 0;
 }
 
 #define page_full	\
@@ -670,23 +687,27 @@ end_of_line (struct a2ps_job * job)
 static inline void
 fold_line (struct a2ps_job * job, enum face_e new_face)
 {
-  end_of_line (job);							
-  if (page_full) {						
-    end_virtual (job);						
-    begin_virtual (job);						
-    job->status->face_declared = FALSE ;			
-  }								
-  if (job->numbering) {					
-    output (jdiv, "0 T (");					
-  } else {							
-    /* This is a new line, hence we can consider that there is	
-     * no need to close the current font: just consider it is	
-     * the new font, but not declared */			
-    output_char (jdiv, '(');					
-    job->status->face_declared &= (job->status->face 		
-				   == new_face);		
-    job->status->face = new_face;				
-  }								
+  end_of_line (job);
+  if (page_full)
+    {
+      virtual_end (job);
+      virtual_begin (job);
+      job->status->face_declared = FALSE ;
+    }
+  if (job->numbering)
+    {
+      output (jdiv, "0 T (");
+    }
+  else
+    {
+      /* This is a new line, hence we can consider that there is no
+	 need to close the current font: just consider it is the new
+	 font, but not declared. */
+      output_char (jdiv, '(');
+      job->status->face_declared &= (job->status->face
+				     == new_face);
+      job->status->face = new_face;
+  }
 }
 
 /*
@@ -707,7 +728,7 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
    * - prefix parents and backslash ['(', ')', '\'] by backslash
    *   (escape character in postcript)
    */
-  if (job->status->is_in_cut 
+  if (job->status->is_in_cut
       && (c != '\f' )
       /* FIXME: May be some day, using a more flexible scheme
        * would be good
@@ -717,48 +738,54 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
   job->status->is_in_cut = FALSE;
 
   /* Start a new line ? */
-  if (job->status->start_line) {
-    if (job->status->start_page) 
-      {
-	/* only if there is something to print! */
-	begin_virtual (job);
-	job->status->start_page = FALSE ;
-	/* This is the first line of a new page, hence we need (page
-	 * independance) to repeat the current font */
-	job->status->face = new_face;
-	job->status->face_declared = FALSE;
-
-	if (job->numbering) {
-	  if (CURRENT_FILE (job)->lines % job->numbering == 0)
-	    output (jdiv, "(%d) # (", CURRENT_FILE (job)->lines);
-	  else
-	    output (jdiv, "0 T (");
-	} else
-	  output_char (jdiv, '(');
-      } 
-    else
-      {
-	/* This is a new line, but not the first in the page */
-	if (job->numbering) {
-	  if (CURRENT_FILE (job)->lines % job->numbering == 0)
-	    output (jdiv, "(%d) # (", CURRENT_FILE (job)->lines);
-	  else
-	    output (jdiv, "0 T (");
-	} else {
-	  /* This is a new line, hence we can consider that there is
-	   * no need to close the current font: just consider it is
-	   * the new font, but not declared */
-	  output_char (jdiv, '(');
-	  /* Why the hell did I do this? */
-	  /* FIXME: This is suppresed because of the changes of encoding */
-	  job->status->face_declared = (job->status->face_declared
-					&& (job->status->face == new_face));
+  if (job->status->start_line)
+    {
+      if (job->status->start_page)
+	{
+	  /* only if there is something to print! */
+	  virtual_begin (job);
+	  job->status->start_page = FALSE ;
+	  /* This is the first line of a new page, hence we need (page
+	   * independance) to repeat the current font */
 	  job->status->face = new_face;
+	  job->status->face_declared = FALSE;
+
+	  if (job->numbering)
+	    {
+	      if (CURRENT_FILE (job)->lines % job->numbering == 0)
+		output (jdiv, "(%d) # (", CURRENT_FILE (job)->lines);
+	      else
+		output (jdiv, "0 T (");
+	    }
+	  else
+	    output_char (jdiv, '(');
 	}
-      }
-    job->status->start_line = FALSE;
-  }
-  
+      else
+	{
+	  /* This is a new line, but not the first in the page */
+	  if (job->numbering)
+	    {
+	      if (CURRENT_FILE (job)->lines % job->numbering == 0)
+		output (jdiv, "(%d) # (", CURRENT_FILE (job)->lines);
+	      else
+		output (jdiv, "0 T (");
+	    }
+	  else
+	    {
+	      /* This is a new line, hence we can consider that there is
+		 no need to close the current font: just consider it is
+		 the new font, but not declared. */
+	      output_char (jdiv, '(');
+	      /* Why the hell did I do this? */
+	      /* FIXME: This is suppresed because of the changes of encoding */
+	      job->status->face_declared = (job->status->face_declared
+					    && (job->status->face == new_face));
+	      job->status->face = new_face;
+	    }
+	}
+      job->status->start_line = FALSE;
+    }
+
   /*
    * Interpret each character
    *
@@ -772,16 +799,18 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
       goto print;
 
     /* Close current line */
-    if (!job->status->start_line) {
-      end_of_line (job);
-      job->status->start_line = TRUE;
-    }
+    if (!job->status->start_line)
+      {
+	end_of_line (job);
+	job->status->start_line = TRUE;
+      }
     /* start a new page ? */
-    if (job->status->start_page) {
-      begin_virtual (job);
-    }
+    if (job->status->start_page)
+      {
+	virtual_begin (job);
+      }
     /* Close current page and begin another */
-    end_virtual (job);
+    virtual_end (job);
     job->status->start_page = TRUE;
     break;
 
@@ -799,7 +828,7 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
     end_of_line (job);
 
     if (page_full) {
-      end_virtual (job);
+      virtual_end (job);
       job->status->start_page = TRUE ;
     }
     break;
@@ -817,11 +846,11 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
       job->status->face = new_face;
       job->status->face_declared = FALSE;
     }
-    
+
     /* Tabs are interpreted  but we want to go to the same
      * column as if the font were Courier
      */
-    job->status->column = 
+    job->status->column =
       (A2_MAX(job->status->wx / COURIER_WX, job->status->column)
        / job->tabsize + 1) * job->tabsize;
     job->status->wx = job->status->column * COURIER_WX;
@@ -847,7 +876,7 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
       uchar buf[256];
       int nchars;
       *buf = '\0';
-      
+
   /* Is this a new font? */
   if (job->status->face != new_face) {
     if (!job->status->face_declared)
@@ -857,7 +886,7 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
     job->status->face = new_face;
     job->status->face_declared = FALSE;
   }
-  
+
       nchars = ps_escape_char (job, c, buf);
       job->status->wx += char_WX (job, c);
       job->status->column += nchars;
@@ -893,7 +922,7 @@ ps_print_string (a2ps_job * job, uchar * string, enum face_e face)
  */
 void
 ps_print_buffer (a2ps_job * job,
-		 const uchar * buffer, 
+		 const uchar * buffer,
 		 size_t start, size_t end,
 		 enum face_e face)
 {
@@ -901,26 +930,73 @@ ps_print_buffer (a2ps_job * job,
   for (i = start ; i < end ; i++)
     ps_print_char (job, buffer [i], face);
 }
+
+/* Handling the input sessions in an output session, i.e., typically
+   the files. */
+
+/* A new file will be printed. */
 
 void
-ps_end_file (a2ps_job * job)
+ps_begin_file (a2ps_job *job)
 {
-  /* If a line was not finished, close it. 
+  /* Reinitialize the ps status */
+  initialize_ps_status (job->status);
+
+  /* Alignment is not needed for the first file. */
+  if (job->jobs->len == 0)
+    return;
+
+  switch (job->file_align)
+    {
+    case file_align_virtual:
+      /* Nothing to do. */
+      break;
+
+    case file_align_rank:
+      /* Issue virtual until we are in a new rank. */
+      {
+	int alignment = job->madir == madir_rows ? job->columns : job->rows;
+	while (job->pages % alignment != 0)
+	  virtual_empty_output (job);
+      }
+      break;
+
+    case file_align_page:
+      /* End the page if needed. */
+      page_flush (job);
+      break;
+
+    case file_align_sheet:
+      /* End the sheet if needed. */
+      sheet_flush (job);
+      break;
+
+    default:
+      /* job->file_align is a number.  We must issue as many pages as
+         needed to have a page number which is a multiple of
+         FILE_ALIGN plus one. */
+      page_flush (job);
+      while ((job->sheets) % job->file_align != 0)
+	page_empty_output (job);
+      break;
+    }
+}
+
+void
+ps_end_file (a2ps_job *job)
+{
+  /* If a line was not finished, close it.
    * Typically, no \n at end of file */
-  if (!job->status->start_line) {
-    if (!job->status->face_declared)
-      output (jdiv, ") %s\n", face_eo_ps (job->status->face));
-    else
-      output (jdiv, ") S\n");
-  }
+  if (!job->status->start_line)
+    {
+      if (!job->status->face_declared)
+	output (jdiv, ") %s\n", face_eo_ps (job->status->face));
+      else
+	output (jdiv, ") S\n");
+    }
   if (!job->status->start_page)
-    end_virtual (job);
-  
-  /* If we are in compact mode and the sheet is not full,
-   * end the current sheet */
-  if (!job->compact_mode)
-    require_fresh_page (job);
-  
+    virtual_end (job);
+
   /* Set the number of pages/sheets printed */
   file_job_synchronize_pages (job);
   file_job_synchronize_sheets (job);
