@@ -3,13 +3,13 @@
  *
  * Uniform access to pipe and files
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -33,10 +33,11 @@
 #include "a2ps.h"
 #include "stream.h"
 #include "routines.h"
+#include "message.h"
+#include "quotearg.h"
+
+/* Open for reading. */
 
-/*
- * Open for reading
- */
 static inline struct stream *
 _stream_ropen (const char * command, boolean is_file)
 {
@@ -68,7 +69,7 @@ stream_ropen (const char * command, boolean is_file)
  * Open for writing
  */
 static inline struct stream *
-_stream_wopen (const char * command, boolean is_file, 
+_stream_wopen (const char * command, boolean is_file,
 	       enum backup_type backup_type)
 {
   NEW (struct stream, res);
@@ -79,11 +80,9 @@ _stream_wopen (const char * command, boolean is_file,
   else
     {
       if (!IS_EMPTY (command))
-	{
-	  res->fp = fopen_backup (command, backup_type);
-	}
+	res->fp = fopen_backup (command, backup_type);
       else
-	res->fp  = stdout;
+	res->fp = stdout;
     }
   return res;
 }
@@ -98,44 +97,61 @@ stream_wopen (const char * command, boolean is_file)
 }
 
 struct stream *
-stream_wopen_backup (const char * command, boolean is_file, 
+stream_wopen_backup (const char * command, boolean is_file,
 		     enum backup_type backup_type)
 {
   return _stream_wopen (command, is_file, backup_type);
 }
 
-/*
- * Uses the perl convention to know whether file or pipe
- * I.e., command starts by '>', or '|'
- */
+
+/* Open a r or w stream depending in PERL_COMMAND.  PERL_COMMAND can
+   be:
+   - `> file', open a w-stream on FILE, and backup is asked.
+   - `| cmd', open a w-pipe on CMD.
+   - `cmd |', open a r-pipe on CMD.
+   - otherwise, PERL_COMMAND is a file to read.
+
+   Once the decoding done, NAME points to the first char of the file
+   name.
+*/
+
 struct stream *
-stream_perl_open (const char * perl_command)
+stream_perl_open_backup (const char * perl_command,
+			 enum backup_type backup,
+			 const char **name)
 {
-  char const * command;
   char * cp;
   int len;
 
-  command = perl_command + strspn (perl_command, "\t >|");
+  message (msg_file,
+	   (stderr, "perl-open (%s)\n", quotearg (perl_command)));
 
-  switch (*perl_command) {
-  case '|':
-    return _stream_wopen (command, TRUE, none);
+  *name = perl_command + strspn (perl_command, "\t >|");
 
-  case '>':
-    return _stream_wopen (command, FALSE, none);
-
-  default:
-    len = strlen (perl_command);
-    switch (perl_command [len - 1]) {
+  switch (*perl_command)
+    {
     case '|':
-      cp = ALLOCA (char, len);
-      strncpy (cp, command, len - 1);
-      return _stream_ropen (cp, TRUE);
-      
+      return _stream_wopen (*name, false, none);
+
+    case '>':
+      return _stream_wopen (*name, true, backup);
+
     default:
-      return _stream_ropen (command, FALSE);
+      /* Open for reading. */
+      len = strlen (perl_command);
+      switch (perl_command [len - 1])
+	{
+	case '|':
+	  /* Read a pipe. */
+	  cp = ALLOCA (char, len);
+	  strncpy (cp, *name, len - 1);
+	  return _stream_ropen (cp, false);
+
+	default:
+	  /* Read a file. */
+	  return _stream_ropen (*name, true);
+	}
     }
-  }
 }
 
 /*
