@@ -1,16 +1,16 @@
 /*
  * stream.h
  *
- * 
+ * Open streams on pipes or files.
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  *
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -34,27 +34,30 @@
 #define _STREAM_H_
 #include "xbackupfile.h"
 
-struct stream {
+struct stream
+{
   boolean is_file;	/* Is it a file (no : pipe)		*/
-  FILE * fp;
+  FILE *fp;
 };
 
 /* Open for reading */
-struct stream * stream_ropen PARAMS ((const char * command, 
+struct stream * stream_ropen PARAMS ((const char * command,
 				      boolean is_file));
 /* Open for writing */
 struct stream * stream_wopen PARAMS ((const char * command,
 				      boolean is_file));
 /* Open for writing, using backup services */
-struct stream * stream_wopen_backup PARAMS ((const char * command, 
-					     boolean is_file, 
+struct stream * stream_wopen_backup PARAMS ((const char * command,
+					     boolean is_file,
 					     enum backup_type backup_type));
 
 /*
  * Uses the perl convention to know whether file or pipe
  * I.e., command starts by '>', or '|'
  */
-struct stream * stream_perl_open PARAMS ((char const * perl_command));
+struct stream * stream_perl_open_backup
+   PARAMS ((char const * perl_command, enum backup_type backup,
+	    const char **name));
 
 /* Closes and frees.  */
 void stream_close PARAMS ((struct stream * stream));
