@@ -21,9 +21,15 @@
 #ifndef WITH_DMALLOC
 
 #include "xsystypes.h"
-#include "xnls.h"
 #include "error.h"
 
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(Text) gettext (Text)
+#else
+# define _(Text) Text
+#endif
+
 #ifndef EXIT_FAILURE
 # define EXIT_FAILURE 1
 #endif
@@ -35,7 +41,7 @@ static void * fixup_null_alloc PARAMS ((size_t n));
    The caller may set it to some other value.  */
 int xmalloc_exit_failure = EXIT_FAILURE;
 
-static void * 
+static void *
 fixup_null_alloc (size_t n)
 {
   void * p;
@@ -50,7 +56,7 @@ fixup_null_alloc (size_t n)
 
 /* Allocate N bytes of memory dynamically, with error checking.  */
 
-void * 
+void *
 xmalloc (size_t n)
 {
   void * p;
@@ -63,7 +69,7 @@ xmalloc (size_t n)
 
 /* Allocate memory for N elements of S bytes, with error checking.  */
 
-void * 
+void *
 xcalloc (size_t n, size_t s)
 {
   void * p;
@@ -78,7 +84,7 @@ xcalloc (size_t n, size_t s)
    with error checking.
    If P is NULL, run xmalloc.  */
 
-void * 
+void *
 xrealloc (void * p, size_t n)
 {
   if (p == 0)
@@ -89,14 +95,4 @@ xrealloc (void * p, size_t n)
   return p;
 }
 
-/*
- * Copy STRING in a newly xmalloc'd chunk
- */
-char *
-xstrdup (const char * string)
-{
-  return strcpy (xmalloc (strlen (string) + 1), string);
-}
-
-
 #endif /* not defined WITH_DMALLOC */
