@@ -21,7 +21,7 @@
 
 #include <obstack.h>
 
-#define obstack_chunk_alloc malloc
+#define obstack_chunk_alloc xmalloc
 #define obstack_chunk_free free
 
 #include "xmalloc.h"
