@@ -1,6 +1,6 @@
 /* xstrrpl.c -- replacement of substrings
-   Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
-   Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+   Copyright (c) 1988-1993 Miguel Santana
+   Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #endif
 
 #include <stdio.h>
-
+#include "xstrrpl.h"
 #include "xstdlib.h"
 #include "xstring.h"
 #include "xstdarg.h"
@@ -33,7 +33,6 @@
 #endif
 
 #include "xmalloc.h"
-#include "xstrrpl.h"
 
 extern char * stpcpy();
 
@@ -42,9 +41,7 @@ extern char * stpcpy();
    where subst = { {"1", "11"}, {"3", "333"}, { "4", ""}}
    */
 char *
-xstrrpl (string, subst)
-     const char * string;
-     const char * subst[][2];
+xstrrpl (const char * string, const char * subst[][2])
 {
   char * res, * cp;
   int i;
@@ -57,9 +54,9 @@ xstrrpl (string, subst)
      to return NULL.  */
   res = MALLOC (strlen (string) * (max + 1) + 1);
   cp = res;
-  
+
   /* Perform the substitutions */
-  while (*string) 
+  while (*string)
     {
       for (i = 0 ; subst[i][0] ; i++)
 	if (!strncmp (subst[i][0], string, strlen (subst[i][0])))
@@ -72,22 +69,20 @@ xstrrpl (string, subst)
     __next_char:
       /* nothing */;
     }
-  
+
   *cp = '\0';
 #if DEBUG
   assert (strlen (string) * (max + 1) < cp - res);
 #endif
 
   res = REMALLOC (res, cp - res + 1);
- 
+
   return res;
 }
 
 /* Destructive version */
 void
-strrpl (string, subst)
-     char ** string;
-     const char * subst[][2];
+strrpl (char ** string, const char * subst[][2])
 {
   char * res;
   res = xstrrpl (*string, subst);
@@ -146,7 +141,7 @@ vstrrpl (va_alist)
   va_list ap;
   const char * subst[100][2];	/* Hope this is enough :) */
   int i = 0;
-  
+
   /* Copy arguments into `args'. */
 #if __STDC__
   va_start (ap, string);
