@@ -1,15 +1,13 @@
 /*
- * buffer.c
+ * buffer.h - read line by line with conversion of EOL types
  *
- * routines of input
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
- * $Id: buffer.h,v 1.31 1998/04/04 17:57:37 demaille Exp $
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -32,6 +30,7 @@
 
 #ifndef _BUFFER_H_
 #define _BUFFER_H_
+
 #include "xobstack.h"
 
 /****************************************************************/
@@ -40,7 +39,8 @@
 /*
  * How the eol should be handled
  */
-enum eol_e {
+enum eol_e
+{
   eol_r, 	/* \r only */
   eol_n,	/* \n 	*/
   eol_rn,	/* \r\n	*/
@@ -52,16 +52,17 @@ const char * eol_to_string PARAMS ((enum eol_e eol));
 enum eol_e option_string_to_eol PARAMS ((const char * option,
 					 const char * arg));
 
-/* 
+/*
  * The buffer structure, which encloses a lower case version to
  * speed up case insensitive parsing
  * There are two types:
  * - buf != NULL :    input comes from stream_string, read line by line.
  * - stream != NULL : input comes from stream, read line by line.
  * - both non NULL :  first read buf, then stream.
- * 
+ *
  */
-typedef struct buffer_s {
+typedef struct buffer_s
+{
   const uchar * buf;	/* != 0 -> buffer on a string		*/
   size_t bufsize;	/* size of buf				*/
   size_t bufoffset;	/* Used when a buffer string		*/
@@ -69,7 +70,7 @@ typedef struct buffer_s {
   boolean pipe_p;	/* true -> stream has been popened	*/
 
   uchar * content;	/* Exactly what is read			*/
-  enum eol_e eol;	/* What is an end of line?		*/  
+  enum eol_e eol;	/* What is an end of line?		*/
   boolean lower_case;
   uchar * value;	/* if LOWER_CASE, then lower case of content */
   size_t line;		/* Num of the current line 		*/
@@ -79,7 +80,7 @@ typedef struct buffer_s {
   struct obstack obstack;
 } buffer_t;
 
-void buffer_stream_set PARAMS ((buffer_t * buffer, 
+void buffer_stream_set PARAMS ((buffer_t * buffer,
 				FILE * stream, enum eol_e eol));
 
 void buffer_pipe_set PARAMS ((buffer_t * buffer,
@@ -88,7 +89,7 @@ void buffer_pipe_set PARAMS ((buffer_t * buffer,
 void buffer_string_set PARAMS ((buffer_t * buffer,
 				const uchar * string, enum eol_e eol));
 
-void buffer_buffer_set PARAMS ((buffer_t * buffer, 
+void buffer_buffer_set PARAMS ((buffer_t * buffer,
 				const uchar * buf, size_t bufsize,
 				enum eol_e eol));
 
