@@ -36,8 +36,8 @@ void print PARAMS ((uchar * name, int * native_jobs, int * delegated_jobs));
 void print_toc PARAMS ((const uchar * name, const uchar * value,
 		       int * native_jobs));
 
-void message_job_pages_printed PARAMS ((a2ps_job * job));
-void message_nothing_printed PARAMS ((void));
+void msg_job_pages_printed PARAMS ((a2ps_job * job));
+void msg_nothing_printed PARAMS ((void));
 
 void guess PARAMS ((uchar * name));
 #endif /* not defined(_GENERATE_H_) */
