@@ -43,14 +43,14 @@
 /*
  * Grammar for parsing the style sheets
  *
- * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ * Copyright (c) 1988-1993 Miguel Santana
+ * Copyright (c) 1995-1999 Akim Demaille, Miguel Santana
  *
  */
 
 /*
  * This file is part of a2ps
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -68,14 +68,14 @@
  */
 
 /*
- * $Id: parsessh.c,v 1.133 1998/03/04 18:03:28 demaille Exp $
+ * $Id: parsessh.y,v 1.71 1998/03/04 17:27:51 demaille Exp $
  */
 
 #include "a2ps.h"
 #include "jobs.h"
 #include "ffaces.h"
 #include "ssheet.h"
-#include "msg.h"
+#include "message.h"
 #include "routines.h"
 #include "yy2ssh.h"
 #include "regex.h"
@@ -244,14 +244,14 @@ static const short yyrhs[] = {    40,
 static const short yyrline[] = { 0,
    128,   135,   144,   148,   157,   163,   172,   178,   186,   191,
    196,   204,   211,   218,   221,   224,   227,   230,   244,   246,
-   248,   251,   252,   267,   269,   271,   276,   284,   293,   300,
-   307,   312,   323,   331,   336,   340,   344,   348,   352,   358,
-   364,   375,   381,   388,   397,   403,   408,   414,   428,   432,
-   437,   448,   452,   458,   464,   469,   474,   485,   490,   496,
-   505,   509,   515,   522,   528,   533,   544,   549,   554,   563,
-   566,   572,   591,   603,   610,   617,   624,   631,   638,   645,
-   652,   660,   666,   673,   677,   683,   692,   698,   704,   711,
-   716
+   248,   251,   252,   267,   269,   271,   276,   283,   291,   298,
+   305,   310,   321,   329,   334,   338,   342,   346,   350,   356,
+   362,   373,   379,   386,   395,   401,   406,   412,   426,   430,
+   435,   446,   450,   456,   462,   467,   472,   483,   488,   494,
+   503,   507,   513,   520,   526,   531,   542,   547,   552,   561,
+   564,   570,   589,   601,   608,   615,   622,   629,   636,   643,
+   650,   658,   664,   671,   675,   681,   690,   696,   702,   709,
+   714
 };
 #endif
 
@@ -908,13 +908,13 @@ yyreduce:
 
 case 1:
 #line 130 "parsessh.y"
-{ 
+{
       parsed_style_sheet = yyvsp[0].sheet;
     ;
     break;}
 case 2:
 #line 137 "parsessh.y"
-{ 
+{
 	yyvsp[-2].sheet->name = yyvsp[-4].string;
 	yyvsp[-2].sheet->key = "<No key yet>";
 	yyval.sheet = yyvsp[-2].sheet;
@@ -922,8 +922,8 @@ case 2:
     break;}
 case 3:
 #line 145 "parsessh.y"
-{ 
-	  yyval.sheet = new_style_sheet ((const uchar *) "<no name>"); 
+{
+	  yyval.sheet = new_style_sheet ((const uchar *) "<no name>");
 	;
     break;}
 case 4:
@@ -1048,12 +1048,12 @@ case 17:
     break;}
 case 18:
 #line 230 "parsessh.y"
-{ 
+{
 	  /* Make sure now that we won't encounter new tokens.
 	   * This avoids nasty error messages, or worse:
 	   * unexpected behavior at run time */
 	  if (!style_sheet_set_requirement (yyvsp[-1].sheet, (const char *) yyvsp[0].string))
-	    error (1, 0, 
+	    error (1, 0,
 		   _("cannot process `%s' which requires a2ps version %s"),
 		   sshfilename, yyvsp[0].string);
 	;
@@ -1079,7 +1079,7 @@ case 23:
 {
 	  size_t len1;
 	  size_t len2;
-	  
+
 	  len1 = ustrlen (yyvsp[-1].string);
 	  yyvsp[-1].string[len1] = '\n';
 	  len2 = ustrlen (yyvsp[0].string);
@@ -1103,42 +1103,40 @@ case 26:
     break;}
 case 27:
 #line 277 "parsessh.y"
-{ 
-	  /* The list of keys of style sheets from which it
-	   * inherits */
-  	  yyval.array = yyvsp[-2].array; 
+{
+	  /* The list of keys of style sheets from which it inherits */
+  	  yyval.array = yyvsp[-2].array;
 	;
     break;}
 case 28:
-#line 285 "parsessh.y"
-{ 
-	  /* Create a list of ancestors, and drop the new
-	   * one in */
-	  yyval.array = da_new ("Ancestors tmp", 2, 
+#line 284 "parsessh.y"
+{
+	  /* Create a list of ancestors, and drop the new one in */
+	  yyval.array = da_new ("Ancestors tmp", 2,
 		       da_linear, 2,
 		       (da_print_func_t) da_str_print, NULL);
 	  da_append (yyval.array, yyvsp[0].string);
 	;
     break;}
 case 29:
-#line 294 "parsessh.y"
+#line 292 "parsessh.y"
 {
-	  da_append (yyvsp[-2].array, yyvsp[0].string); 
+	  da_append (yyvsp[-2].array, yyvsp[0].string);
 	  yyval.array = yyvsp[-2].array;
 	;
     break;}
 case 30:
-#line 300 "parsessh.y"
+#line 298 "parsessh.y"
 { yyval.sensitiveness = yyvsp[0].sensitiveness ; ;
     break;}
 case 31:
-#line 309 "parsessh.y"
+#line 307 "parsessh.y"
 {
 	  yyval.pattern = yyvsp[0].pattern;
 	;
     break;}
 case 32:
-#line 313 "parsessh.y"
+#line 311 "parsessh.y"
 {
 	  /* Concatenate $2 to $1 makes $$ */
 	  yyval.pattern = yyvsp[-1].pattern;
@@ -1149,80 +1147,80 @@ case 32:
 	;
     break;}
 case 33:
-#line 325 "parsessh.y"
-{ 
+#line 323 "parsessh.y"
+{
 	  yyval.array = rhs_new ();
 	  rhs_add (yyval.array, yyvsp[0].faced_string);
 	;
     break;}
 case 34:
-#line 333 "parsessh.y"
-{ 
+#line 331 "parsessh.y"
+{
 	  yyval.faced_string = faced_string_new  (yyvsp[-1].string, 0, yyvsp[0].fface);
 	;
     break;}
 case 35:
-#line 337 "parsessh.y"
-{ 
+#line 335 "parsessh.y"
+{
 	  yyval.faced_string = faced_string_new  (yyvsp[0].string, 0, No_fface);
 	;
     break;}
 case 36:
-#line 341 "parsessh.y"
-{ 
+#line 339 "parsessh.y"
+{
 	  yyval.faced_string = faced_string_new  (NULL, 0, yyvsp[0].fface);
 	;
     break;}
 case 37:
-#line 345 "parsessh.y"
-{ 
+#line 343 "parsessh.y"
+{
 	  yyval.faced_string = faced_string_new  (UNULL, yyvsp[-1].integer, yyvsp[0].fface);
 	;
     break;}
 case 38:
-#line 349 "parsessh.y"
-{ 
+#line 347 "parsessh.y"
+{
 	  yyval.faced_string = faced_string_new  (UNULL, yyvsp[0].integer, No_fface);
 	;
     break;}
 case 39:
-#line 353 "parsessh.y"
+#line 351 "parsessh.y"
 {
 	  yyval.faced_string = faced_string_new (yyvsp[0].string, 0, Symbol_fface);
 	;
     break;}
 case 40:
-#line 360 "parsessh.y"
+#line 358 "parsessh.y"
 {
 	  yyval.array = rhs_new ();
 	  rhs_add (yyval.array, yyvsp[0].faced_string);
 	;
     break;}
 case 41:
-#line 366 "parsessh.y"
+#line 364 "parsessh.y"
 {
 	  rhs_add (yyvsp[-2].array, yyvsp[0].faced_string);
 	  yyval.array = yyvsp[-2].array;
 	;
     break;}
 case 42:
-#line 377 "parsessh.y"
-{ 
-	  fface_set_face (yyval.fface, yyvsp[0].face); 
-	  fface_reset_flags (yyval.fface); 
+#line 375 "parsessh.y"
+{
+	  fface_set_face (yyval.fface, yyvsp[0].face);
+	  fface_reset_flags (yyval.fface);
 	;
     break;}
 case 43:
-#line 382 "parsessh.y"
-{ 
-	  fface_reset_face (yyval.fface); 
+#line 380 "parsessh.y"
+{
+	  fface_reset_face (yyval.fface);
 	  fface_set_flags (yyval.fface, yyvsp[0].fflags);
 	  /* If there is no face, then set Invisible */
 	  fface_add_flags (yyval.fface, ff_Invisible);
 	;
     break;}
 case 44:
-#line 389 "parsessh.y"
+#line 387 "parsessh.y"
 {
 	  yyval.fface = yyvsp[-1].fface;
 	  /* If there is no face, then set Invisible */
@@ -1231,103 +1229,103 @@ case 44:
 	;
     break;}
 case 45:
-#line 399 "parsessh.y"
-{ 
-	  fface_set_face(yyval.fface, yyvsp[0].face); 
-	  fface_reset_flags(yyval.fface); 
+#line 397 "parsessh.y"
+{
+	  fface_set_face(yyval.fface, yyvsp[0].face);
+	  fface_reset_flags(yyval.fface);
 	;
     break;}
 case 46:
-#line 404 "parsessh.y"
-{ 
-	  fface_reset_face(yyval.fface); 
-	  fface_set_flags(yyval.fface, yyvsp[0].fflags); 
+#line 402 "parsessh.y"
+{
+	  fface_reset_face(yyval.fface);
+	  fface_set_flags(yyval.fface, yyvsp[0].fflags);
 	;
     break;}
 case 47:
-#line 409 "parsessh.y"
-{ 
+#line 407 "parsessh.y"
+{
 	  /* FIXME: Overloading of the face should be forbidden */
 	  yyval.fface = yyvsp[-2].fface;
-	  fface_set_face(yyval.fface, yyvsp[0].face); 
+	  fface_set_face(yyval.fface, yyvsp[0].face);
 	;
     break;}
 case 48:
-#line 415 "parsessh.y"
-{ 
+#line 413 "parsessh.y"
+{
 	  yyval.fface = yyvsp[-2].fface;
-	  fface_add_flags(yyval.fface, yyvsp[0].fflags); 
+	  fface_add_flags(yyval.fface, yyvsp[0].fflags);
 	;
     break;}
 case 49:
-#line 429 "parsessh.y"
+#line 427 "parsessh.y"
 {
-	  yyval.rule = rule_new (yyvsp[-1].string, NULL, yyvsp[0].array);	
+	  yyval.rule = rule_new (yyvsp[-1].string, NULL, yyvsp[0].array);
 	;
     break;}
 case 50:
-#line 433 "parsessh.y"
+#line 431 "parsessh.y"
 {
 	  yyval.rule = rule_new (yyvsp[0].string, NULL,
 			   rhs_new_single (UNULL, 0, No_fface));
 	;
     break;}
 case 51:
-#line 438 "parsessh.y"
+#line 436 "parsessh.y"
 {
 	  yyval.rule = rule_new (yyvsp[-2].string, NULL, yyvsp[-1].array);
 	;
     break;}
 case 52:
-#line 448 "parsessh.y"
-{ 
+#line 446 "parsessh.y"
+{
 	  words_set_no_face (yyvsp[-1].words, yyvsp[-3].fface);
 	  yyval.words = yyvsp[-1].words;
 	;
     break;}
 case 53:
-#line 452 "parsessh.y"
-{ 
+#line 450 "parsessh.y"
+{
   	  /* First of all, the No_face must be turned into Plain */
   	  yyval.words = yyvsp[-1].words;
 	;
     break;}
 case 54:
-#line 460 "parsessh.y"
+#line 458 "parsessh.y"
 {
 	  yyval.words = words_new ("Keywords: Strings", "Keywords: Regexps", 100, 100);
 	  words_add_string (yyval.words, yyvsp[0].rule);
 	;
     break;}
 case 55:
-#line 465 "parsessh.y"
+#line 463 "parsessh.y"
 {
 	  yyval.words = words_new ("Keywords: Strings", "Keywords: Regexps", 100, 100);
 	  words_add_regex (yyval.words, yyvsp[0].rule);
 	;
     break;}
 case 56:
-#line 470 "parsessh.y"
+#line 468 "parsessh.y"
 {
-	  words_add_string (yyvsp[-2].words, yyvsp[0].rule); 
+	  words_add_string (yyvsp[-2].words, yyvsp[0].rule);
 	  yyval.words = yyvsp[-2].words;
 	;
     break;}
 case 57:
-#line 475 "parsessh.y"
+#line 473 "parsessh.y"
 {
-	  words_add_regex (yyvsp[-2].words, yyvsp[0].rule); 
+	  words_add_regex (yyvsp[-2].words, yyvsp[0].rule);
 	  yyval.words = yyvsp[-2].words;
 	;
     break;}
 case 58:
-#line 487 "parsessh.y"
-{ 
+#line 485 "parsessh.y"
+{
 	  yyval.rule = keyword_rule_new (UNULL, yyvsp[-1].pattern, yyvsp[0].array);
 	;
     break;}
 case 59:
-#line 491 "parsessh.y"
+#line 489 "parsessh.y"
 {
 	  yyval.rule = keyword_rule_new (UNULL, yyvsp[0].pattern,
 				   rhs_new_single (UNULL, 0,
@@ -1335,106 +1333,106 @@ case 59:
 	;
     break;}
 case 60:
-#line 497 "parsessh.y"
+#line 495 "parsessh.y"
 {
 	  yyval.rule = keyword_rule_new (UNULL, yyvsp[-2].pattern, yyvsp[-1].array);
 	;
     break;}
 case 61:
-#line 505 "parsessh.y"
-{ 
+#line 503 "parsessh.y"
+{
 	  words_set_no_face (yyvsp[-1].words, yyvsp[-3].fface);
 	  yyval.words = yyvsp[-1].words;
 	;
     break;}
 case 62:
-#line 509 "parsessh.y"
-{ 
+#line 507 "parsessh.y"
+{
   	  /* First of all, the No_face must be turned into Plain */
   	  yyval.words = yyvsp[-1].words;
 	;
     break;}
 case 63:
-#line 517 "parsessh.y"
-{ 
+#line 515 "parsessh.y"
+{
 	  yyval.words = words_new ("Operators: Strings", "Operators: Regexps",
 			  100, 100);
 	  words_add_string (yyval.words, yyvsp[0].rule);
 	;
     break;}
 case 64:
-#line 523 "parsessh.y"
-{ 
+#line 521 "parsessh.y"
+{
 	  yyval.words = words_new ("Operators: Strings", "Operators: Regexps",
 			  100, 100);
 	  words_add_regex (yyval.words, yyvsp[0].rule);
 	;
     break;}
 case 65:
-#line 529 "parsessh.y"
+#line 527 "parsessh.y"
 {
-	  words_add_string (yyvsp[-2].words, yyvsp[0].rule); 
+	  words_add_string (yyvsp[-2].words, yyvsp[0].rule);
 	  yyval.words = yyvsp[-2].words;
 	;
     break;}
 case 66:
-#line 534 "parsessh.y"
+#line 532 "parsessh.y"
 {
-	  words_add_regex (yyvsp[-2].words, yyvsp[0].rule); 
+	  words_add_regex (yyvsp[-2].words, yyvsp[0].rule);
 	  yyval.words = yyvsp[-2].words;
 	;
     break;}
 case 67:
-#line 546 "parsessh.y"
+#line 544 "parsessh.y"
 {
 	  yyval.rule = rule_new (UNULL, yyvsp[-1].pattern, yyvsp[0].array);
 	;
     break;}
 case 68:
-#line 550 "parsessh.y"
+#line 548 "parsessh.y"
 {
 	  yyval.rule = rule_new (UNULL, yyvsp[0].pattern,
 			   rhs_new_single (UNULL, 0, No_fface));
 	;
     break;}
 case 69:
-#line 555 "parsessh.y"
+#line 553 "parsessh.y"
 {
 	  yyval.rule = rule_new (UNULL, yyvsp[-2].pattern, yyvsp[-1].array);
 	;
     break;}
 case 70:
-#line 563 "parsessh.y"
+#line 561 "parsessh.y"
 { yyval.array = yyvsp[-2].array; ;
     break;}
 case 71:
-#line 566 "parsessh.y"
-{ 
-	  yyval.array = da_new ("Sequence tmp", 100, 
+#line 564 "parsessh.y"
+{
+	  yyval.array = da_new ("Sequence tmp", 100,
 		       da_linear, 100,
 		       (da_print_func_t) sequence_self_print, NULL);
 	  da_append (yyval.array, yyvsp[0].sequence);
 	;
     break;}
 case 72:
-#line 572 "parsessh.y"
+#line 570 "parsessh.y"
 {
-	  da_append (yyvsp[-2].array, yyvsp[0].sequence); 
+	  da_append (yyvsp[-2].array, yyvsp[0].sequence);
 	  yyval.array = yyvsp[-2].array;
 	;
     break;}
 case 73:
-#line 596 "parsessh.y"
+#line 594 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-4].string, NULL,
-				    rhs_new_single (yyvsp[-3].string, 0, 
+				    rhs_new_single (yyvsp[-3].string, 0,
 							     Symbol_fface));
 	  yyval.sequence = sequence_new (open_rule, yyvsp[-2].fface, yyvsp[-1].words, yyvsp[0].words);
 	;
     break;}
 case 74:
-#line 604 "parsessh.y"
+#line 602 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-5].string, NULL,
@@ -1443,25 +1441,25 @@ case 74:
 	;
     break;}
 case 75:
-#line 611 "parsessh.y"
+#line 609 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-4].string, NULL,
-				    rhs_new_single (UNULL, 0, yyvsp[-3].fface)); 
+				    rhs_new_single (UNULL, 0, yyvsp[-3].fface));
 	  yyval.sequence = sequence_new (open_rule, yyvsp[-2].fface, yyvsp[-1].words, yyvsp[0].words);
 	;
     break;}
 case 76:
-#line 618 "parsessh.y"
+#line 616 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-4].string, NULL,
-				    rhs_new_single (yyvsp[-3].string, 0, yyvsp[-2].fface)); 
+				    rhs_new_single (yyvsp[-3].string, 0, yyvsp[-2].fface));
 	  yyval.sequence = sequence_new (open_rule, yyvsp[-2].fface, yyvsp[-1].words, yyvsp[0].words);
 	;
     break;}
 case 77:
-#line 625 "parsessh.y"
+#line 623 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-3].string, NULL,
@@ -1470,7 +1468,7 @@ case 77:
 	;
     break;}
 case 78:
-#line 632 "parsessh.y"
+#line 630 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (UNULL, yyvsp[-5].pattern,
@@ -1479,7 +1477,7 @@ case 78:
 	;
     break;}
 case 79:
-#line 639 "parsessh.y"
+#line 637 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (UNULL, yyvsp[-4].pattern,
@@ -1488,7 +1486,7 @@ case 79:
 	;
     break;}
 case 80:
-#line 646 "parsessh.y"
+#line 644 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (UNULL, yyvsp[-4].pattern,
@@ -1497,17 +1495,17 @@ case 80:
 	;
     break;}
 case 81:
-#line 653 "parsessh.y"
+#line 651 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (UNULL, yyvsp[-3].pattern,
-				    rhs_new_single (UNULL, 0, 
+				    rhs_new_single (UNULL, 0,
 							     No_fface));
 	  yyval.sequence = sequence_new (open_rule, yyvsp[-2].fface, yyvsp[-1].words, yyvsp[0].words);
 	;
     break;}
 case 82:
-#line 661 "parsessh.y"
+#line 659 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (yyvsp[-5].string, NULL, yyvsp[-4].array);
@@ -1515,7 +1513,7 @@ case 82:
 	;
     break;}
 case 83:
-#line 667 "parsessh.y"
+#line 665 "parsessh.y"
 {
 	  struct rule * open_rule;
 	  open_rule = rule_new (UNULL, yyvsp[-5].pattern, yyvsp[-4].array);
@@ -1523,19 +1521,19 @@ case 83:
 	;
     break;}
 case 84:
-#line 674 "parsessh.y"
-{ 
+#line 672 "parsessh.y"
+{
 	  yyval.sequence = new_C_string_sequence ("\"");
 	;
     break;}
 case 85:
-#line 678 "parsessh.y"
+#line 676 "parsessh.y"
 {
 	  yyval.sequence = new_C_string_sequence ("\'");
 	;
     break;}
 case 86:
-#line 685 "parsessh.y"
+#line 683 "parsessh.y"
 {
 	  /* This is a shortcut which means "up to the end of the line". */
 	  yyval.words = words_new ("Closing: Strings", "Closing: Regexps", 2, 2);
@@ -1545,7 +1543,7 @@ case 86:
 	;
     break;}
 case 87:
-#line 693 "parsessh.y"
+#line 691 "parsessh.y"
 {
 	  /* Only one */
 	  yyval.words = words_new ("Closing: Strings", "Closing: Regexps", 2, 2);
@@ -1553,7 +1551,7 @@ case 87:
 	;
     break;}
 case 88:
-#line 699 "parsessh.y"
+#line 697 "parsessh.y"
 {
 	  /* Only one */
 	  yyval.words = words_new ("Closing: Strings", "Closing: Regexps", 2, 2);
@@ -1561,21 +1559,21 @@ case 88:
 	;
     break;}
 case 89:
-#line 705 "parsessh.y"
-{ 
+#line 703 "parsessh.y"
+{
 	  /* Several, comma separated, between () */
-	  yyval.words = yyvsp[-1].words; 
+	  yyval.words = yyvsp[-1].words;
 	;
     break;}
 case 90:
-#line 713 "parsessh.y"
-{ 
+#line 711 "parsessh.y"
+{
 	  yyval.words = words_new ("Exceptions: Strings", "Exceptions: Regexps", 1, 1);
 	;
     break;}
 case 91:
-#line 717 "parsessh.y"
-{ 
+#line 715 "parsessh.y"
+{
 	  yyval.words = yyvsp[-1].words;
 	;
     break;}
@@ -1777,7 +1775,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 721 "parsessh.y"
+#line 719 "parsessh.y"
 
 
 void
@@ -1830,12 +1828,12 @@ parse_style_sheet (const char * filename)
 
   sshlex_initialize ();
 
-  if (message_test (msg_parse))
+  if (msg_test (msg_parse))
     yydebug = TRUE;
   res = yyparse ();  /* FIXME: test the result of parsing */
 
-  if (message_test (msg_sheet)) {
-    fprintf (stderr, "---------- Right after parsing of %s\n", 
+  if (msg_test (msg_sheet)) {
+    fprintf (stderr, "---------- Right after parsing of %s\n",
 	     parsed_style_sheet->key);
     style_sheet_self_print (parsed_style_sheet, stderr);
     fprintf (stderr, "---------- End of after parsing of %s\n",
