@@ -4,12 +4,12 @@
  * definition of the languages style sheets
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
  * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
- * $Id: ssheet.c,v 1.100 1998/03/04 17:27:52 demaille Exp $
+ * $Id: ssheet.c 1.2.1.3 Thu, 22 Oct 1998 14:21:51 +0200 demaille $
  */
 
 /*
  * This file is part of a2ps.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -31,17 +31,23 @@
 #include "ssheet.h"
 #include "pathwalk.h"
 #include "jobs.h"
-#include "msg.h"
+#include "message.h"
 #include "filtdir.h"
 #include "xalloca.h"
 #include "versions.h"
 #include "document.h"
+#include "metaseq.h"
+#include "title.h"
+#include "quotearg.h"
 
 /* Suffix of the style sheet files */
 #define SSH_SUFFIX ".ssh"
 
+/* Separator of ancestors in mixed style sheets. */
+#define MIXED_SHEET_SEP ",;+"
+
 extern a2ps_job * job;
-extern struct hash_table * style_sheets;
+extern struct hash_table_s * style_sheets;
 
 /* Comes from regex */
 extern char * re_syntax_table;
@@ -119,7 +125,7 @@ faced_string_self_print (struct faced_string * faced_string, FILE * stream)
 {
   if (faced_string->string)
     fprintf (stream, "\"%s\": ", faced_string->string);
-  else 
+  else
     fprintf (stream, "\\%d: ", faced_string->reg_ref);
 
   fface_self_print (faced_string->face, stream);
@@ -167,7 +173,7 @@ style_sheet_set_requirement (struct style_sheet * sheet,
 
   if (version_cmp (sheet->requirement, a2ps_version) > 0)
     return FALSE;
-  
+
   return TRUE;
 }
 
@@ -208,7 +214,7 @@ rhs_set_no_face (struct darray * dest, struct fface_s face)
   size_t i;
 
   for (i = 0 ; i < dest->len ; i++)
-    if (fface_squ (((struct faced_string *) dest->content [i])->face, 
+    if (fface_squ (((struct faced_string *) dest->content [i])->face,
 		   No_fface))
       ((struct faced_string *) dest->content [i])->face = face;
 }
@@ -231,7 +237,7 @@ void
 rhs_self_print (struct darray * rhs, FILE * stream)
 {
   size_t i;
-  
+
   putc ('(', stream);
   for (i = 0 ; i < rhs->len ; i++)
     faced_string_self_print ((struct faced_string *) rhs->content [i], stream);
@@ -269,20 +275,20 @@ new_rule_internal_regexp (struct pattern * pattern,
   /* This is a regular expression.  We want to keep the original
    * pattern to ease the debugging of a style sheet */
   res->word = (uchar *) pattern->pattern;
-  
+
   /* Build the regex structure, and compile the pattern */
   res->regex = ALLOC (struct re_pattern_buffer, 1);
   res->regex->translate = NULL;
   res->regex->fastmap = NULL;
   res->regex->buffer = NULL;
   res->regex->allocated = 0;
-  
+
   error_msg = re_compile_pattern (pattern_to_compile, pattern_len,
 				  res->regex);
   /* The pattern is nul terminated, no fear to have */
   if (error_msg)
     error (1, 0, _("cannot compile regular expression `%s': %s"),
-	   pattern_to_compile, error_msg);
+	   quotearg (pattern_to_compile), error_msg);
   FREE (pattern);
   res->rhs = rhs;
   return res;
@@ -299,14 +305,14 @@ rule_new (uchar * word, struct pattern * pattern,
     return new_rule_internal_regexp (pattern,
 				       pattern->pattern, pattern->len,
 				       rhs);
-  else 
+  else
     return new_rule_internal_word (word,
 				     rhs);
 }
 
 /*
  * Dealing with the rules and operators
- * This is pretty much the same as the previous case, 
+ * This is pretty much the same as the previous case,
  * but the difference is that the regexp much be built appending
  * a \\b at the end, to make sure that, for instance,
  * as a keyword matcher /c*f/ does not match "ccfe": the keyword
@@ -338,10 +344,11 @@ static void
 free_rule (struct rule * rule)
 {
   XFREE (rule->word);
-  if (rule->regex) {
-    regfree (rule->regex);
-    FREE (rule->regex);
-  }
+  if (rule->regex)
+    {
+      regfree (rule->regex);
+      FREE (rule->regex);
+    }
   da_free (rule->rhs, (da_map_func_t) faced_string_free);
   FREE (rule);
 }
@@ -380,7 +387,7 @@ static void
 rules_set_no_faces (struct darray * arr, struct fface_s face)
 {
   size_t i;
-  
+
   for (i = 0 ; i < arr->len ; i++)
     rule_set_no_face (((struct rule *) arr->content[i]), face);
 }
@@ -400,19 +407,19 @@ words_new (/* Regular darray parameters */
 {
   int c;
   struct words * res = ALLOC (struct words, 1);
-  
+
   /* Initialize the underlying darray of strings */
-  res->strings = da_new (name_strings, size, 
-			 da_linear, increment, 
+  res->strings = da_new (name_strings, size,
+			 da_linear, increment,
 			 (da_print_func_t) rule_self_print,
 			 (da_cmp_func_t) rule_cmp);
-  
+
   /* Initialize the underlying darray of regexps */
-  res->regexps = da_new (name_regexps, size, 
-			 da_linear, increment, 
+  res->regexps = da_new (name_regexps, size,
+			 da_linear, increment,
 			 (da_print_func_t) rule_self_print,
 			 (da_cmp_func_t) NULL);
-  
+
   /* Set min and max intervals to void values */
   for (c = 0 ; c < 256 ; c ++)
     res->min[c] = res->max[c] = NULL;
@@ -500,7 +507,7 @@ words_finalize (struct words * list)
 
   /* Sort in alphabetical order */
   da_qsort (list->strings);
-  
+
   /* Build two arrays which contains for each letter, its first
    * and last occurence index in (*DEST) array */
   content = (struct rule **) list->strings->content;
@@ -514,70 +521,79 @@ words_finalize (struct words * list)
 /************************************************************************/
 /*		Handling the ancestors of a style sheet 		*/
 /************************************************************************/
+static struct darray *
+ancestors_new (void)
+{
+  return da_new ("Ancestors", 2,
+		 da_linear, 2,
+		 (da_print_func_t) da_str_print, NULL);
+}
+
 static void
 ancestors_finalize (struct style_sheet * sheet)
 {
   int i;
   struct style_sheet * ancestor = NULL;
-  
+
   if (sheet->ancestors->len == 0)
     return;	/* No inheritance */
 
   /* The string defined rules must be read in order, so that the last
    * definition overwrites the first (and not the converse) */
-  for (i = 0 ; i < (int) sheet->ancestors->len ; i++) {
-    ancestor = get_style_sheet ((char *) sheet->ancestors->content [i]);
-    /* It is possible that the ancestors is not available */
-    if (!ancestor)
-      continue;
-    
-    /* Inherit from their string keywords.  The keywords we already
-     * have are the oldest, hence the one to keep.  This is why
-     * da_1_wins.  Do not free the item, coz' your being killing
-     * another style sheet! */
-    da_merge (sheet->keywords->strings, ancestor->keywords->strings, 
-	      NULL, da_1_wins);
-    /* Inherit from their string operators */
-    da_merge (sheet->operators->strings, ancestor->operators->strings, 
-	      NULL, da_1_wins);
-  }
-  
+  for (i = 0 ; i < (int) sheet->ancestors->len ; i++)
+    {
+      ancestor = get_style_sheet ((char *) sheet->ancestors->content [i]);
+      /* It is possible that the ancestors is not available */
+      if (!ancestor)
+	continue;
+
+      /* Inherit from their string keywords.  The keywords we already
+	 have are the oldest, hence the one to keep.  This is why
+	 da_1_wins.  Do not free the item, coz' your being killing
+	 another style sheet! */
+      da_merge (sheet->keywords->strings, ancestor->keywords->strings,
+		NULL, da_1_wins);
+      /* Inherit from their string operators */
+      da_merge (sheet->operators->strings, ancestor->operators->strings,
+		NULL, da_1_wins);
+    }
+
   /* We want to inherit from the _last_ alphabets if it has not been
-   * defined in this sheet.  The problem is when there are several
-   * ancestors, and some are missing.  To maximize the chance to get
-   * something defined last, we drop the tests for alphabets and
-   * sensitiveness in the backward loop.
-   * The cost is low (in general there are few ancestors), and
-   * ensures that the last existing (not the last required) ancestor
-   * defines the syntactic definitions */
-  for (i = (int) sheet->ancestors->len - 1; i >= 0 ; i--) {
-    ancestor = get_style_sheet ((char *) sheet->ancestors->content [i]);
-    if (!ancestor)
-      continue;
+     defined in this sheet.  The problem is when there are several
+     ancestors, and some are missing.  To maximize the chance to get
+     something defined last, we drop the tests for alphabets and
+     sensitiveness in the backward loop.  The cost is low (in general
+     there are few ancestors), and ensures that the last existing (not
+     the last required) ancestor defines the syntactic definitions */
+  for (i = (int) sheet->ancestors->len - 1; i >= 0 ; i--)
+    {
+      ancestor = get_style_sheet ((char *) sheet->ancestors->content [i]);
+      if (!ancestor)
+	continue;
 
-    /* Inherit of the _last_ alphabets if it has not been defined in
-     * this sheet.  */
-    if (ALPHABET_IS_UNDEFINED (sheet->alpha1))
-      memcpy (sheet->alpha1, ancestor->alpha1, sizeof (uchar) * 256);
-    if (ALPHABET_IS_UNDEFINED (sheet->alpha2))
-      memcpy (sheet->alpha2, ancestor->alpha2, sizeof (uchar) * 256);
-    
-    /* Inherit of the _last_ case sensitivity */
-    if (sheet->sensitiveness == case_undefined)
-      sheet->sensitiveness = ancestor->sensitiveness;
-    
-    /* Inherit from their regex keywords */
-    da_prefix (sheet->keywords->regexps,
-	       ancestor->keywords->regexps);
+      /* Inherit of the _last_ alphabets if it has not been defined in
+       * this sheet.  */
+      if (ALPHABET_IS_UNDEFINED (sheet->alpha1))
+	memcpy (sheet->alpha1, ancestor->alpha1, sizeof (uchar) * 256);
+      if (ALPHABET_IS_UNDEFINED (sheet->alpha2))
+	memcpy (sheet->alpha2, ancestor->alpha2, sizeof (uchar) * 256);
 
-    /* Inherit from their regex operators */
-    da_prefix (sheet->operators->regexps,
-	       ancestor->operators->regexps);
+      /* Inherit of the _last_ case sensitivity */
+      if (sheet->sensitiveness == case_undefined)
+	sheet->sensitiveness = ancestor->sensitiveness;
 
-    /* Inherit from their sequences */
-    /* They must be read in reverse order */
-    da_prefix (sheet->sequences, ancestor->sequences);
-  }
+      /* Inherit from their regex keywords */
+      da_prefix (sheet->keywords->regexps,
+		 ancestor->keywords->regexps);
+
+      /* Inherit from their regex operators */
+      da_prefix (sheet->operators->regexps,
+		 ancestor->operators->regexps);
+
+      /* Inherit from their sequences */
+      /* They must be read in reverse order */
+      da_prefix (sheet->sequences, ancestor->sequences);
+    }
 }
 
 /************************************************************************/
@@ -635,7 +651,7 @@ new_C_exceptions (void)
 {
   struct words * res;
 
-  res = words_new ("C usual exceptions: strings", 
+  res = words_new ("C usual exceptions: strings",
 		   "C usual exceptions: regexps",
 		   3, 3);
   C_add_exception ("\\\'");
@@ -655,7 +671,7 @@ new_C_string_sequence (const char * delimitor)
   res->face = String_fface;
   res->exceptions = new_C_exceptions ();
   res->close = words_new ("C Close: strings", "C Close: regex", 5, 5);
-  words_add_string (res->close, 
+  words_add_string (res->close,
 		    rule_new (xustrdup (delimitor), NULL,
 			      rhs_new_single (NULL, 0, Plain_fface)));
   words_finalize (res->close);
@@ -689,10 +705,7 @@ new_style_sheet (const uchar * name)
   version_set_to_null (res->version);
   res->documentation = UNULL;
   version_set_to_null (res->requirement);
-  res->ancestors = da_new ("Ancestors", 2, 
-			   da_linear, 2,
-			   (da_print_func_t) da_str_print, NULL);
-
+  res->ancestors = ancestors_new ();
   res->sensitiveness = case_undefined;
 
   UNDEFINE_ALPHABET (res->alpha1);
@@ -703,13 +716,57 @@ new_style_sheet (const uchar * name)
   res->operators =  words_new ("Operators: Strings", "Operators: Regexps",
 			       100, 100);
 
-  res->sequences = da_new ("Sequences", 100, 
+  res->sequences = da_new ("Sequences", 100,
 			   da_linear, 100,
 			   (da_print_func_t) sequence_self_print, NULL);
 
   return res;
 }
 
+
+/* Build a style sheet which is only defined by ancestors. */
+static char *
+style_sheet_mixed_new (const uchar * ancestors)
+{
+  struct style_sheet *sheet, *ancestor;
+  char *ancestor_key, *key, *cp;
+  char *ancestors_keys;
+  struct darray *ancestors_array;
+
+  /* It cannot be longer than ancestors_keys. */
+  key = ALLOCA (char, strlen (ancestors) + 1);
+  astrcpy (ancestors_keys, ancestors);
+
+  /* Create the darray of ancestors keys, and build the final key. */
+  ancestors_array = ancestors_new ();
+  ancestor_key = strtok (ancestors_keys, MIXED_SHEET_SEP);
+  da_append (ancestors_array, ancestor_key);
+  ancestor = get_style_sheet (ancestor_key);
+  cp = stpcpy (key, ancestor->key);
+
+  while ((ancestor_key = strtok (NULL, MIXED_SHEET_SEP)))
+    {
+      da_append (ancestors_array, ancestor_key);
+      ancestor = get_style_sheet (ancestor_key);
+      *cp++ =',';
+      cp = stpcpy (cp, ancestor->key);
+    }
+  *cp = NUL;
+  key = xstrdup (key);
+
+  /* Create the style sheet. */
+  message (msg_sheet,
+	   (stderr, "Creating a mixed style sheet \"%s\"\n", key));
+  /* Its name is its key. */
+  sheet = new_style_sheet (key);
+  sheet->key = strdup (key);
+  da_concat (sheet->ancestors, ancestors_array);
+  da_erase (ancestors_array);
+  style_sheet_finalize (sheet);
+
+  return key;
+}
+
 /*
  * Once a style sheet has been read, make it usable (sorts etc.)
  * NOTE: It must _not_ be called from the parser, since
@@ -737,21 +794,25 @@ style_sheet_finalize (struct style_sheet * sheet)
 
   words_finalize (sheet->keywords);
   words_finalize (sheet->operators);
-  if (message_test (msg_sheet)) {
-    fprintf (stderr, "---------- After Finalization of %s\n", sheet->key);
-    style_sheet_self_print (sheet, stderr);
+  if (msg_test (msg_sheet))
+    {
+      fprintf (stderr, "---------- After Finalization of %s\n", sheet->key);
+      style_sheet_self_print (sheet, stderr);
 
-    message (msg_sheet,
-	     (stderr, "Checking coherence of \"%s\" (%s)\n",
-	      sheet->name, sheet->key));
-    if (style_sheet_check (sheet))
-      message (msg_sheet, (stderr, "\"%s\" (%s) is sane.\n", 
-			   sheet->name, sheet->key));
-    else
-      message (msg_sheet, (stderr, "\"%s\" (%s) is corrupted.\n", 
-			   sheet->name, sheet->key));
-    fprintf (stderr, "---------- End of Finalization of %s\n", sheet->key);
-  }
+      message (msg_sheet,
+	       (stderr, "Checking coherence of \"%s\" (%s)\n",
+		sheet->name, sheet->key));
+      if (style_sheet_check (sheet))
+	message (msg_sheet, (stderr, "\"%s\" (%s) is sane.\n",
+			     sheet->name, sheet->key));
+      else
+	message (msg_sheet, (stderr, "\"%s\" (%s) is corrupted.\n",
+			     sheet->name, sheet->key));
+      fprintf (stderr, "---------- End of Finalization of %s\n", sheet->key);
+    }
+
+  /* Put it in the sheets' table. */
+  hash_insert (style_sheets, sheet);
 }
 
 /*
@@ -766,27 +827,31 @@ load_style_sheet (const char * pseudo_key)
   /* Is the key a real key, or a path? */
   if (strsuffix (pseudo_key, SSH_SUFFIX))
     {
-      /* This style sheet has to have its path as key
-       * This is used to ease to allow specifying a style sheet by
-       * its real path, instead of allowing key based file search.
-       * It is also used to to check that an ssh
-       * file is OK, from a2ps-mode in emacs.*/
+      /* This style sheet has to have its path as key.  This is used
+	 to ease to allow specifying a style sheet by its real path,
+	 instead of allowing key based file search.  It is also used
+	 to to check that an ssh file is OK, from a2ps-mode in
+	 emacs.*/
       res = parse_style_sheet (pseudo_key);
-    } else {
+    }
+  else
+    {
       /* This is the regular process: find the file */
       char * path = pw_find_file (job->common.path, pseudo_key, SSH_SUFFIX);
-      if (!path) {
-	error (0, 0, _("cannot find file `%s%s'"), pseudo_key, SSH_SUFFIX);
-	return NULL;
-      }
+      if (!path)
+	{
+	  char *file;
+	  astrcat2 (file, pseudo_key, SSH_SUFFIX);
+	  error (0, 0, _("cannot find file `%s'"), quotearg (file));
+	  return NULL;
+	}
       res = parse_style_sheet (path);
       FREE (path);
     }
-  
+
   /* Now, finalize it, and store it in the hash tab */
   res->key = strdup (pseudo_key);
   style_sheet_finalize (res);
-  hash_insert (style_sheets, res);
 
   return res;
 }
@@ -794,15 +859,14 @@ load_style_sheet (const char * pseudo_key)
 /************************************************************************/
 /*		Reporting information about the style sheets		*/
 /************************************************************************/
-/*
- * Compare the names of two style sheets
- * We use the collate comparison, so that `a' is not after `Z'
- */
+
+/* Compare the names of two style sheets */
+
 static int
 sheet_name_cmp (const char * key1, const char * key2)
 {
-  return strcoll ((const char *) get_style_sheet (key1) -> name,
-		  (const char *) get_style_sheet (key2) -> name);
+  return strcasecmp ((const char *) get_style_sheet (key1) -> name,
+		     (const char *) get_style_sheet (key2) -> name);
 }
 
 /*
@@ -811,21 +875,22 @@ sheet_name_cmp (const char * key1, const char * key2)
 void
 style_sheet_self_print (struct style_sheet * sheet, FILE * stream)
 {
-  fprintf (stream, "Style sheet \"%s\" (%s), version ", 
+  fprintf (stream, "Style sheet \"%s\" (%s), version ",
 	   sheet->name, sheet->key);
   version_self_print (sheet->version, stream);
   putc ('\n', stream);
   if (!IS_EMPTY (sheet->author))
     fprintf (stream, "Written by %s\n", sheet->author);
-  if (sheet->ancestors->len > 0) {
-    size_t i;
-    fprintf (stream, "It inherits from: ");
-    for (i = 0 ; i < sheet->ancestors->len ; i++)
-      fprintf (stream, "%s ", (char *) sheet->ancestors->content [i]);
-    fputs ("\n", stream);
-  }
+  if (sheet->ancestors->len > 0)
+    {
+      size_t i;
+      fprintf (stream, "It inherits from: ");
+      for (i = 0 ; i < sheet->ancestors->len ; i++)
+	fprintf (stream, "%s ", (char *) sheet->ancestors->content [i]);
+      fputs ("\n", stream);
+    }
 
-  fprintf (stream, "Case %ssensitive\n", 
+  fprintf (stream, "Case %ssensitive\n",
 	   sheet->sensitiveness == case_insensitive ? "in" : "");
 
   /* Alphabets */
@@ -855,22 +920,25 @@ style_sheet_print_signature (FILE * stream, struct style_sheet * sheet)
 {
   int i, title_bar_len;
 
-  if (!version_is_null (sheet->version)) {
-    title_bar_len = (strlen (" (.ssh version )")
-		     + ustrlen (sheet->name)
-		     + strlen (sheet->key)
-		     + version_length (sheet->version));
-    fprintf (stream, "%s (%s.ssh version ", 
-	     sheet->name, sheet->key);
-    version_self_print (sheet->version, stream);
-    fputs (")\n", stream);
-  } else {
-    title_bar_len = (strlen (" (.ssh)")
-		     + ustrlen (sheet->name)
-		     + strlen (sheet->key));
-    fprintf (stream, "%s (%s.ssh)\n", 
-	     sheet->name, sheet->key);
-  }
+  if (!version_is_null (sheet->version))
+    {
+      title_bar_len = (strlen (" (.ssh version )")
+		       + ustrlen (sheet->name)
+		       + strlen (sheet->key)
+		       + version_length (sheet->version));
+      fprintf (stream, "%s (%s.ssh version ",
+	       sheet->name, sheet->key);
+      version_self_print (sheet->version, stream);
+      fputs (")\n", stream);
+    }
+  else
+    {
+      title_bar_len = (strlen (" (.ssh)")
+		       + ustrlen (sheet->name)
+		       + strlen (sheet->key));
+      fprintf (stream, "%s (%s.ssh)\n",
+	       sheet->name, sheet->key);
+    }
   for (i = 0 ; i < title_bar_len ; i++)
     putc ('-', stream);
   putc ('\n', stream);
@@ -887,20 +955,9 @@ style_sheet_print_signature (FILE * stream, struct style_sheet * sheet)
 void
 list_style_sheets_short (FILE * stream)
 {
-  struct darray * entries;
-  size_t i;
-
-  entries = pw_glob_on_suffix (job->common.path, SSH_SUFFIX);
-
-  fputs (_("Known style sheets (languages)"), stream);
-    
-  for (i = 0 ; i < entries->len ; i++) {
-    if (!(i % 7))
-      fputs ("\n  ", stream);
-    fprintf (stream, "%-10s", (char *) entries->content[i]);
-  }
+  fputs (_("Known Style Sheets"), stream);
   putc ('\n', stream);
-  da_free (entries, (da_map_func_t) free);
+  pw_lister_on_suffix (stream, job->common.path, SSH_SUFFIX);
 }
 
 /*
@@ -919,14 +976,14 @@ list_style_sheets_long (FILE * stream)
   entries->cmp = (da_cmp_func_t) sheet_name_cmp;
   da_qsort (entries);
 
-  fputs (_("Known style sheets (languages)"), stream);
-  putc ('\n', stream);
-  
-  for (i = 0 ; i < entries->len ; i++) {
-    sheet = get_style_sheet ((char *) entries->content[i]);
-    style_sheet_print_signature (stream, sheet);
-  }
+  title (stream, '=', true, _("Known Style Sheets"));
   putc ('\n', stream);
+
+  for (i = 0 ; i < entries->len ; i++)
+    {
+      sheet = get_style_sheet ((char *) entries->content[i]);
+      style_sheet_print_signature (stream, sheet);
+    }
   da_free (entries, (da_map_func_t) free);
 }
 
@@ -941,15 +998,15 @@ static void
 style_sheet_html_print_signature (FILE * stream, struct style_sheet * sheet)
 {
   if (version_is_null (sheet->version))
-    fprintf (stream, 
+    fprintf (stream,
 	     "<li><p><strong><a href=\"%s.ssh\" name=\"%s\">%s</a></strong>.</p>\n",
 	     sheet->key, sheet->key, sheet->name);
   else {
-    fprintf (stream, 
+    fprintf (stream,
 	     "<li><p><strong><a href=\"%s.ssh\" name=\"%s\">%s</a></strong> version ",
 	     sheet->key, sheet->key, sheet->name);
     version_self_print (sheet->version, stream);
-    fputs (".</p>\n", stream);      
+    fputs (".</p>\n", stream);
   }
 
   if (!IS_EMPTY(sheet->author)
@@ -972,7 +1029,7 @@ style_sheet_html_print_signature (FILE * stream, struct style_sheet * sheet)
       }
       fputs ("</p>", stream);
     }
-  
+
   documentation_print_html (sheet->documentation, "<p>%s</p>\n", stream);
   fputs ("</li>\n\n", stream);
 }
@@ -987,86 +1044,91 @@ list_style_sheets_html (FILE * stream)
 {
   struct style_sheet * sheet;
   struct darray * entries;
-  size_t i; 
+  size_t i;
   int version_index[4], requirement [4], rows;
 
   entries = pw_glob_on_suffix (job->common.path, SSH_SUFFIX);
 
-  /* Sort them by key first */
-/*  da_qsort (entries);*/
-
   style_sheets_versions (entries, version_index, requirement);
 
+  /* Sort them by name, not key. */
+  entries->cmp = (da_cmp_func_t) sheet_name_cmp;
+  da_qsort (entries);
+
   /*
    * The HTML header.  It should definitely be taken out of here.
    */
-  fputs ("\
-<html>\n\
+  fputs (expand_user_string (job, CURRENT_FILE (job), "sheets.html generation",
+"<html>\n\
+#{html.begin.hook}\n\
 <head>\n\
-   <title>a2ps' languages support</title>\n\
+   <title>#{html.title:-GNU a2ps language support}</title>\n\
 </head>\n\
-<body background=\"http://www.inf.enst.fr/~demaille/images/chalk.jpg\" bgcolor=\"#FFFFFF\">\n\
 \n\
-<h1 align=\"CENTER\">\n\
-a2ps' languages support\n\
+<h1 align=\\\"CENTER\\\">\n\
+#{html.title.1:-GNU a2ps language support}\n\
 </h1>\n\
-<img src=\"http://www.inf.enst.fr/~demaille/images/barre.bleu.bleu.gif\" ALT=\"\" height=4 width=100% align=center>\n\
+#{html.hline:-<hline>}\n\
 <h2>Current <a href=\"sheets.mp\">sheets.map</a></h2>\n\
-<p>It is the list of rules to decide what style sheet to use.</p>\n", stream);
+<p>It is the list of rules to decide what style sheet to use.</p>\n"), stream);
 
   /*
    * The short list, presented as a table
    */
-  /* How many rows?  Let's make three columns */
-  rows = entries->len / 4 + 1;
-
-  fputs ("\
-<img src=\"http://www.inf.enst.fr/~demaille/images/barre.bleu.bleu.gif\" ALT=\"\" height=4 width=100% align=center>\n\
+  fputs (expand_user_string (job, CURRENT_FILE (job), "sheets.html generation",
+"\
+#{html.hline:-<hline>}\n\
 <h2>Sumary of current style sheets</h2>\n\
 <p>There is a <a href=\"sheets.tar.gz\">package</a> which contains \
 them all.  Its index (sum of the style sheets version number) is \
-<strong>", stream);
+<strong>"), stream);
   version_self_print (version_index, stream);
   fputs ("</strong>, and the highest requirement is a2ps version ", stream);
   version_self_print (requirement, stream);
-  fputs (".  Links point to their detailed exposure.</p>\n", stream); 
+  fputs (".  Links point to their detailed exposure.</p>\n", stream);
+
+  /* How many rows?  Let's make two columns */
+  rows = entries->len / 3 + 1;
+
   fputs ("<table border=\"0\" cellpadding=\"0\"><tr>\n", stream);
-  for (i = 0 ; i < entries->len ; /* Nothing */) {
-    fputs ("<td valign=\"top\"><ul>\n", stream);
-    do {
-      sheet = get_style_sheet ((char *) entries->content[i]);
-      fprintf (stream, "<li><a href=\"#%s\"><code>%s</code> ",
-	       sheet->key, sheet->key);
-      version_self_print (sheet->version, stream);
-      fputs ("</a></code></li>\n", stream);
-    } while ((++i % rows) && (i < entries->len));
-    fputs ("</ul></td>\n", stream);
-  }
+  for (i = 0 ; i < entries->len ; /* Nothing */)
+    {
+      fputs ("<td valign=\"top\"><ul>\n", stream);
+      do
+	{
+	  sheet = get_style_sheet ((char *) entries->content[i]);
+	  fprintf (stream, "<li><a href=\"#%s\">%s <code>",
+		   sheet->key, sheet->name);
+	  version_self_print (sheet->version, stream);
+	  fputs ("</a></code></li>\n", stream);
+	}
+      while ((++i % rows) && (i < entries->len));
+      fputs ("</ul></td>\n", stream);
+    }
   fputs ("</tr></table>\n", stream);
 
   /*
    * The real detail list
    */
-  fputs ("\
-<img src=\"http://www.inf.enst.fr/~demaille/images/barre.bleu.bleu.gif\" ALT=\"\" height=4 width=100% align=center>\n\
-<h2>Detailed list of current style sheets</h2>\n", stream);
+  fputs (expand_user_string (job, CURRENT_FILE (job), "sheets.html generation",
+"\
+#{html.hline:-<hline>}\n\
+<h2>Detailed list of current style sheets</h2>\n"), stream);
   fputs ("Links point to their files.</p>\n", stream);
 
   fputs ("<ul>\n", stream);
-  /* Sort them by name now */
-  entries->cmp = (da_cmp_func_t) sheet_name_cmp;
-  da_qsort (entries);
-
-  for (i = 0 ; i < entries->len ; i++) {
-    sheet = get_style_sheet ((char *) entries->content[i]);
-    style_sheet_html_print_signature (stream, sheet);
-  }
+  for (i = 0 ; i < entries->len ; i++)
+    {
+      sheet = get_style_sheet ((char *) entries->content[i]);
+      style_sheet_html_print_signature (stream, sheet);
+    }
   fputs ("</ul>\n", stream);
 
-  fputs ("\
-<!--#include virtual=\"/~demaille/images/bottom.html\"-->\n\
+  fputs (expand_user_string (job, CURRENT_FILE (job), "sheets.html generation",
+"\
+#{html.end.hook}\n\
 </body>\n\
-</html>\n", stream);
+</html>\n"), stream);
 
   da_free (entries, (da_map_func_t) free);
 }
@@ -1082,36 +1144,39 @@ static void
 style_sheet_texinfo_print_signature (FILE * stream,
 				     struct style_sheet * sheet)
 {
-  fprintf (stream, 
+  fprintf (stream,
 	   "@deftp {Style Sheet} {%s} (@file{%s.ssh})\n",
 	   sheet->name, sheet->key);
 
   /* Report details (version numbers, ancestors...) only if
    * verbose mode for sheets */
   if (!IS_EMPTY(sheet->author)
-      || (message_test (msg_sheet) 
+      || (msg_test (msg_sheet)
 	  && (!version_is_null (sheet->requirement)
-	      || (sheet->ancestors->len > 0)))) {
-    authors_print_texinfo (sheet->author, stream, "Written by ");
-    if (message_test (msg_sheet))
-      {
-	if (!version_is_null (sheet->requirement)) {
-	  fputs ("It requires @code{a2ps} version ", stream);
-	  version_self_print (sheet->requirement, stream);
-	  fputs (".  \n", stream);
+	      || (sheet->ancestors->len > 0))))
+    {
+      authors_print_texinfo (sheet->author, stream, "Written by ");
+      if (msg_test (msg_sheet))
+	{
+	  if (!version_is_null (sheet->requirement))
+	    {
+	      fputs ("It requires @code{a2ps} version ", stream);
+	      version_self_print (sheet->requirement, stream);
+	      fputs (".  \n", stream);
+	    }
+	  if (sheet->ancestors->len > 0)
+	    {
+	      size_t i;
+	      fputs ("It inherits from: ", stream);
+	      for (i = 0 ; i < sheet->ancestors->len ; i++)
+		fprintf (stream, "%s@file{%s.ssh}",
+			 i ? ", " : "",
+			 (char *) sheet->ancestors->content [i]);
+	      fputs (".  \n", stream);
+	    }
 	}
-	if (sheet->ancestors->len > 0) {
-	  size_t i;
-	  fputs ("It inherits from: ", stream);
-	  for (i = 0 ; i < sheet->ancestors->len ; i++)
-	    fprintf (stream, "%s@file{%s.ssh}",
-		     i ? ", " : "",
-		     (char *) sheet->ancestors->content [i]);
-	  fputs (".  \n", stream);
-	}
-      }
-  }
-  
+    }
+
   documentation_print_texinfo (sheet->documentation, "%s\n", stream);
 
   fputs ("@end deftp\n\n", stream);
@@ -1133,12 +1198,12 @@ list_style_sheets_texinfo (FILE * stream)
   /* Sort them by name (not key) */
   entries->cmp = (da_cmp_func_t) sheet_name_cmp;
   da_qsort (entries);
-  
+
   style_sheets_versions (entries, version_index, requirement);
 
   /* Report details (version numbers etc.) only if verbose
    * mode is set for sheets */
-  if (message_test (msg_sheet))
+  if (msg_test (msg_sheet))
     {
       fputs ("\
 The current index (sum of all the style sheets version number) is ", stream);
@@ -1146,14 +1211,14 @@ The current index (sum of all the style sheets version number) is ", stream);
       fputs (" and the highest requirement is a2ps version ", stream);
       version_self_print (requirement, stream);
       fputs (".\n", stream);
-      
+
     }
-  
+
   for (i = 0 ; i < entries->len ; i++) {
     sheet = get_style_sheet ((char *) entries->content[i]);
     style_sheet_texinfo_print_signature (stream, sheet);
   }
-  
+
   da_free (entries, (da_map_func_t) free);
 }
 
@@ -1172,7 +1237,7 @@ check_rules_lower_case (const char * name, struct darray * rules)
 
   for (i = 0 ; i < rules->len ; i++)
     if (!is_strlower (((struct rule *) rules->content[i])->word)) {
-      error (0, 0, 
+      error (0, 0,
 	     "%s.ssh:`%s' uses upper case characters",
 	     name, ((struct rule *) rules->content[i])->word);
       res = FALSE;
@@ -1199,12 +1264,12 @@ check_sequence_lower_case (const char * name, struct sequence * sequence)
   boolean res = true;
 
   if (!is_strlower (sequence->open->word)) {
-    error (0, 0, 
+    error (0, 0,
 	   "%s.ssh:`%s' uses upper case characters",
 	   name, sequence->open->word);
     res = FALSE;
   }
-  
+
   res &= check_words_lower_case (name, sequence->exceptions);
   res &= check_words_lower_case (name, sequence->close);
 
@@ -1232,7 +1297,7 @@ static boolean
 check_sensitivity (struct style_sheet * sheet)
 {
   boolean res = true;
-  
+
   if (sheet->sensitiveness == case_sensitive)
     return TRUE;
 
@@ -1254,7 +1319,7 @@ check_rules_doubles (const char * name, struct darray * rules)
 {
   size_t i;
   boolean res = true;
-  
+
   for (i = 1 ; i < rules->len ; i++)
     if (!ustrcmp (((struct rule *) rules->content [i - 1])->word,
 		  ((struct rule *) rules->content [i])->word))
@@ -1284,7 +1349,7 @@ check_doubles (struct style_sheet * sheet)
 
   res &= check_words_doubles (sheet->key, sheet->keywords);
   res &= check_words_doubles (sheet->key, sheet->operators);
-  
+
   return res;
 }
 
@@ -1327,10 +1392,10 @@ sheet_hash_cmp (void const *x, void const *y)
 /*
  * Create the hash table
  */
-struct hash_table *
+struct hash_table_s *
 new_style_sheets (void)
 {
-  struct hash_table * res; 
+  struct hash_table_s * res;
   res = ALLOC (hash_table, 1);
   hash_init (res, 8,
              sheet_hash_1, sheet_hash_2, sheet_hash_cmp);
@@ -1351,19 +1416,19 @@ require_style_sheet (const char * key)
 
   token.key = (char *) key;
   item = (struct style_sheet *) hash_find_item (style_sheets, &token);
-  
-  if (item == NULL) {
-    /* The style has never been seen before, then load it
-     * through a correct call to yyparse, which in turn
-     * in charge to put the newly read style in the hash
-     * table.  Hence, look for it again.  An error at this
-     * point would be highly strange
-     */
-    item = load_style_sheet (key);
 
-    if (item == NULL)
-      error (0, 0, _("cannot find style sheet `%s': using plain style"), key);
-  }
+  if (item == NULL)
+    {
+      /* The style has never been seen before, then load it through a
+	 correct call to yyparse, which in turn is in charge of
+	 putting the newly read style in the hash table.  Hence, look
+	 for it again.  An error at this point is really an error. */
+      item = load_style_sheet (key);
+
+      if (item == NULL)
+	error (0, 0,
+	       _("cannot find style sheet `%s': using plain style"), key);
+    }
 
   return;
 }
@@ -1382,8 +1447,19 @@ get_style_sheet (const char * key)
   if (strequ (key, "plain"))
     return NULL;
 
-  /* Make sure the hash table contains the corresponding style sheet */
-  require_style_sheet (key);
+  /* There is some specials magic to build on the fly mixed style
+     sheets, for instance `-Email,sh' build a new style sheet which is
+     only defined by:
+     ancestors are
+        mail, sh
+     end ancestors.
+     In this case the key become `mail,sh'.
+  */
+  if (strpbrk (key, MIXED_SHEET_SEP))
+    key = style_sheet_mixed_new (key);
+  else
+    /* Make sure the hash table contains the corresponding style sheet */
+    require_style_sheet (key);
 
   token.key = (char *) key;
   item = (struct style_sheet *) hash_find_item (style_sheets, &token);
@@ -1404,12 +1480,12 @@ get_style_sheet (const char * key)
  * sheets which key is in KEYS
  */
 static inline void
-style_sheets_versions (struct darray * keys, 
+style_sheets_versions (struct darray * keys,
 		       int sum [4], int requirement [4])
 {
   size_t i;
   struct style_sheet * sheet;
-  
+
   version_set_to_null (sum);
   version_set_to_null (requirement);
   for (i = 0 ; i < keys->len ; i++)
@@ -1419,5 +1495,5 @@ style_sheets_versions (struct darray * keys,
       if (version_cmp (sheet->requirement, requirement) > 0)
 	version_cpy (requirement, sheet->requirement);
     }
-  
+
 }
